@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2016 Intel Corp, Inc. All rights reserved.
  * Copyright (c) 2018 Amazon.com, Inc. or its affiliates. All rights reserved.
+ * Copyright (c) 2022 DataDirect Networks, Inc. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -65,11 +66,6 @@ static inline int ofi_alloc_hugepage_buf(void **memptr, size_t size)
 	return -FI_ENOSYS;
 }
 
-static inline int ofi_free_hugepage_buf(void *memptr, size_t size)
-{
-	return -FI_ENOSYS;
-}
-
 static inline size_t ofi_ifaddr_get_speed(struct ifaddrs *ifa)
 {
 	return 0;
@@ -151,6 +147,38 @@ ofi_recvmsg_tcp(SOCKET fd, struct msghdr *msg, int flags)
 	return recvmsg(fd, msg, flags);
 }
 
+static inline ssize_t
+ofi_sendv_socket(SOCKET fd, const struct iovec *iov, size_t cnt, int flags)
+{
+       struct msghdr msg;
+
+       msg.msg_control = NULL;
+       msg.msg_controllen = 0;
+       msg.msg_flags = 0;
+       msg.msg_name = NULL;
+       msg.msg_namelen = 0;
+       msg.msg_iov = (struct iovec *) iov;
+       msg.msg_iovlen = cnt;
+
+       return ofi_sendmsg_tcp(fd, &msg, flags);
+}
+
+static inline ssize_t
+ofi_recvv_socket(SOCKET fd, const struct iovec *iov, size_t cnt, int flags)
+{
+       struct msghdr msg;
+
+       msg.msg_control = NULL;
+       msg.msg_controllen = 0;
+       msg.msg_flags = 0;
+       msg.msg_name = NULL;
+       msg.msg_namelen = 0;
+       msg.msg_iov = (struct iovec *) iov;
+       msg.msg_iovlen = cnt;
+
+       return ofi_recvmsg_tcp(fd, &msg, flags);
+}
+
 #endif /* _FREEBSD_OSD_H_ */
 
 
