@@ -39,7 +39,7 @@
 static void
 rxm_ep_format_atomic_pkt_hdr(struct rxm_conn *rxm_conn,
 		 struct rxm_tx_buf *tx_buf, size_t data_len,
-		 uint32_t pkt_op, enum fi_datatype datatype,
+		 uint8_t pkt_op, enum fi_datatype datatype,
 		 uint8_t atomic_op, uint64_t flags, uint64_t data,
 		 const struct fi_rma_ioc *rma_ioc, size_t rma_ioc_count)
 {
@@ -52,18 +52,18 @@ rxm_ep_format_atomic_pkt_hdr(struct rxm_conn *rxm_conn,
 	tx_buf->pkt.hdr.op = pkt_op;
 	tx_buf->pkt.hdr.atomic.datatype = datatype;
 	tx_buf->pkt.hdr.atomic.op = atomic_op;
-	tx_buf->pkt.hdr.atomic.ioc_count = rma_ioc_count;
+	tx_buf->pkt.hdr.atomic.ioc_count = (uint8_t) rma_ioc_count;
 	if (rma_ioc_count)
 		memcpy(atomic_hdr->rma_ioc, rma_ioc,
 		       rma_ioc_count * sizeof(struct fi_rma_ioc));
 	tx_buf->flags = flags;
 }
 
-static inline int
+static inline ssize_t
 rxm_ep_send_atomic_req(struct rxm_ep *rxm_ep, struct rxm_conn *rxm_conn,
 		       struct rxm_tx_buf *tx_buf, uint64_t len)
 {
-	int ret;
+	ssize_t ret;
 
 	/* Atomic request TX completion processing is performed when the
 	 * software generated atomic response message is received. */
@@ -92,16 +92,16 @@ rxm_ep_atomic_common(struct rxm_ep *rxm_ep, struct rxm_conn *rxm_conn,
 		const struct fi_msg_atomic *msg, const struct fi_ioc *comparev,
 		void **compare_desc, size_t compare_iov_count,
 		struct fi_ioc *resultv, void **result_desc,
-		size_t result_iov_count, uint32_t op, uint64_t flags)
+		size_t result_iov_count, uint8_t op, uint64_t flags)
 {
 	struct rxm_tx_buf *tx_buf;
 	struct rxm_atomic_hdr *atomic_hdr;
 	struct iovec buf_iov[RXM_IOV_LIMIT];
 	struct iovec cmp_iov[RXM_IOV_LIMIT];
 	enum fi_hmem_iface buf_iface = FI_HMEM_SYSTEM;
-	enum fi_hmem_iface cmp_iface;
+	enum fi_hmem_iface cmp_iface = FI_HMEM_SYSTEM;
 	uint64_t buf_device = 0;
-	uint64_t cmp_device;
+	uint64_t cmp_device = 0;
 	size_t datatype_sz = ofi_datatype_size(msg->datatype);
 	size_t buf_len = 0;
 	size_t cmp_len = 0;
@@ -164,16 +164,16 @@ rxm_ep_atomic_common(struct rxm_ep *rxm_ep, struct rxm_conn *rxm_conn,
 
 	ret = ofi_copy_from_hmem_iov(atomic_hdr->data, buf_len, buf_iface,
 				     buf_device, buf_iov, msg->iov_count, 0);
-	assert(ret == buf_len);
+	assert((size_t) ret == buf_len);
 
 	if (cmp_len) {
 		ret = ofi_copy_from_hmem_iov(atomic_hdr->data + buf_len,
 					     cmp_len, cmp_iface, cmp_device,
 					     cmp_iov, compare_iov_count, 0);
-		assert(ret == cmp_len);
+		assert((size_t) ret == cmp_len);
 	}
 
-	tx_buf->atomic_result.count = result_iov_count;
+	tx_buf->atomic_result.count = (uint8_t) result_iov_count;
 	if (resultv) {
 		ofi_ioc_to_iov(resultv, tx_buf->atomic_result.iov,
 			       result_iov_count, datatype_sz);
@@ -186,7 +186,7 @@ rxm_ep_atomic_common(struct rxm_ep *rxm_ep, struct rxm_conn *rxm_conn,
 
 	ret = rxm_ep_send_atomic_req(rxm_ep, rxm_conn, tx_buf, tot_len);
 	if (ret)
-		rxm_free_rx_buf(rxm_ep, tx_buf);
+		rxm_free_tx_buf(rxm_ep, tx_buf);
 
 	return ret;
 }
@@ -509,15 +509,24 @@ int rxm_ep_query_atomic(struct fid_domain *domain, enum fi_datatype datatype,
 		return -FI_EINVAL;
 	}
 
+	if ((datatype == FI_INT128) || (datatype == FI_UINT128)) {
+		FI_WARN(&rxm_prov, FI_LOG_EP_CTRL,
+			"128-bit integers not supported\n");
+		return -FI_EOPNOTSUPP;
+	}
+
 	ret = ofi_atomic_valid(&rxm_prov, datatype, op, flags);
 	if (ret || !attr)
 		return ret;
 
 	tot_size = flags & FI_COMPARE_ATOMIC ?
-		rxm_domain->max_atomic_size / 2 : rxm_domain->max_atomic_size;
+		   rxm_domain->max_atomic_size / 2 :
+		   rxm_domain->max_atomic_size;
 	attr->size = ofi_datatype_size(datatype);
-	attr->count = tot_size / attr->size;
+	if (!attr->size)
+		return -FI_EOPNOTSUPP;
 
+	attr->count = tot_size / attr->size;
 	if (attr->count == 0)
 		return -FI_EOPNOTSUPP;
 
