@@ -35,21 +35,30 @@
 #endif
 
 #include "ofi_hmem.h"
+#include "ofi_mem.h"
 #include "ofi.h"
 
-#if HAVE_LIBZE
+#if HAVE_ZE
 
+#include <stdio.h>
 #include <dirent.h>
 #include <level_zero/ze_api.h>
 
+static ze_driver_handle_t driver;
 static ze_context_handle_t context;
 static ze_device_handle_t devices[ZE_MAX_DEVICES];
 static ze_command_queue_handle_t cmd_queue[ZE_MAX_DEVICES];
 static int num_devices = 0;
+static int ordinals[ZE_MAX_DEVICES];
+static int indices[ZE_MAX_DEVICES];
 static int dev_fds[ZE_MAX_DEVICES];
+static ze_device_uuid_t dev_uuids[ZE_MAX_DEVICES];
 static bool p2p_enabled = false;
+static bool host_reg_enabled = true;
+static struct ofi_bufpool *cl_pool[ZE_MAX_DEVICES];
+ofi_spin_t cl_lock;
 
-static const ze_command_queue_desc_t cq_desc = {
+static ze_command_queue_desc_t cq_desc = {
 	.stype		= ZE_STRUCTURE_TYPE_COMMAND_QUEUE_DESC,
 	.pNext		= NULL,
 	.ordinal	= 0,
@@ -59,7 +68,7 @@ static const ze_command_queue_desc_t cq_desc = {
 	.priority	= ZE_COMMAND_QUEUE_PRIORITY_NORMAL,
 };
 
-static const ze_command_list_desc_t cl_desc = {
+static ze_command_list_desc_t cl_desc = {
 	.stype				= ZE_STRUCTURE_TYPE_COMMAND_LIST_DESC,
 	.pNext				= NULL,
 	.commandQueueGroupOrdinal	= 0,
@@ -73,6 +82,9 @@ struct libze_ops {
 	ze_result_t (*zeDeviceGet)(ze_driver_handle_t hDriver,
 				   uint32_t *pCount,
 				   ze_device_handle_t *phDevices);
+	ze_result_t (*zeDeviceGetCommandQueueGroupProperties)(ze_device_handle_t hDevice,
+			uint32_t *pCount,
+			ze_command_queue_group_properties_t *pCommandQueueGroupProperties);
 	ze_result_t (*zeDeviceCanAccessPeer)(ze_device_handle_t hDevice,
 					     ze_device_handle_t hPeerDevice,
 					     ze_bool_t *value);
@@ -96,6 +108,7 @@ struct libze_ops {
 					   ze_command_list_handle_t *phCommandList);
 	ze_result_t (*zeCommandListDestroy)(ze_command_list_handle_t hCommandList);
 	ze_result_t (*zeCommandListClose)(ze_command_list_handle_t hCommandList);
+	ze_result_t (*zeCommandListReset)(ze_command_list_handle_t hCommandList);
 	ze_result_t (*zeCommandListAppendMemoryCopy)(
 				ze_command_list_handle_t hCommandList,
 				void *dstptr, const void *srcptr, size_t size,
@@ -119,9 +132,30 @@ struct libze_ops {
 					  void **pptr);
 	ze_result_t (*zeMemCloseIpcHandle)(ze_context_handle_t hContext,
 					   const void *ptr);
+	ze_result_t (*zeDeviceGetProperties)(ze_device_handle_t hDevice,
+					     ze_device_properties_t *pDeviceProperties);
+	ze_result_t (*zeDriverGetExtensionFunctionAddress)(
+				ze_driver_handle_t hDriver, const char *name,
+				void **ppFunctionAddress);
+	ze_result_t (*zexDriverImportExternalPointer)(ze_driver_handle_t hDriver,
+						      void *ptr, size_t len);
+	ze_result_t (*zexDriverReleaseImportedPointer)(ze_driver_handle_t hDriver,
+						       void *ptr);
 };
 
-#ifdef ENABLE_ZE_DLOPEN
+#define DEVICE_MASK  ((1 << 16) - 1)
+
+static inline int ze_get_device_idx(uint64_t device)
+{
+	return device & DEVICE_MASK;
+}
+
+static inline int ze_get_driver_idx(uint64_t device)
+{
+	return device >> 16;
+}
+
+#if ENABLE_ZE_DLOPEN
 
 #include <dlfcn.h>
 
@@ -134,6 +168,7 @@ static struct libze_ops libze_ops = {
 	.zeInit = zeInit,
 	.zeDriverGet = zeDriverGet,
 	.zeDeviceGet = zeDeviceGet,
+	.zeDeviceGetCommandQueueGroupProperties = zeDeviceGetCommandQueueGroupProperties,
 	.zeDeviceCanAccessPeer = zeDeviceCanAccessPeer,
 	.zeContextCreate = zeContextCreate,
 	.zeContextDestroy = zeContextDestroy,
@@ -143,12 +178,15 @@ static struct libze_ops libze_ops = {
 	.zeCommandListCreate = zeCommandListCreate,
 	.zeCommandListDestroy = zeCommandListDestroy,
 	.zeCommandListClose = zeCommandListClose,
+	.zeCommandListReset = zeCommandListReset,
 	.zeCommandListAppendMemoryCopy = zeCommandListAppendMemoryCopy,
 	.zeMemGetAllocProperties = zeMemGetAllocProperties,
 	.zeMemGetAddressRange = zeMemGetAddressRange,
 	.zeMemGetIpcHandle = zeMemGetIpcHandle,
 	.zeMemOpenIpcHandle = zeMemOpenIpcHandle,
 	.zeMemCloseIpcHandle = zeMemCloseIpcHandle,
+	.zeDeviceGetProperties = zeDeviceGetProperties,
+	.zeDriverGetExtensionFunctionAddress = zeDriverGetExtensionFunctionAddress,
 };
 
 #endif /* ENABLE_ZE_DLOPEN */
@@ -169,6 +207,14 @@ ze_result_t ofi_zeDeviceGet(ze_driver_handle_t hDriver, uint32_t *pCount,
 	return (*libze_ops.zeDeviceGet)(hDriver, pCount, phDevices);
 }
 
+ze_result_t ofi_zeDeviceGetCommandQueueGroupProperties(ze_device_handle_t hDevice,
+	       uint32_t *pCount,
+	       ze_command_queue_group_properties_t *pCommandQueueGroupProperties)
+{
+	return (*libze_ops.zeDeviceGetCommandQueueGroupProperties)(hDevice,
+					pCount, pCommandQueueGroupProperties);
+}
+
 ze_result_t ofi_zeDeviceCanAccessPeer(ze_device_handle_t hDevice,
 				      ze_device_handle_t hPeerDevice,
 				      ze_bool_t *value)
@@ -232,6 +278,11 @@ ze_result_t ofi_zeCommandListClose(ze_command_list_handle_t hCommandList)
 	return (*libze_ops.zeCommandListClose)(hCommandList);
 }
 
+ze_result_t ofi_zeCommandListReset(ze_command_list_handle_t hCommandList)
+{
+	return (*libze_ops.zeCommandListReset)(hCommandList);
+}
+
 ze_result_t ofi_zeCommandListAppendMemoryCopy(
 				ze_command_list_handle_t hCommandList,
 				void *dstptr, const void *srcptr, size_t size,
@@ -284,18 +335,51 @@ ze_result_t ofi_zeMemCloseIpcHandle(ze_context_handle_t hContext,
 	return (*libze_ops.zeMemCloseIpcHandle)(hContext, ptr);
 }
 
+ze_result_t ofi_zeDeviceGetProperties(ze_device_handle_t hDevice,
+				      ze_device_properties_t *pDeviceProperties)
+{
+	return (*libze_ops.zeDeviceGetProperties)(hDevice, pDeviceProperties);
+}
+
+ze_result_t ofi_zeDriverGetExtensionFunctionAddress(ze_driver_handle_t hDriver,
+						    const char *name,
+						    void **ppFuncAddress)
+{
+	return (*libze_ops.zeDriverGetExtensionFunctionAddress)(hDriver, name,
+							        ppFuncAddress);
+}
+
+ze_result_t ofi_zexDriverImportExternalPointer(ze_driver_handle_t hDriver,
+					       void *ptr, size_t len)
+{
+	return (*libze_ops.zexDriverImportExternalPointer)(hDriver, ptr, len);
+}
+
+ze_result_t ofi_zexDriverReleaseImportedPointer(ze_driver_handle_t hDriver,
+						void *ptr)
+{
+	return (*libze_ops.zexDriverReleaseImportedPointer)(hDriver, ptr);
+}
+
+#if HAVE_DRM || HAVE_LIBDRM
+
 #if HAVE_DRM
 #include <drm/i915_drm.h>
+#else
+#include <libdrm/i915_drm.h>
+#endif
+
 #include <sys/ioctl.h>
+#include <stdio.h>
 
 static int ze_hmem_init_fds(void)
 {
-	const char *dev_dir = "/dev/dri/by-path";
+	const char *dev_dir = "/dev/dri/by-path/";
 	const char *suffix = "-render";
 	DIR *dir;
 	struct dirent *ent = NULL;
-	char dev_name[128];
-	int i = 0;
+	char dev_name[NAME_MAX];
+	int i = 0, ret;
 
 	dir = opendir(dev_dir);
 	if (dir == NULL)
@@ -307,20 +391,23 @@ static int ze_hmem_init_fds(void)
 			continue;
 
 		memset(dev_name, 0, sizeof(dev_name));
-		strncpy(dev_name, dev_dir, sizeof(dev_name));
-		strncat(dev_name, "/",
-			sizeof(dev_name) - strlen(dev_name));
-		strncat(dev_name, ent->d_name,
-			sizeof(dev_name) - strlen(dev_name));
+		ret = snprintf(dev_name, NAME_MAX, "%s%s", dev_dir, ent->d_name);
+		if (ret < 0 || ret >= NAME_MAX)
+			goto err;
+
 		dev_fds[i] = open(dev_name, O_RDWR);
-		if (dev_fds[i] == -1) {
-			FI_WARN(&core_prov, FI_LOG_CORE,
-				"Failed open device %d\n", i);
-			return -FI_EIO;
-		}
+		if (dev_fds[i] == -1)
+			goto err;
 		i++;
 	}
+	(void) closedir(dir);
 	return FI_SUCCESS;
+
+err:
+	(void) closedir(dir);
+	FI_WARN(&core_prov, FI_LOG_CORE,
+		"Failed open device %d\n", i);
+	return -FI_EIO;
 }
 
 int ze_hmem_get_shared_handle(int dev_fd, void *dev_buf, int *ze_fd,
@@ -330,7 +417,8 @@ int ze_hmem_get_shared_handle(int dev_fd, void *dev_buf, int *ze_fd,
 	ze_ipc_mem_handle_t ze_handle;
 	int ret;
 
-	ret = ze_hmem_get_handle(dev_buf, (void **) &ze_handle);
+	assert(dev_fd != -1);
+	ret = ze_hmem_get_handle(dev_buf, 0, (void **) &ze_handle);
 	if (ret)
 		return ret;
 
@@ -367,12 +455,12 @@ int ze_hmem_open_shared_handle(int dev_fd, void **handle, int *ze_fd,
 	*ze_fd = open_fd.fd;
 	memset(&ze_handle, 0, sizeof(ze_handle));
 	memcpy(&ze_handle, &open_fd.fd, sizeof(open_fd.fd));
-	return ze_hmem_open_handle((void **) &ze_handle, device, ipc_ptr);
+	return ze_hmem_open_handle((void **) &ze_handle, 0, device, ipc_ptr);
 }
 
 bool ze_hmem_p2p_enabled(void)
 {
-	return p2p_enabled;
+	return !ofi_hmem_p2p_disabled() && p2p_enabled;
 }
 
 #else
@@ -398,11 +486,11 @@ bool ze_hmem_p2p_enabled(void)
 	return false;
 }
 
-#endif //HAVE_DRM
+#endif //HAVE_DRM || HAVE_LIBDRM
 
 static int ze_hmem_dl_init(void)
 {
-#ifdef ENABLE_ZE_DLOPEN
+#if ENABLE_ZE_DLOPEN
 	libze_handle = dlopen("libze_loader.so", RTLD_NOW);
 	if (!libze_handle) {
 		FI_WARN(&core_prov, FI_LOG_CORE,
@@ -428,6 +516,14 @@ static int ze_hmem_dl_init(void)
 		goto err_dlclose;
 	}
 
+	libze_ops.zeDeviceGetCommandQueueGroupProperties = dlsym(libze_handle,
+				"zeDeviceGetCommandQueueGroupProperties");
+	if (!libze_ops.zeDeviceGetCommandQueueGroupProperties) {
+		FI_WARN(&core_prov, FI_LOG_CORE,
+			"Failed to find zeDeviceGetCommandQueueGroupProperties\n");
+		goto err_dlclose;
+	}
+
 	libze_ops.zeDeviceCanAccessPeer = dlsym(libze_handle, "zeDeviceCanAccessPeer");
 	if (!libze_ops.zeDeviceCanAccessPeer) {
 		FI_WARN(&core_prov, FI_LOG_CORE, "Failed to find zeDeviceCanAccessPeer\n");
@@ -446,12 +542,6 @@ static int ze_hmem_dl_init(void)
 		goto err_dlclose;
 	}
 
-	libze_ops.zeContextDestroy = dlsym(libze_handle, "zeContextDestroy");
-	if (!libze_ops.zeContextDestroy) {
-		FI_WARN(&core_prov, FI_LOG_CORE, "Failed to find zeContextDestroy\n");
-		goto err_dlclose;
-	}
-
 	libze_ops.zeCommandQueueCreate = dlsym(libze_handle, "zeCommandQueueCreate");
 	if (!libze_ops.zeCommandQueueCreate) {
 		FI_WARN(&core_prov, FI_LOG_CORE, "Failed to find zeCommandQueueCreate\n");
@@ -488,6 +578,12 @@ static int ze_hmem_dl_init(void)
 		goto err_dlclose;
 	}
 
+	libze_ops.zeCommandListReset = dlsym(libze_handle, "zeCommandListReset");
+	if (!libze_ops.zeCommandListReset) {
+		FI_WARN(&core_prov, FI_LOG_CORE, "Failed to find zeCommandListReset\n");
+		goto err_dlclose;
+	}
+
 	libze_ops.zeCommandListAppendMemoryCopy = dlsym(libze_handle, "zeCommandListAppendMemoryCopy");
 	if (!libze_ops.zeCommandListAppendMemoryCopy) {
 		FI_WARN(&core_prov, FI_LOG_CORE, "Failed to find zeCommandListAppendMemoryCopy\n");
@@ -524,6 +620,20 @@ static int ze_hmem_dl_init(void)
 		goto err_dlclose;
 	}
 
+	libze_ops.zeDeviceGetProperties = dlsym(libze_handle, "zeDeviceGetProperties");
+	if (!libze_ops.zeDeviceGetProperties) {
+		FI_WARN(&core_prov, FI_LOG_CORE, "Failed to find zeDeviceGetProperties\n");
+		goto err_dlclose;
+	}
+
+	libze_ops.zeDriverGetExtensionFunctionAddress =
+		dlsym(libze_handle, "zeDriverGetExtensionFunctionAddress");
+	if (!libze_ops.zeDriverGetExtensionFunctionAddress) {
+		FI_WARN(&core_prov, FI_LOG_CORE,
+			"Failed to find zeDriverGetExtensionFunctionAddress\n");
+		goto err_dlclose;
+	}
+
 	return FI_SUCCESS;
 
 err_dlclose:
@@ -539,20 +649,129 @@ err_out:
 
 static void ze_hmem_dl_cleanup(void)
 {
-#ifdef ENABLE_ZE_DLOPEN
+#if ENABLE_ZE_DLOPEN
 	dlclose(libze_handle);
 #endif
 }
 
+static int ze_hmem_find_copy_only_engine(int device_num, int *ordinal, int *index)
+{
+	ze_result_t ze_ret;
+	uint32_t cq_grp_count = 0;
+	ze_command_queue_group_properties_t *cq_grp_props = NULL;
+	int i = 0, j = 0;
+
+	ze_ret = ofi_zeDeviceGetCommandQueueGroupProperties(devices[device_num],
+							    &cq_grp_count, NULL);
+	if (ze_ret)
+		goto out;
+
+	cq_grp_props = calloc(cq_grp_count, sizeof(*cq_grp_props));
+
+	ze_ret = ofi_zeDeviceGetCommandQueueGroupProperties(devices[device_num],
+							    &cq_grp_count,
+							    cq_grp_props);
+	if (ze_ret)
+		goto out;
+
+	i = *ordinal;
+	j = *index;
+
+	/* Use user specified engine group if it is valid */
+	if (i != -1 && i >= 0 && i < cq_grp_count) {
+		if (j < 0 || j >= cq_grp_props[i].numQueues) {
+			FI_WARN(&core_prov, FI_LOG_CORE,
+				"ZE device %d: invalid engine index %d for "
+				"group %d, use default.\n", device_num, j, i);
+			j = 0;
+		}
+		goto out;
+	}
+
+	if (i != -1)
+		FI_WARN(&core_prov, FI_LOG_CORE,
+			"ZE device %d: invalid engine group %d, use default.\n",
+			device_num, i);
+
+	/* Auto select the first copy-only engine group if possible */
+	j = 0;
+	for (i = 0; i < cq_grp_count; i++) {
+		if (cq_grp_props[i].flags &
+		    ZE_COMMAND_QUEUE_GROUP_PROPERTY_FLAG_COPY &&
+		    !(cq_grp_props[i].flags &
+		      ZE_COMMAND_QUEUE_GROUP_PROPERTY_FLAG_COMPUTE)) {
+			break;
+		}
+	}
+
+out:
+	free(cq_grp_props);
+	*ordinal = i == cq_grp_count ? 0 : i;
+	*index = j;
+	return ze_ret;
+}
+
+/*
+ * Some L0 calls may segfault when called from a "destructor" if any
+ * HMEM-capable DL provider is enabled due to premature unloading of GPU
+ * specific L0 library. Before a permanent fix is available, don't make
+ * L0 calls insider a "destructor".
+ */
+static int ze_hmem_cleanup_internal(int fini_workaround)
+{
+	int i, ret = FI_SUCCESS;
+
+	for (i = 0; i < num_devices; i++) {
+		if (!fini_workaround) {
+			if (cmd_queue[i] &&
+			    ofi_zeCommandQueueDestroy(cmd_queue[i])) {
+				FI_WARN(&core_prov, FI_LOG_CORE,
+					"Failed to destroy ZE cmd_queue\n");
+				ret = -FI_EINVAL;
+			}
+		}
+		if (cl_pool[i])
+			ofi_bufpool_destroy(cl_pool[i]);
+		if (dev_fds[i] != -1) {
+			close(dev_fds[i]);
+			dev_fds[i] = -1;
+		}
+	}
+
+	if (!fini_workaround) {
+		if (ofi_zeContextDestroy(context))
+			ret = -FI_EINVAL;
+	}
+
+	ofi_spin_destroy(&cl_lock);
+	ze_hmem_dl_cleanup();
+	return ret;
+}
+
+int ze_hmem_cleanup(void)
+{
+	return ze_hmem_cleanup_internal(1);
+}
+
 int ze_hmem_init(void)
 {
-	ze_driver_handle_t driver;
 	ze_context_desc_t context_desc = {0};
+	ze_device_properties_t dev_prop = {0};
 	ze_result_t ze_ret;
 	ze_bool_t access;
 	uint32_t count, i;
 	bool p2p = true;
 	int ret;
+	char *enginestr = NULL;
+	int ordinal = -1;
+	int index = 0;
+
+	fi_param_define(NULL, "hmem_ze_copy_engine", FI_PARAM_STRING,
+                        "Specify GPU engine used for copy operation: <group>, "
+			"<group>.<index> (default: 1st copy-only engine)");
+        fi_param_get_str(NULL, "log_level", &enginestr);
+	if (enginestr)
+		sscanf(enginestr, "%d.%d", &ordinal, &index);
 
 	ret = ze_hmem_dl_init();
 	if (ret)
@@ -567,6 +786,22 @@ int ze_hmem_init(void)
 	if (ze_ret)
 		return -FI_EIO;
 
+	ret = ofi_spin_init(&cl_lock);
+	if (ret)
+		return ret;
+
+	ze_ret = ofi_zeDriverGetExtensionFunctionAddress(
+			driver, "zexDriverImportExternalPointer",
+			(void *)&libze_ops.zexDriverImportExternalPointer);
+	if (ze_ret)
+		host_reg_enabled = false;
+
+	ze_ret = ofi_zeDriverGetExtensionFunctionAddress(
+			driver, "zexDriverReleaseImportedPointer",
+			(void *)&libze_ops.zexDriverReleaseImportedPointer);
+	if (ze_ret)
+		host_reg_enabled = false;
+
 	ze_ret = ofi_zeContextCreate(driver, &context_desc, &context);
 	if (ze_ret)
 		return -FI_EIO;
@@ -588,10 +823,20 @@ int ze_hmem_init(void)
 		goto err;
 
 	for (num_devices = 0; num_devices < count; num_devices++) {
-		ze_ret = ofi_zeCommandQueueCreate(context,
-						  devices[num_devices],
-						  &cq_desc,
-						  &cmd_queue[num_devices]);
+		cl_pool[num_devices] = NULL;
+		ze_ret = ofi_zeDeviceGetProperties(devices[num_devices],
+						   &dev_prop);
+		if (ze_ret)
+			goto err;
+
+		memcpy(&dev_uuids[num_devices], &dev_prop.uuid,
+		       sizeof(*dev_uuids));
+
+		ordinals[num_devices] = ordinal;
+		indices[num_devices] = index;
+		ze_ret = ze_hmem_find_copy_only_engine(num_devices,
+						       &ordinals[num_devices],
+						       &indices[num_devices]);
 		if (ze_ret)
 			goto err;
 
@@ -606,84 +851,152 @@ int ze_hmem_init(void)
 	return FI_SUCCESS;
 
 err:
-	(void) ze_hmem_cleanup();
+	(void) ze_hmem_cleanup_internal(0);
 	FI_WARN(&core_prov, FI_LOG_CORE,
 		"Failed to initialize ZE driver resources\n");
 
 	return -FI_EIO;
 }
 
-int ze_hmem_cleanup(void)
+static int ze_cl_alloc_fn(struct ofi_bufpool_region *region)
 {
-	int i, ret = FI_SUCCESS;
+	uint64_t dev_id = (uint64_t) region->pool->attr.context;
+	ze_result_t ze_ret;
 
-	for (i = 0; i < num_devices; i++) {
-		if (cmd_queue[i] && ofi_zeCommandQueueDestroy(cmd_queue[i])) {
-			FI_WARN(&core_prov, FI_LOG_CORE,
-				"Failed to destroy ZE cmd_queue\n");
-			ret = -FI_EINVAL;
-		}
-		if (dev_fds[i] != -1) {
-			close(dev_fds[i]);
-			dev_fds[i] = -1;
-		}
-	}
+	cl_desc.commandQueueGroupOrdinal = ordinals[dev_id];
+	ze_ret = ofi_zeCommandListCreate(context, devices[dev_id], &cl_desc,
+			(ze_command_list_handle_t *) region->mem_region);
+	return ze_ret ? -FI_EINVAL : FI_SUCCESS;
+}
 
-	if (ofi_zeContextDestroy(context))
-		ret = -FI_EINVAL;
+static void ze_cl_free_fn(struct ofi_bufpool_region *region)
+{
+	(void) ofi_zeCommandListDestroy(
+			*(ze_command_list_handle_t *) region->mem_region);
+}
 
-	ze_hmem_dl_cleanup();
-	return ret;
+static ze_result_t ze_init_res(int dev_id)
+{
+	uint64_t device = dev_id;
+	ze_result_t ze_ret;
+	struct ofi_bufpool_attr attr = {
+		.size		= sizeof(ze_command_list_handle_t),
+		.alignment	= 0,
+		.max_cnt	= 0,
+		.chunk_cnt	= 1,
+		.alloc_fn	= ze_cl_alloc_fn,
+		.free_fn	= ze_cl_free_fn,
+		.init_fn	= NULL,
+		.context	= (void *) device,
+		.flags		= 0,
+	};
+
+	cq_desc.ordinal = ordinals[dev_id];
+	cq_desc.index = indices[dev_id];
+	ze_ret = ofi_zeCommandQueueCreate(context,
+					  devices[dev_id],
+					  &cq_desc,
+					  &cmd_queue[dev_id]);
+	if (ze_ret)
+		return ze_ret;
+
+	return ofi_bufpool_create_attr(&attr, &cl_pool[dev_id]);
 }
 
 int ze_hmem_copy(uint64_t device, void *dst, const void *src, size_t size)
 {
-	ze_command_list_handle_t cmd_list;
+	ze_command_list_handle_t *cmd_list;
 	ze_result_t ze_ret;
-	int dev_id = (int) device;
+	int dev_id = ze_get_device_idx(device);
 
-	ze_ret = ofi_zeCommandListCreate(context, devices[dev_id], &cl_desc,
-					 &cmd_list);
+	assert(!ze_get_driver_idx(device));
+
+	/* Host memory allocated via ZE */
+	if (dev_id < 0) {
+		memcpy(dst, src, size);
+		return 0;
+	}
+
+	ofi_spin_lock(&cl_lock);
+	if (!cmd_queue[device]) {
+		ze_ret = ze_init_res(dev_id);
+		if (ze_ret) {
+			ofi_spin_unlock(&cl_lock);
+			goto out;
+		}
+	}
+
+	cmd_list = ofi_buf_alloc(cl_pool[dev_id]);
+	ofi_spin_unlock(&cl_lock);
+	if (!cmd_list)
+		goto out;
+
+	ze_ret = ofi_zeCommandListReset(*cmd_list);
 	if (ze_ret)
-		goto err;
+		goto free;
 
-	ze_ret = ofi_zeCommandListAppendMemoryCopy(cmd_list, dst, src, size,
+	ze_ret = ofi_zeCommandListAppendMemoryCopy(*cmd_list, dst, src, size,
 						   NULL, 0, NULL);
 	if (ze_ret)
 		goto free;
 
-	ze_ret = ofi_zeCommandListClose(cmd_list);
+	ze_ret = ofi_zeCommandListClose(*cmd_list);
 	if (ze_ret)
 		goto free;
 
 	ze_ret = ofi_zeCommandQueueExecuteCommandLists(cmd_queue[dev_id], 1,
-						       &cmd_list, NULL);
+						       cmd_list, NULL);
 
 free:
-	if (!ofi_zeCommandListDestroy(cmd_list) && !ze_ret)
-		return FI_SUCCESS;
-err:
-	FI_WARN(&core_prov, FI_LOG_CORE,
-		"Failed to perform ze copy (%d)\n", ze_ret);
-
-	return -FI_EIO;
+	ofi_spin_lock(&cl_lock);
+	ofi_buf_free(cmd_list);
+	ofi_spin_unlock(&cl_lock);
+out:
+	if (ze_ret) {
+		FI_WARN(&core_prov, FI_LOG_CORE,
+			"Failed to perform ze copy (%d)\n", ze_ret);
+		return -FI_EIO;
+	}
+	return FI_SUCCESS;
 }
 
-bool ze_is_addr_valid(const void *addr)
+bool ze_hmem_is_addr_valid(const void *addr, uint64_t *device, uint64_t *flags)
 {
 	ze_result_t ze_ret;
-	ze_memory_allocation_properties_t mem_props;
-	ze_device_handle_t device;
+	ze_memory_allocation_properties_t mem_props = {0};
+	ze_device_properties_t dev_prop = {0};
+	ze_device_handle_t device_ptr;
+	int i;
 
-	mem_props.stype = ZE_STRUCTURE_TYPE_MEMORY_ALLOCATION_PROPERTIES;
-	mem_props.pNext = NULL;
 	ze_ret = ofi_zeMemGetAllocProperties(context, addr, &mem_props,
-					     &device);
+					     &device_ptr);
+	if (ze_ret || mem_props.type == ZE_MEMORY_TYPE_UNKNOWN)
+		return false;
 
-	return (!ze_ret && mem_props.type == ZE_MEMORY_TYPE_DEVICE);
+	if (flags)
+		*flags = mem_props.type == ZE_MEMORY_TYPE_DEVICE ?
+			 FI_HMEM_DEVICE_ONLY : 0;
+
+	if (!device)
+		return true;
+
+	ze_ret = ofi_zeDeviceGetProperties(device_ptr, &dev_prop);
+	if (ze_ret)
+		return false;
+
+	for (i = 0, *device = 0; i < num_devices; i++) {
+		if (!memcmp(&dev_prop.uuid, &dev_uuids[i],
+			    sizeof(*dev_uuids))) {
+			*device = i;
+			return true;
+		}
+	}
+
+	assert(1);
+	return true;
 }
 
-int ze_hmem_get_handle(void *dev_buf, void **handle)
+int ze_hmem_get_handle(void *dev_buf, size_t size, void **handle)
 {
 	ze_result_t ze_ret;
 
@@ -697,11 +1010,16 @@ int ze_hmem_get_handle(void *dev_buf, void **handle)
 	return FI_SUCCESS;
 }
 
-int ze_hmem_open_handle(void **handle, uint64_t device, void **ipc_ptr)
+int ze_hmem_open_handle(void **handle, size_t size, uint64_t device,
+			void **ipc_ptr)
 {
 	ze_result_t ze_ret;
+	int dev_id = ze_get_device_idx(device);
 
-	ze_ret = ofi_zeMemOpenIpcHandle(context, devices[device],
+	/* only device memory is supported */
+	assert(!ze_get_driver_idx(device) && dev_id >= 0);
+
+	ze_ret = ofi_zeMemOpenIpcHandle(context, devices[dev_id],
 					*((ze_ipc_mem_handle_t *) handle),
 					0, ipc_ptr);
 	if (ze_ret) {
@@ -727,12 +1045,17 @@ int ze_hmem_close_handle(void *ipc_ptr)
 	return FI_SUCCESS;
 }
 
-int ze_hmem_get_base_addr(const void *ptr, void **base)
+int ze_hmem_get_ipc_handle_size(size_t *size)
+{
+	*size = sizeof(ze_ipc_mem_handle_t);
+	return FI_SUCCESS;
+}
+
+int ze_hmem_get_base_addr(const void *ptr, void **base, size_t *size)
 {
 	ze_result_t ze_ret;
-	size_t size;
 
-	ze_ret = ofi_zeMemGetAddressRange(context, ptr, base, &size);
+	ze_ret = ofi_zeMemGetAddressRange(context, ptr, base, size);
 	if (ze_ret) {
 		FI_WARN(&core_prov, FI_LOG_CORE,
 			"Could not get base addr\n");
@@ -766,6 +1089,35 @@ int *ze_hmem_get_dev_fds(int *nfds)
 	return dev_fds;
 }
 
+int ze_hmem_host_register(void *ptr, size_t size)
+{
+	ze_result_t ze_ret;
+
+	if (host_reg_enabled) {
+		ze_ret = ofi_zexDriverImportExternalPointer(driver, ptr, size);
+		if (ze_ret) {
+			FI_WARN(&core_prov, FI_LOG_CORE,
+				"Failed to import host memory: ptr %p size %zd",
+				ptr, size);
+		}
+	}
+	return FI_SUCCESS;
+}
+
+int ze_hmem_host_unregister(void *ptr)
+{
+	ze_result_t ze_ret;
+
+	if (host_reg_enabled) {
+		ze_ret = ofi_zexDriverReleaseImportedPointer(driver, ptr);
+		if (ze_ret) {
+			FI_WARN(&core_prov, FI_LOG_CORE,
+				"Failed to release imported memory: ptr %p", ptr);
+		}
+	}
+	return FI_SUCCESS;
+}
+
 #else
 
 int ze_hmem_init(void)
@@ -783,17 +1135,18 @@ int ze_hmem_copy(uint64_t device, void *dst, const void *src, size_t size)
 	return -FI_ENOSYS;
 }
 
-bool ze_is_addr_valid(const void *addr)
+bool ze_hmem_is_addr_valid(const void *addr, uint64_t *device, uint64_t *flags)
 {
 	return false;
 }
 
-int ze_hmem_get_handle(void *dev_buf, void **handle)
+int ze_hmem_get_handle(void *dev_buf, size_t size, void **handle)
 {
 	return -FI_ENOSYS;
 }
 
-int ze_hmem_open_handle(void **handle, uint64_t device, void **ipc_ptr)
+int ze_hmem_open_handle(void **handle, size_t size, uint64_t device,
+			void **ipc_ptr)
 {
 	return -FI_ENOSYS;
 }
@@ -820,7 +1173,12 @@ bool ze_hmem_p2p_enabled(void)
 	return false;
 }
 
-int ze_hmem_get_base_addr(const void *ptr, void **base)
+int ze_hmem_get_ipc_handle_size(size_t *size)
+{
+	return -FI_ENOSYS;
+}
+
+int ze_hmem_get_base_addr(const void *ptr, void **base, size_t *size)
 {
 	return -FI_ENOSYS;
 }
@@ -836,4 +1194,14 @@ int *ze_hmem_get_dev_fds(int *nfds)
 	return NULL;
 }
 
-#endif /* HAVE_LIBZE */
+int ze_hmem_host_register(void *ptr, size_t size)
+{
+	return FI_SUCCESS;
+}
+
+int ze_hmem_host_unregister(void *ptr)
+{
+	return FI_SUCCESS;
+}
+
+#endif /* HAVE_ZE */
