@@ -1,5 +1,5 @@
 /**
- * Copyright (C) Mellanox Technologies Ltd. 2021.  ALL RIGHTS RESERVED.
+ * Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2021. ALL RIGHTS RESERVED.
  *
  * See file LICENSE for terms.
  */
@@ -12,6 +12,8 @@
 #include <ucp/tag/proto_eager.inl>
 #include <ucp/proto/proto_single.inl>
 
+#define UCP_PROTO_EAGER_OFFLOAD_DESC "eager offloaded"
+
 
 static ucs_status_t
 ucp_proto_eager_tag_offload_short_progress(uct_pending_req_t *self)
@@ -21,7 +23,7 @@ ucp_proto_eager_tag_offload_short_progress(uct_pending_req_t *self)
     const ucp_proto_single_priv_t *spriv = req->send.proto_config->priv;
     ucs_status_t status;
 
-    status = uct_ep_tag_eager_short(ep->uct_eps[spriv->super.lane],
+    status = uct_ep_tag_eager_short(ucp_ep_get_fast_lane(ep, spriv->super.lane),
                                     req->send.msg_proto.tag,
                                     req->send.state.dt_iter.type.contig.buffer,
                                     req->send.state.dt_iter.length);
@@ -45,12 +47,13 @@ static ucs_status_t ucp_proto_eager_tag_offload_short_init(
     const ucp_proto_select_param_t *select_param = init_params->select_param;
     ucp_proto_single_init_params_t params        = {
         .super.super         = *init_params,
-        .super.latency       = -150e-9, /* no extra mem access to fetch data */
+        .super.latency       = 0,
         .super.overhead      = 0,
         .super.cfg_thresh    = UCS_MEMUNITS_AUTO,
         .super.cfg_priority  = 0,
         .super.min_length    = 0,
         .super.max_length    = SIZE_MAX,
+        .super.min_iov       = 0,
         .super.min_frag_offs = UCP_PROTO_COMMON_OFFSET_INVALID,
         .super.max_frag_offs = ucs_offsetof(uct_iface_attr_t,
                                             cap.tag.eager.max_short),
@@ -59,29 +62,31 @@ static ucs_status_t ucp_proto_eager_tag_offload_short_init(
         .super.send_op       = UCT_EP_OP_EAGER_SHORT,
         .super.memtype_op    = UCT_EP_OP_LAST,
         .super.flags         = UCP_PROTO_COMMON_INIT_FLAG_SINGLE_FRAG |
-                               UCP_PROTO_COMMON_INIT_FLAG_RECV_ZCOPY,
+                               UCP_PROTO_COMMON_INIT_FLAG_RECV_ZCOPY |
+                               UCP_PROTO_COMMON_INIT_FLAG_CAP_SEG_SIZE,
+        .super.exclude_map   = 0,
         .lane_type           = UCP_LANE_TYPE_TAG,
         .tl_cap_flags        = UCT_IFACE_FLAG_TAG_EAGER_SHORT
     };
 
     if (!ucp_proto_eager_check_op_id(init_params, UCP_OP_ID_TAG_SEND, 1) ||
-        /* short protocol requires contig/host */
-        (select_param->dt_class != UCP_DATATYPE_CONTIG) ||
-        !UCP_MEM_IS_HOST(select_param->mem_type)) {
+        !ucp_proto_is_short_supported(select_param)) {
         return UCS_ERR_UNSUPPORTED;
     }
 
     return ucp_proto_single_init(&params);
 }
 
-static ucp_proto_t ucp_eager_tag_offload_short_proto = {
-    .name       = "egr/offload/short",
-    .flags      = UCP_PROTO_FLAG_TAG_SHORT,
-    .init       = ucp_proto_eager_tag_offload_short_init,
-    .config_str = ucp_proto_single_config_str,
-    .progress   = {ucp_proto_eager_tag_offload_short_progress}
+ucp_proto_t ucp_eager_tag_offload_short_proto = {
+    .name     = "egr/offload/short",
+    .desc     = UCP_PROTO_EAGER_OFFLOAD_DESC " " UCP_PROTO_SHORT_DESC,
+    .flags    = UCP_PROTO_FLAG_TAG_SHORT,
+    .init     = ucp_proto_eager_tag_offload_short_init,
+    .query    = ucp_proto_single_query,
+    .progress = {ucp_proto_eager_tag_offload_short_progress},
+    .abort    = ucp_proto_abort_fatal_not_implemented,
+    .reset    = ucp_proto_request_bcopy_reset
 };
-UCP_PROTO_REGISTER(&ucp_eager_tag_offload_short_proto);
 
 static size_t ucp_eager_tag_offload_pack(void *dest, void *arg)
 {
@@ -102,7 +107,8 @@ ucp_proto_eager_tag_offload_bcopy_common(ucp_request_t *req,
 {
     ssize_t packed_len;
 
-    packed_len = uct_ep_tag_eager_bcopy(req->send.ep->uct_eps[spriv->super.lane],
+    packed_len = uct_ep_tag_eager_bcopy(ucp_ep_get_fast_lane(req->send.ep,
+                                                             spriv->super.lane),
                                         req->send.msg_proto.tag, imm_data,
                                         ucp_eager_tag_offload_pack, req, 0);
 
@@ -121,6 +127,7 @@ static ucs_status_t ucp_proto_eager_tag_offload_bcopy_init_common(
         .super.cfg_priority  = 20,
         .super.min_length    = 0,
         .super.max_length    = SIZE_MAX,
+        .super.min_iov       = 0,
         .super.min_frag_offs = UCP_PROTO_COMMON_OFFSET_INVALID,
         .super.max_frag_offs = ucs_offsetof(uct_iface_attr_t,
                                             cap.tag.eager.max_bcopy),
@@ -129,7 +136,9 @@ static ucs_status_t ucp_proto_eager_tag_offload_bcopy_init_common(
         .super.send_op       = UCT_EP_OP_EAGER_BCOPY,
         .super.memtype_op    = UCT_EP_OP_LAST,
         .super.flags         = UCP_PROTO_COMMON_INIT_FLAG_SINGLE_FRAG |
-                               UCP_PROTO_COMMON_INIT_FLAG_RECV_ZCOPY,
+                               UCP_PROTO_COMMON_INIT_FLAG_RECV_ZCOPY |
+                               UCP_PROTO_COMMON_INIT_FLAG_CAP_SEG_SIZE,
+        .super.exclude_map   = 0,
         .lane_type           = UCP_LANE_TYPE_TAG,
         .tl_cap_flags        = UCT_IFACE_FLAG_TAG_EAGER_BCOPY
     };
@@ -164,14 +173,16 @@ static ucs_status_t ucp_proto_eager_tag_offload_bcopy_init(
                                                          UCP_OP_ID_TAG_SEND);
 }
 
-static ucp_proto_t ucp_eager_bcopy_single_proto = {
-    .name       = "egr/offload/bcopy",
-    .flags      = 0,
-    .init       = ucp_proto_eager_tag_offload_bcopy_init,
-    .config_str = ucp_proto_single_config_str,
-    .progress   = {ucp_proto_eager_tag_offload_bcopy_progress}
+ucp_proto_t ucp_tag_offload_eager_bcopy_single_proto = {
+    .name     = "egr/offload/bcopy",
+    .desc     = UCP_PROTO_EAGER_OFFLOAD_DESC " " UCP_PROTO_COPY_IN_DESC,
+    .flags    = 0,
+    .init     = ucp_proto_eager_tag_offload_bcopy_init,
+    .query    = ucp_proto_single_query,
+    .progress = {ucp_proto_eager_tag_offload_bcopy_progress},
+    .abort    = ucp_proto_abort_fatal_not_implemented,
+    .reset    = ucp_proto_request_bcopy_reset
 };
-UCP_PROTO_REGISTER(&ucp_eager_bcopy_single_proto);
 
 static UCS_F_ALWAYS_INLINE ucs_status_t
 ucp_proto_eager_sync_tag_offload_bcopy_posted(ucp_request_t *req)
@@ -187,6 +198,7 @@ ucp_proto_eager_sync_tag_offload_bcopy_progress(uct_pending_req_t *self)
     const ucp_proto_single_priv_t *spriv = req->send.proto_config->priv;
     ucs_status_t status;
 
+    req->flags |= UCP_REQUEST_FLAG_PROTO_INITIALIZED;
     status = ucp_proto_eager_tag_offload_bcopy_common(
             req, spriv, ucp_send_request_get_ep_remote_id(req));
     return ucp_proto_single_status_handle(
@@ -201,14 +213,16 @@ static ucs_status_t ucp_proto_eager_sync_tag_offload_bcopy_init(
             init_params, UCP_OP_ID_TAG_SEND_SYNC);
 }
 
-static ucp_proto_t ucp_eager_sync_bcopy_single_proto = {
-    .name       = "egrsnc/offload/bcopy",
-    .flags      = 0,
-    .init       = ucp_proto_eager_sync_tag_offload_bcopy_init,
-    .config_str = ucp_proto_single_config_str,
-    .progress   = {ucp_proto_eager_sync_tag_offload_bcopy_progress}
+ucp_proto_t ucp_eager_sync_bcopy_single_proto = {
+    .name     = "egrsnc/offload/bcopy",
+    .desc     = UCP_PROTO_EAGER_OFFLOAD_DESC " " UCP_PROTO_COPY_IN_DESC,
+    .flags    = 0,
+    .init     = ucp_proto_eager_sync_tag_offload_bcopy_init,
+    .query    = ucp_proto_single_query,
+    .progress = {ucp_proto_eager_sync_tag_offload_bcopy_progress},
+    .abort    = ucp_proto_abort_fatal_not_implemented,
+    .reset    = ucp_proto_request_bcopy_id_reset
 };
-UCP_PROTO_REGISTER(&ucp_eager_sync_bcopy_single_proto);
 
 static ucs_status_t ucp_proto_eager_tag_offload_zcopy_init_common(
         const ucp_proto_init_params_t *init_params, ucp_proto_id_t op_id)
@@ -222,16 +236,20 @@ static ucs_status_t ucp_proto_eager_tag_offload_zcopy_init_common(
         .super.cfg_priority  = 30,
         .super.min_length    = 0,
         .super.max_length    = SIZE_MAX,
+        .super.min_iov       = 1,
         .super.min_frag_offs = UCP_PROTO_COMMON_OFFSET_INVALID,
         .super.max_frag_offs = ucs_offsetof(uct_iface_attr_t,
                                             cap.tag.eager.max_zcopy),
-        .super.max_iov_offs  = UCP_PROTO_COMMON_OFFSET_INVALID,
+        .super.max_iov_offs  = ucs_offsetof(uct_iface_attr_t,
+                                            cap.tag.eager.max_iov),
         .super.hdr_size      = sizeof(ucp_tag_t),
         .super.send_op       = UCT_EP_OP_EAGER_ZCOPY,
         .super.memtype_op    = UCT_EP_OP_LAST,
         .super.flags         = UCP_PROTO_COMMON_INIT_FLAG_SEND_ZCOPY |
                                UCP_PROTO_COMMON_INIT_FLAG_RECV_ZCOPY |
-                               UCP_PROTO_COMMON_INIT_FLAG_SINGLE_FRAG,
+                               UCP_PROTO_COMMON_INIT_FLAG_SINGLE_FRAG |
+                               UCP_PROTO_COMMON_INIT_FLAG_CAP_SEG_SIZE,
+        .super.exclude_map   = 0,
         .lane_type           = UCP_LANE_TYPE_TAG,
         .tl_cap_flags        = UCT_IFACE_FLAG_TAG_EAGER_ZCOPY
     };
@@ -255,9 +273,10 @@ static ucs_status_t ucp_proto_eager_tag_offload_zcopy_init(
 static ucs_status_t
 ucp_proto_tag_offload_zcopy_send_func(ucp_request_t *req,
                                       const ucp_proto_single_priv_t *spriv,
-                                      const uct_iov_t *iov)
+                                      uct_iov_t *iov)
 {
-    return uct_ep_tag_eager_zcopy(req->send.ep->uct_eps[spriv->super.lane],
+    return uct_ep_tag_eager_zcopy(ucp_ep_get_fast_lane(req->send.ep,
+                                                       spriv->super.lane),
                                   req->send.msg_proto.tag, 0ul, iov, 1, 0,
                                   &req->send.state.uct_comp);
 }
@@ -271,17 +290,19 @@ ucp_proto_eager_tag_offload_zcopy_progress(uct_pending_req_t *self)
             req, UCT_MD_MEM_ACCESS_LOCAL_READ,
             ucp_proto_tag_offload_zcopy_send_func,
             ucp_request_invoke_uct_completion_success,
-            ucp_proto_request_zcopy_completion);
+            ucp_proto_request_zcopy_completion, ucp_proto_request_zcopy_init);
 }
 
-static ucp_proto_t ucp_eager_zcopy_single_proto = {
-    .name       = "egr/offload/zcopy",
-    .flags      = 0,
-    .init       = ucp_proto_eager_tag_offload_zcopy_init,
-    .config_str = ucp_proto_single_config_str,
-    .progress   = {ucp_proto_eager_tag_offload_zcopy_progress}
+ucp_proto_t ucp_tag_offload_eager_zcopy_single_proto = {
+    .name     = "egr/offload/zcopy",
+    .desc     = UCP_PROTO_EAGER_OFFLOAD_DESC " " UCP_PROTO_ZCOPY_DESC,
+    .flags    = 0,
+    .init     = ucp_proto_eager_tag_offload_zcopy_init,
+    .query    = ucp_proto_single_query,
+    .progress = {ucp_proto_eager_tag_offload_zcopy_progress},
+    .abort    = ucp_proto_abort_fatal_not_implemented,
+    .reset    = ucp_proto_request_zcopy_reset
 };
-UCP_PROTO_REGISTER(&ucp_eager_zcopy_single_proto);
 
 static ucs_status_t ucp_proto_eager_sync_tag_offload_zcopy_init(
         const ucp_proto_init_params_t *init_params)
@@ -293,9 +314,10 @@ static ucs_status_t ucp_proto_eager_sync_tag_offload_zcopy_init(
 static ucs_status_t
 ucp_proto_tag_offload_zcopy_sync_send_func(ucp_request_t *req,
                                            const ucp_proto_single_priv_t *spriv,
-                                           const uct_iov_t *iov)
+                                           uct_iov_t *iov)
 {
-    return uct_ep_tag_eager_zcopy(req->send.ep->uct_eps[spriv->super.lane],
+    return uct_ep_tag_eager_zcopy(ucp_ep_get_fast_lane(req->send.ep,
+                                                       spriv->super.lane),
                                   req->send.msg_proto.tag,
                                   ucp_send_request_get_ep_remote_id(req), iov,
                                   1, 0, &req->send.state.uct_comp);
@@ -307,7 +329,9 @@ ucp_proto_eager_sync_tag_offload_zcopy_send_completion(uct_completion_t *self)
     ucp_request_t *req = ucs_container_of(self, ucp_request_t,
                                           send.state.uct_comp);
 
-    ucp_proto_request_zcopy_cleanup(req, UCS_BIT(UCP_DATATYPE_CONTIG));
+    ucp_proto_request_zcopy_clean(req, UCS_BIT(UCP_DATATYPE_CONTIG));
+    ucp_datatype_iter_cleanup(&req->send.state.dt_iter,
+                              UCS_BIT(UCP_DATATYPE_CONTIG));
     ucp_proto_eager_sync_send_completed_common(req);
 }
 
@@ -327,14 +351,17 @@ ucp_proto_eager_sync_tag_offload_zcopy_progress(uct_pending_req_t *self)
             req, UCT_MD_MEM_ACCESS_LOCAL_READ,
             ucp_proto_tag_offload_zcopy_sync_send_func,
             ucp_proto_eager_sync_tag_offload_zcopy_posted,
-            ucp_proto_eager_sync_tag_offload_zcopy_send_completion);
+            ucp_proto_eager_sync_tag_offload_zcopy_send_completion,
+            ucp_proto_request_zcopy_init);
 }
 
-static ucp_proto_t ucp_eager_sync_zcopy_single_proto = {
-    .name       = "egrsnc/offload/zcopy",
-    .flags      = 0,
-    .init       = ucp_proto_eager_sync_tag_offload_zcopy_init,
-    .config_str = ucp_proto_single_config_str,
-    .progress   = {ucp_proto_eager_sync_tag_offload_zcopy_progress}
+ucp_proto_t ucp_eager_sync_zcopy_single_proto = {
+    .name     = "egrsnc/offload/zcopy",
+    .desc     = UCP_PROTO_EAGER_OFFLOAD_DESC " " UCP_PROTO_ZCOPY_DESC,
+    .flags    = 0,
+    .init     = ucp_proto_eager_sync_tag_offload_zcopy_init,
+    .query    = ucp_proto_single_query,
+    .progress = {ucp_proto_eager_sync_tag_offload_zcopy_progress},
+    .abort    = ucp_proto_abort_fatal_not_implemented,
+    .reset    = ucp_proto_request_zcopy_id_reset
 };
-UCP_PROTO_REGISTER(&ucp_eager_sync_zcopy_single_proto);
