@@ -1,5 +1,5 @@
 /**
- * Copyright (C) Mellanox Technologies Ltd. 2001-2020.  ALL RIGHTS RESERVED.
+ * Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2020. ALL RIGHTS RESERVED.
  *
  * See file LICENSE for terms.
  */
@@ -22,7 +22,6 @@ UCS_PROFILE_FUNC(ucp_tag_message_h, ucp_tag_probe_nb,
                  ucp_worker_h worker, ucp_tag_t tag, ucp_tag_t tag_mask,
                  int rem, ucp_tag_recv_info_t *info)
 {
-    ucp_context_h UCS_V_UNUSED context = worker->context;
     ucp_recv_desc_t *rdesc;
     uint16_t flags;
 
@@ -33,7 +32,7 @@ UCS_PROFILE_FUNC(ucp_tag_message_h, ucp_tag_probe_nb,
     ucs_trace_req("probe_nb tag %"PRIx64"/%"PRIx64" remove=%d", tag, tag_mask,
                   rem);
 
-    rdesc = ucp_tag_unexp_search(&worker->tm, tag, tag_mask, rem, "probe");
+    rdesc = ucp_tag_unexp_search(&worker->tm, tag, tag_mask, 0, "probe");
     if (rdesc != NULL) {
         flags            = rdesc->flags;
         info->sender_tag = ucp_rdesc_get_tag(rdesc);
@@ -41,13 +40,30 @@ UCS_PROFILE_FUNC(ucp_tag_message_h, ucp_tag_probe_nb,
         if (flags & UCP_RECV_DESC_FLAG_EAGER_ONLY) {
             info->length = rdesc->length - rdesc->payload_offset;
         } else if (flags & UCP_RECV_DESC_FLAG_EAGER) {
+            if (ucs_test_all_flags(flags, UCP_RECV_DESC_FLAG_EAGER_OFFLOAD |
+                                          UCP_RECV_DESC_FLAG_RECV_STARTED)) {
+                /* Do not return the rdesc, because not all fragments arrived
+                 * yet. Need to wait until last fragment arrives to know the
+                 * correct length of the message. */
+                rdesc = NULL;
+                goto out;
+            }
+
+            UCS_STATIC_ASSERT(
+                    ucs_offsetof(ucp_eager_first_hdr_t, total_len) ==
+                    ucs_offsetof(ucp_offload_first_desc_t, total_length));
             info->length = ((ucp_eager_first_hdr_t*)(rdesc + 1))->total_len;
         } else {
             ucs_assert(flags & UCP_RECV_DESC_FLAG_RNDV);
             info->length = ucp_tag_rndv_rts_from_rdesc(rdesc)->size;
         }
+
+        if (rem) {
+             ucp_tag_unexp_remove(rdesc);
+        }
     }
 
+out:
     UCP_WORKER_THREAD_CS_EXIT_CONDITIONAL(worker);
 
     return rdesc;
