@@ -1,5 +1,5 @@
 /**
- * Copyright (C) Mellanox Technologies Ltd. 2001-2015.  ALL RIGHTS RESERVED.
+ * Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2015. ALL RIGHTS RESERVED.
  *
  * See file LICENSE for terms.
  */
@@ -29,7 +29,10 @@ enum {
     UCP_WIREUP_EP_FLAG_REMOTE_CONNECTED = UCS_BIT(2),
 
     /* Send client id */
-    UCP_WIREUP_EP_FLAG_SEND_CLIENT_ID   = UCS_BIT(3)
+    UCP_WIREUP_EP_FLAG_SEND_CLIENT_ID   = UCS_BIT(3),
+
+    /* Indicates that aux_ep is CONNECT_TO_EP */
+    UCP_WIREUP_EP_FLAG_AUX_P2P          = UCS_BIT(4)
 };
 
 
@@ -45,15 +48,17 @@ struct ucp_wireup_ep {
     struct sockaddr_storage   cm_remote_sockaddr;  /**< sockaddr of the remote peer -
                                                         used only on the client side
                                                         in a client-server flow */
+    struct sockaddr_storage   cm_local_sockaddr;   /**< local sockaddr
+                                                        used only on the client side
+                                                        in a client-server flow */
     ucp_rsc_index_t           aux_rsc_index; /**< Index of auxiliary transport */
     volatile uint32_t         pending_count; /**< Number of pending wireup operations */
     volatile uint32_t         flags;         /**< Connection state flags */
-    uct_worker_cb_id_t        progress_id;   /**< ID of progress function */
     unsigned                  ep_init_flags; /**< UCP wireup EP init flags */
-    /**< TLs which are awailable on client side resolved device */
+    /**< TLs which are available on client side resolved device */
     ucp_tl_bitmap_t           cm_resolve_tl_bitmap;
     /**< Destination resource indicies used for checking intersection between
-         between two configurations in case of CM */
+         two configurations in case of CM */
     ucp_rsc_index_t           dst_rsc_indices[UCP_MAX_LANES];
 };
 
@@ -61,7 +66,9 @@ struct ucp_wireup_ep {
 /**
  * Create a proxy endpoint for wireup.
  */
-ucs_status_t ucp_wireup_ep_create(ucp_ep_h ep, uct_ep_h *ep_p);
+ucs_status_t ucp_wireup_ep_create(ucp_ep_h ep,
+                                  const ucp_rsc_index_t *dst_rsc_indices,
+                                  uct_ep_h *ep_p);
 
 
 /**
@@ -94,25 +101,22 @@ void ucp_wireup_ep_pending_queue_purge(uct_ep_h uct_ep,
                                        void *arg);
 
 void ucp_wireup_ep_set_aux(ucp_wireup_ep_t *wireup_ep, uct_ep_h uct_ep,
-                           ucp_rsc_index_t rsc_index);
-
-ucs_status_t
-ucp_wireup_ep_connect_aux(ucp_wireup_ep_t *wireup_ep, unsigned ep_init_flags,
-                          const ucp_unpacked_address_t *remote_address);
+                           ucp_rsc_index_t rsc_index, int is_p2p);
 
 void ucp_wireup_ep_discard_aux_ep(ucp_wireup_ep_t *wireup_ep,
                                   unsigned ep_flush_flags,
                                   uct_pending_purge_callback_t purge_cb,
                                   void *purge_arg);
 
-void ucp_wireup_ep_set_next_ep(uct_ep_h uct_ep, uct_ep_h next_ep);
+int ucp_wireup_ep_has_next_ep(ucp_wireup_ep_t *wireup_ep);
+
+void ucp_wireup_ep_set_next_ep(uct_ep_h uct_ep, uct_ep_h next_ep,
+                               ucp_rsc_index_t rsc_index);
 
 uct_ep_h ucp_wireup_ep_extract_next_ep(uct_ep_h uct_ep);
 
 void ucp_wireup_ep_destroy_next_ep(ucp_wireup_ep_t *wireup_ep);
 
-void ucp_wireup_ep_remote_connected(uct_ep_h uct_ep, int ready);
-
 int ucp_wireup_ep_test(uct_ep_h uct_ep);
 
 int ucp_wireup_aux_ep_is_owner(ucp_wireup_ep_t *wireup_ep, uct_ep_h owned_ep);
@@ -127,4 +131,14 @@ ucs_status_t ucp_wireup_ep_progress_pending(uct_pending_req_t *self);
 
 ucp_wireup_ep_t *ucp_wireup_ep(uct_ep_h uct_ep);
 
+unsigned ucp_wireup_ep_pending_extract(ucp_wireup_ep_t *wireup_ep,
+                                       ucs_queue_head_t *queue);
+
+void ucp_wireup_eps_pending_extract(ucp_ep_t *ucp_ep, ucs_queue_head_t *queue);
+
+ucs_status_t
+ucp_wireup_ep_connect_to_ep_v2(uct_ep_h tl_ep,
+                               const ucp_address_entry_t *address_entry,
+                               const ucp_address_entry_ep_addr_t *ep_entry);
+
 #endif
