@@ -1,5 +1,5 @@
 /**
- * Copyright (C) Mellanox Technologies Ltd. 2018.  ALL RIGHTS RESERVED.
+ * Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2018. ALL RIGHTS RESERVED.
  *
  * See file LICENSE for terms.
  */
@@ -7,10 +7,36 @@
 #ifndef UCS_REG_CACHE_INT_H_
 #define UCS_REG_CACHE_INT_H_
 
+#include "rcache.h"
+
 #include <ucs/datastruct/list.h>
+#include <ucs/stats/stats.h>
 #include <ucs/type/spinlock.h>
 
 
+#define ucs_rcache_region_log_lvl(_level, _message, ...) \
+    do { \
+        if (ucs_log_is_enabled(_level)) { \
+            ucs_rcache_region_log(__FILE__, __LINE__, __FUNCTION__, (_level), \
+                                  _message, ## __VA_ARGS__); \
+        } \
+    } while (0)
+
+
+#define ucs_rcache_region_error(_message, ...) \
+    ucs_rcache_region_log_lvl(UCS_LOG_LEVEL_ERROR, _message, ## __VA_ARGS__)
+#define ucs_rcache_region_warn(_message, ...)  \
+    ucs_rcache_region_log_lvl(UCS_LOG_LEVEL_WARN, _message, ## __VA_ARGS__)
+#define ucs_rcache_region_debug(_message, ...) \
+    ucs_rcache_region_log_lvl(UCS_LOG_LEVEL_DEBUG, _message, ## __VA_ARGS__)
+#define ucs_rcache_region_trace(_message, ...) \
+    ucs_rcache_region_log_lvl(UCS_LOG_LEVEL_TRACE, _message, ## __VA_ARGS__)
+
+
+#define UCS_RCACHE_STAT_MIN_POW2 \
+    ucs_roundup_pow2(ucs_global_opts.rcache_stat_min)
+
+
 /* Names of rcache stats counters */
 enum {
     UCS_RCACHE_GETS,                /* number of get operations */
@@ -28,6 +54,14 @@ enum {
 };
 
 
+/* The structure represents a group in regestration cache regions distribution.
+   Regions are distributed by thier size.
+ */
+typedef struct ucs_rcache_distribution {
+    size_t count; /**< Number of regions in the group */
+    size_t total_size; /**< Total size of regions in the group */
+} ucs_rcache_distribution_t;
+
 struct ucs_rcache {
     ucs_rcache_params_t params;          /**< rcache parameters (immutable) */
 
@@ -63,14 +97,49 @@ struct ucs_rcache {
                                               The head of the list is the least
                                               recently used region, and the tail
                                               is the most recently used region. */
-        unsigned long   count;           /**< Number of regions on list */
     } lru;
-    
+
     char                *name;           /**< Name of the cache, for debug purpose */
 
     UCS_STATS_NODE_DECLARE(stats)
 
-    ucs_list_link_t          list;       /**< list entry in global ucs_rcache list */
+    ucs_list_link_t           list; /**< List entry in global ucs_rcache list */
+    ucs_rcache_distribution_t *distribution; /**< Distribution of registration
+                                                  cache regions by size */
 };
 
+
+/**
+ * @brief Create objects in VFS to represent registration cache and its
+ *        features.
+ *
+ * @param [in] rcache Registration cache object to be described.
+ */
+void ucs_rcache_vfs_init(ucs_rcache_t *rcache);
+
+
+/**
+ * @brief Get number of bins in the distribution of registration cache region
+ *        sizes.
+ *
+ * @return Number of bins.
+ */
+size_t ucs_rcache_distribution_get_num_bins();
+
+
+ucs_status_t
+ucs_rcache_create_region(ucs_rcache_t *rcache, void *address, size_t length,
+                         int prot, void *arg, ucs_rcache_region_t **region_p);
+
+
+void ucs_mem_region_destroy_internal(ucs_rcache_t *rcache,
+                                     ucs_rcache_region_t *region,
+                                     int drop_lock);
+
+
+void ucs_rcache_region_log(const char *file, int line, const char *function,
+                           ucs_log_level_t level, ucs_rcache_t *rcache,
+                           ucs_rcache_region_t *region, const char *fmt,
+                           ...) UCS_F_PRINTF(7, 8);
+
 #endif
