@@ -1,5 +1,5 @@
 /**
- * Copyright (C) Mellanox Technologies Ltd. 2001-2018.  ALL RIGHTS RESERVED.
+ * Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2018. ALL RIGHTS RESERVED.
  *
  * See file LICENSE for terms.
  */
@@ -13,6 +13,7 @@
 #include <ucs/datastruct/list.h>
 #include <ucs/debug/debug_int.h>
 #include <ucs/debug/log.h>
+#include <ucs/sys/lib.h>
 #include <ucs/sys/string.h>
 #include <ucs/sys/sys.h>
 #include <ucs/time/time.h>
@@ -20,8 +21,8 @@
 
 
 typedef struct ucs_profile_global_location {
-    ucs_profile_location_t       super;      /*< Location info */
-    volatile int                 *loc_id_p;  /*< Back-pointer to location index */
+    ucs_profile_location_t        super; /*< Location info */
+    volatile ucs_profile_loc_id_t *loc_id_p; /*< Back-pointer to location index */
 } ucs_profile_global_location_t;
 
 
@@ -246,7 +247,7 @@ static void ucs_profile_write(ucs_profile_context_t *ctx)
     ucs_read_file(header.cmdline, sizeof(header.cmdline), 1, "/proc/self/cmdline");
     strncpy(header.hostname, ucs_get_host_name(), sizeof(header.hostname) - 1);
     header.version       = UCS_PROFILE_FILE_VERSION;
-    strncpy(header.ucs_path, ucs_debug_get_lib_path(), sizeof(header.ucs_path) - 1);
+    strncpy(header.ucs_path, ucs_sys_get_lib_path(), sizeof(header.ucs_path) - 1);
     header.pid           = getpid();
     header.mode          = ctx->profile_mode;
     header.num_locations = ctx->num_locations;
@@ -359,17 +360,31 @@ static void ucs_profile_thread_key_destr(void *data)
     ucs_profile_thread_finalize(ctx);
 }
 
+static ucs_profile_loc_id_t
+ucs_profile_location_id(ucs_profile_context_t *ctx,
+                        ucs_profile_global_location_t *loc)
+{
+    size_t raw_loc_id = (loc - ctx->locations) + 1; /* Array index plus 1 */
+    ucs_profile_loc_id_t loc_id;
+
+    loc_id = raw_loc_id;
+    ucs_assertv_always(loc_id == raw_loc_id,
+                       "profile location id overflow loc_id=%d raw_loc_id=%zd",
+                       loc_id, raw_loc_id);
+    return loc_id;
+}
+
 /*
  * Register a profiling location - should be called once per location in the
  * code, before the first record of each such location is made.
  * SHOULD NOT be used directly - use UCS_PROFILE macros instead.
  *
- * @param [in]  ctx          Pofile context.
+ * @param [in]  ctx          Profile context.
  * @param [in]  type         Location type.
+ * @param [in]  name         Location name.
  * @param [in]  file         Source file name.
  * @param [in]  line         Source line number.
  * @param [in]  function     Calling function name.
- * @param [in]  name         Location name.
  * @param [out] loc_id_p     Filled with location ID:
  *                             0   - profiling is disabled
  *                             >0  - location index + 1
@@ -377,7 +392,8 @@ static void ucs_profile_thread_key_destr(void *data)
 static UCS_F_NOINLINE int
 ucs_profile_get_location(ucs_profile_context_t *ctx, ucs_profile_type_t type,
                          const char *name, const char *file, int line,
-                         const char *function, volatile int *loc_id_p)
+                         const char *function,
+                         volatile ucs_profile_loc_id_t *loc_id_p)
 {
     ucs_profile_global_location_t *loc, *new_locations;
     int loc_id;
@@ -392,12 +408,12 @@ ucs_profile_get_location(ucs_profile_context_t *ctx, ucs_profile_type_t type,
 
     /* Check if profiling is disabled */
     if (!ctx->profile_mode) {
-        *loc_id_p = loc_id = 0;
+        *loc_id_p = loc_id = UCS_PROFILE_LOC_ID_DISABLED;
         goto out_unlock;
     }
 
     /* Location ID must be uninitialized */
-    ucs_assert(*loc_id_p == -1);
+    ucs_assert(*loc_id_p == UCS_PROFILE_LOC_ID_UNKNOWN);
 
     ucs_profile_ctx_for_each_location(ctx, loc) {
         if ((type == loc->super.type) && (line == loc->super.line) &&
@@ -418,7 +434,7 @@ ucs_profile_get_location(ucs_profile_context_t *ctx, ucs_profile_type_t type,
                                     "profile_locations");
         if (new_locations == NULL) {
             ucs_warn("failed to expand locations array");
-            *loc_id_p = loc_id = 0;
+            *loc_id_p = loc_id = UCS_PROFILE_LOC_ID_DISABLED;
             goto out_unlock;
         }
 
@@ -435,7 +451,7 @@ ucs_profile_get_location(ucs_profile_context_t *ctx, ucs_profile_type_t type,
     loc->loc_id_p   = loc_id_p;
 
 out_found:
-    *loc_id_p = loc_id = (loc - ctx->locations) + 1;
+    *loc_id_p = loc_id = ucs_profile_location_id(ctx, loc);
     ucs_memory_cpu_store_fence();
 out_unlock:
     pthread_mutex_unlock(&ctx->mutex);
@@ -471,27 +487,27 @@ static void ucs_profile_thread_expand_locations(ucs_profile_context_t *ctx,
 void ucs_profile_record(ucs_profile_context_t *ctx, ucs_profile_type_t type,
                         const char *name, uint32_t param32, uint64_t param64,
                         const char *file, int line, const char *function,
-                        volatile int *loc_id_p)
+                        volatile ucs_profile_loc_id_t *loc_id_p)
 {
     ucs_profile_thread_location_t *loc;
     ucs_profile_thread_context_t *thread_ctx;
+    ucs_profile_loc_id_t loc_id;
     ucs_profile_record_t *rec;
     ucs_time_t current_time;
-    int loc_id;
 
     /* If the location id is -1 or 0, need to re-read it with lock held */
     loc_id = *loc_id_p;
     if (ucs_unlikely(loc_id <= 0)) {
         loc_id = ucs_profile_get_location(ctx, type, name, file, line,
                                           function, loc_id_p);
-        if (loc_id == 0) {
+        if (loc_id == UCS_PROFILE_LOC_ID_DISABLED) {
             return;
         }
     }
 
     ucs_memory_cpu_load_fence();
 
-    ucs_assert(*loc_id_p            != 0);
+    ucs_assert(*loc_id_p != UCS_PROFILE_LOC_ID_DISABLED);
     ucs_assert(ctx->profile_mode != 0);
 
     /* Get thread-specific profiling context */
@@ -550,7 +566,7 @@ static void ucs_profile_check_active_threads(ucs_profile_context_t *ctx)
     }
 }
 
-void ucs_profile_reset_locations(ucs_profile_context_t *ctx)
+void ucs_profile_reset_locations_id(ucs_profile_context_t *ctx)
 {
     ucs_profile_global_location_t *loc;
 
@@ -560,6 +576,13 @@ void ucs_profile_reset_locations(ucs_profile_context_t *ctx)
         *loc->loc_id_p = -1;
     }
 
+    pthread_mutex_unlock(&ctx->mutex);
+}
+
+static void ucs_profile_reset_locations(ucs_profile_context_t *ctx)
+{
+    pthread_mutex_lock(&ctx->mutex);
+
     ctx->num_locations = 0;
     ctx->max_locations = 0;
     ucs_free(ctx->locations);
@@ -621,6 +644,7 @@ ucs_status_t ucs_profile_init(unsigned profile_mode, const char *file_name,
     ctx->profile_mode     = profile_mode;
     ctx->file_name        = file_name;
     ctx->max_file_size    = max_file_size;
+    /* coverity[missing_lock] */
     ctx->num_locations    = 0;
     ctx->locations        = NULL;
     ctx->max_locations    = 0;
