@@ -1,5 +1,5 @@
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2016-2020.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2016-2020. ALL RIGHTS RESERVED.
 
 * See file LICENSE for terms.
 */
@@ -16,6 +16,15 @@
 #define UCT_DC_MLX5_EP_NO_DCI ((uint8_t)-1)
 
 
+#define UCT_DC_MLX5_TXQP_DECL(_txqp, _txwq) \
+    uct_rc_txqp_t UCS_V_UNUSED *_txqp; \
+    uct_ib_mlx5_txwq_t UCS_V_UNUSED *_txwq;
+
+
+#define UCT_DC_MLX5_IFACE_TXQP_GET(_iface, _ep, _txqp, _txwq) \
+    UCT_DC_MLX5_IFACE_TXQP_DCI_GET(_iface, (_ep)->dci, _txqp, _txwq)
+
+
 enum uct_dc_mlx5_ep_flags {
     /* DCI pool EP assigned to according to it's lag port */
     UCT_DC_MLX5_EP_FLAG_POOL_INDEX_MASK     = UCS_MASK(3),
@@ -37,28 +46,37 @@ enum uct_dc_mlx5_ep_flags {
 
     /* Error handler already called or flush(CANCEL) disabled it */
     UCT_DC_MLX5_EP_FLAG_ERR_HANDLER_INVOKED = UCS_BIT(7),
+
+    /* EP supports flush remote operation */
+    UCT_DC_MLX5_EP_FLAG_FLUSH_RKEY          = UCS_BIT(8),
+
+    /* Flush remote operation should be invoked */
+    UCT_DC_MLX5_EP_FLAG_FLUSH_REMOTE        = UCS_BIT(9),
+
+#if UCS_ENABLE_ASSERT
+    /* EP was invalidated without DCI */
+    UCT_DC_MLX5_EP_FLAG_INVALIDATED         = UCS_BIT(10)
+#else
+    UCT_DC_MLX5_EP_FLAG_INVALIDATED         = 0
+#endif
 };
 
+/* Address-vector for link-local scope */
+typedef struct uct_dc_mlx5_base_av {
+    uint32_t              dqp_dct;
+    uint16_t              rlid;
+} UCS_S_PACKED uct_dc_mlx5_base_av_t;
 
 struct uct_dc_mlx5_ep {
-    /*
-     * per value of 'flags':
-     * INVALID   - 'list' is added to iface->tx.gc_list.
-     * Otherwise - 'super' and 'arb_group' are used.
-     */
-    union {
-        struct {
-            uct_base_ep_t         super;
-            ucs_arbiter_group_t   arb_group;
-        };
-        ucs_list_link_t           list;
-    };
-
-    uint8_t                       dci;
-    uint8_t                       flags;
-    uint16_t                      atomic_mr_offset;
-    uct_rc_fc_t                   fc;
-    uct_ib_mlx5_base_av_t         av;
+    uct_base_ep_t         super;
+    ucs_arbiter_group_t   arb_group;
+    uint8_t               dci;
+    uint8_t               atomic_mr_id;
+    uint16_t              flags;
+    uint16_t              flush_rkey_hi;
+    uct_rc_fc_t           fc;
+    uct_dc_mlx5_base_av_t av;
+    uint8_t               dci_channel_index;
 };
 
 typedef struct {
@@ -183,8 +201,18 @@ ucs_status_t uct_dc_mlx5_ep_fence(uct_ep_h tl_ep, unsigned flags);
 
 ucs_status_t uct_dc_mlx5_ep_flush(uct_ep_h tl_ep, unsigned flags, uct_completion_t *comp);
 
+ucs_status_t uct_dc_mlx5_ep_qp_to_err(uct_dc_mlx5_ep_t *ep);
+
+ucs_status_t uct_dc_mlx5_ep_invalidate(uct_ep_h tl_ep, unsigned flags);
+
 ucs_status_t uct_dc_mlx5_ep_fc_pure_grant_send(uct_dc_mlx5_ep_t *ep,
-                                               uct_dc_fc_request_t *fc_req);
+                                               uct_rc_iface_send_op_t *send_op);
+
+unsigned uct_dc_mlx5_ep_dci_release_progress(void *arg);
+
+void
+uct_dc_mlx5_ep_fc_pure_grant_send_completion(uct_rc_iface_send_op_t *send_op,
+                                             const void *resp);
 
 ucs_arbiter_cb_result_t
 uct_dc_mlx5_iface_dci_do_pending_wait(ucs_arbiter_t *arbiter,
@@ -206,11 +234,11 @@ uct_dc_mlx5_iface_dci_do_rand_pending_tx(ucs_arbiter_t *arbiter,
 
 ucs_status_t uct_dc_mlx5_ep_pending_add(uct_ep_h tl_ep, uct_pending_req_t *r,
                                         unsigned flags);
+
 void uct_dc_mlx5_ep_pending_purge(uct_ep_h tl_ep, uct_pending_purge_callback_t cb, void *arg);
 
-void uct_dc_mlx5_ep_pending_common(uct_dc_mlx5_iface_t *iface,
-                                   uct_dc_mlx5_ep_t *ep, uct_pending_req_t *r,
-                                   unsigned flags, int push_to_head);
+void uct_dc_mlx5_ep_do_pending_fc(uct_dc_mlx5_ep_t *fc_ep,
+                                  uct_dc_fc_request_t *fc_req);
 
 ucs_status_t
 uct_dc_mlx5_ep_check(uct_ep_h tl_ep, unsigned flags, uct_completion_t *comp);
@@ -232,10 +260,22 @@ static UCS_F_ALWAYS_INLINE int uct_dc_mlx5_iface_is_dci_rand(uct_dc_mlx5_iface_t
     return iface->tx.policy == UCT_DC_TX_POLICY_RAND;
 }
 
+static UCS_F_ALWAYS_INLINE int
+uct_dc_mlx5_iface_is_hw_dcs(const uct_dc_mlx5_iface_t *iface)
+{
+    return iface->tx.policy == UCT_DC_TX_POLICY_HW_DCS;
+}
+
+static UCS_F_ALWAYS_INLINE int
+uct_dc_mlx5_iface_is_dci_shared(const uct_dc_mlx5_iface_t *iface)
+{
+    return iface->tx.policy >= UCT_DC_TX_POLICY_SHARED_FIRST;
+}
+
 static UCS_F_ALWAYS_INLINE ucs_arbiter_group_t*
 uct_dc_mlx5_ep_rand_arb_group(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
 {
-    ucs_assert(uct_dc_mlx5_iface_is_dci_rand(iface) &&
+    ucs_assert(uct_dc_mlx5_iface_is_dci_shared(iface) &&
                (ep->dci != UCT_DC_MLX5_EP_NO_DCI));
     /* If DCI random policy is used, DCI is always assigned to EP */
     return &iface->tx.dcis[ep->dci].arb_group;
@@ -244,14 +284,14 @@ uct_dc_mlx5_ep_rand_arb_group(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
 static UCS_F_ALWAYS_INLINE ucs_arbiter_group_t*
 uct_dc_mlx5_ep_arb_group(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
 {
-    return (uct_dc_mlx5_iface_is_dci_rand(iface)) ?
+    return (uct_dc_mlx5_iface_is_dci_shared(iface)) ?
             uct_dc_mlx5_ep_rand_arb_group(iface, ep) : &ep->arb_group;
 }
 
 static UCS_F_ALWAYS_INLINE void
 uct_dc_mlx5_iface_dci_sched_tx(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
 {
-    if (uct_dc_mlx5_iface_is_dci_rand(iface)) {
+    if (uct_dc_mlx5_iface_is_dci_shared(iface)) {
         ucs_arbiter_group_schedule(uct_dc_mlx5_iface_tx_waitq(iface),
                                    uct_dc_mlx5_ep_rand_arb_group(iface, ep));
     } else if (uct_dc_mlx5_iface_dci_has_tx_resources(iface, ep->dci)) {
@@ -265,11 +305,12 @@ uct_dc_mlx5_ep_from_dci(uct_dc_mlx5_iface_t *iface, uint8_t dci_index)
 {
     /* Can be used with dcs* policies only, with rand policy every dci may
      * be used by many eps */
-    ucs_assert(!uct_dc_mlx5_iface_is_dci_rand(iface));
+    ucs_assert(!uct_dc_mlx5_iface_is_dci_shared(iface));
     return iface->tx.dcis[dci_index].ep;
 }
 
-void uct_dc_mlx5_ep_handle_failure(uct_dc_mlx5_ep_t *ep, void *arg,
+void uct_dc_mlx5_ep_handle_failure(uct_dc_mlx5_ep_t *ep,
+                                   struct mlx5_cqe64 *cqe,
                                    ucs_status_t status);
 
 static UCS_F_ALWAYS_INLINE ucs_status_t
@@ -279,12 +320,19 @@ uct_dc_mlx5_ep_basic_init(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
 
     if (uct_dc_mlx5_iface_is_dci_rand(iface)) {
         /* coverity[dont_call] */
-        ep->dci = rand_r(&iface->tx.rand_seed) % iface->tx.ndci;
+        ep->dci               = rand_r(&iface->tx.rand_seed) % iface->tx.ndci;
+        ep->dci_channel_index = 0;
+    } else if (uct_dc_mlx5_iface_is_hw_dcs(iface)) {
+        ucs_assertv(iface->tx.ndci == 1, "ndci=%u", iface->tx.ndci);
+        ep->dci               = 0;
+        ep->dci_channel_index = iface->tx.dcis[ep->dci].next_channel_index++ %
+                                iface->tx.num_dci_channels;
     } else {
-        ep->dci = UCT_DC_MLX5_EP_NO_DCI;
+        ep->dci               = UCT_DC_MLX5_EP_NO_DCI;
+        ep->dci_channel_index = 0;
     }
 
-    return uct_rc_fc_init(&ep->fc, iface->super.super.config.fc_wnd_size
+    return uct_rc_fc_init(&ep->fc, &iface->super.super
                           UCS_STATS_ARG(ep->super.stats));
 }
 
@@ -331,7 +379,7 @@ uct_dc_mlx5_iface_progress_pending(uct_dc_mlx5_iface_t *iface,
          * empty.
          */
         if (uct_dc_mlx5_iface_dci_can_alloc(iface, pool_index) &&
-            !uct_dc_mlx5_iface_is_dci_rand(iface)) {
+            !uct_dc_mlx5_iface_is_dci_shared(iface)) {
             ucs_arbiter_dispatch(dci_waitq, 1,
                                  uct_dc_mlx5_iface_dci_do_pending_wait, NULL);
         }
@@ -340,7 +388,6 @@ uct_dc_mlx5_iface_progress_pending(uct_dc_mlx5_iface_t *iface,
 
     } while (ucs_unlikely(!ucs_arbiter_is_empty(dci_waitq) &&
                           uct_dc_mlx5_iface_dci_can_alloc(iface, pool_index)));
-    uct_dc_mlx5_iface_check_tx(iface);
 }
 
 static inline int uct_dc_mlx5_iface_dci_ep_can_send(uct_dc_mlx5_ep_t *ep)
@@ -352,12 +399,12 @@ static inline int uct_dc_mlx5_iface_dci_ep_can_send(uct_dc_mlx5_ep_t *ep)
 }
 
 static UCS_F_ALWAYS_INLINE
-void uct_dc_mlx5_iface_schedule_dci_alloc(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
+void uct_dc_mlx5_iface_schedule_dci_alloc(uct_dc_mlx5_iface_t *iface,
+                                          uct_dc_mlx5_ep_t *ep)
 {
     ucs_arbiter_t *waitq;
 
-    /* If FC window is empty the group will be scheduled when
-     * grant is received */
+    /* If FC window is empty the group will be scheduled when grant is received */
     if (uct_rc_fc_has_resources(&iface->super.super, &ep->fc)) {
         waitq = uct_dc_mlx5_iface_dci_waitq(iface, uct_dc_mlx5_ep_pool_index(ep));
         ucs_arbiter_group_schedule(waitq, &ep->arb_group);
@@ -367,8 +414,10 @@ void uct_dc_mlx5_iface_schedule_dci_alloc(uct_dc_mlx5_iface_t *iface, uct_dc_mlx
 static UCS_F_ALWAYS_INLINE uint8_t
 uct_dc_mlx5_iface_dci_pool_index(uct_dc_mlx5_iface_t *iface, uint8_t dci_index)
 {
-    ucs_assert(iface->tx.dcis[dci_index].pool_index <
-               UCT_DC_MLX5_IFACE_MAX_DCI_POOLS);
+    ucs_assertv(iface->tx.dcis[dci_index].pool_index <
+                        UCT_DC_MLX5_IFACE_MAX_DCI_POOLS,
+                "pool_index=%d dci_index=%d",
+                iface->tx.dcis[dci_index].pool_index, dci_index);
     return iface->tx.dcis[dci_index].pool_index;
 }
 
@@ -378,12 +427,15 @@ uct_dc_mlx5_iface_dci_release(uct_dc_mlx5_iface_t *iface, uint8_t dci_index)
     uint8_t pool_index           = uct_dc_mlx5_iface_dci_pool_index(iface,
                                                                     dci_index);
     uct_dc_mlx5_dci_pool_t *pool = &iface->tx.dci_pool[pool_index];
+    uct_dc_mlx5_ep_t *ep         = uct_dc_mlx5_ep_from_dci(iface, dci_index);
+
+    ucs_debug("iface %p: release dci %d from ep %p", iface, dci_index, ep);
 
     pool->stack_top--;
+    ucs_assertv(pool->stack_top >= 0, "dci pool underflow, stack_top=%d",
+                (int)pool->stack_top);
+    ucs_assert(pool->release_stack_top < pool->stack_top);
     pool->stack[pool->stack_top] = dci_index;
-#if UCS_ENABLE_ASSERT
-    iface->tx.dcis[dci_index].flags = 0;
-#endif
 }
 
 /* Release endpoint's DCI below, if the endpoint does not have outstanding
@@ -395,7 +447,9 @@ uct_dc_mlx5_iface_dci_put(uct_dc_mlx5_iface_t *iface, uint8_t dci_index)
     ucs_arbiter_t *waitq;
     uint8_t pool_index;
 
-    if (uct_dc_mlx5_iface_is_dci_rand(iface) ||
+    ucs_assert(dci_index != UCT_DC_MLX5_EP_NO_DCI);
+
+    if (uct_dc_mlx5_iface_is_dci_shared(iface) ||
         uct_dc_mlx5_iface_is_dci_keepalive(iface, dci_index)) {
         return;
     }
@@ -432,10 +486,9 @@ uct_dc_mlx5_iface_dci_put(uct_dc_mlx5_iface_t *iface, uint8_t dci_index)
 
     uct_dc_mlx5_iface_dci_release(iface, dci_index);
 
-    ucs_assert(uct_dc_mlx5_ep_from_dci(iface, dci_index)->dci !=
-               UCT_DC_MLX5_EP_NO_DCI);
-    ep->dci    = UCT_DC_MLX5_EP_NO_DCI;
-    ep->flags &= ~UCT_DC_MLX5_EP_FLAG_TX_WAIT;
+    ucs_assert(ep->dci != UCT_DC_MLX5_EP_NO_DCI);
+    ep->dci                      = UCT_DC_MLX5_EP_NO_DCI;
+    ep->flags                   &= ~UCT_DC_MLX5_EP_FLAG_TX_WAIT;
     iface->tx.dcis[dci_index].ep = NULL;
 
     /* it is possible that dci is released while ep still has scheduled pending ops.
@@ -454,39 +507,65 @@ static inline void uct_dc_mlx5_iface_dci_alloc(uct_dc_mlx5_iface_t *iface, uct_d
     uint8_t pool_index           = uct_dc_mlx5_ep_pool_index(ep);
     uct_dc_mlx5_dci_pool_t *pool = &iface->tx.dci_pool[pool_index];
 
-    ucs_assert(!uct_dc_mlx5_iface_is_dci_rand(iface));
+    ucs_assert(!uct_dc_mlx5_iface_is_dci_shared(iface));
+    ucs_assert(pool->release_stack_top < pool->stack_top);
     ep->dci = pool->stack[pool->stack_top];
     ucs_assert(ep->dci >= (iface->tx.ndci * pool_index));
     ucs_assert(ep->dci < (iface->tx.ndci * (pool_index + 1)));
     ucs_assert(uct_dc_mlx5_ep_from_dci(iface, ep->dci) == NULL);
-    ucs_assert(iface->tx.dcis[ep->dci].flags == 0);
     iface->tx.dcis[ep->dci].ep = ep;
     pool->stack_top++;
-}
-
-static inline void uct_dc_mlx5_iface_dci_free(uct_dc_mlx5_iface_t *iface,
-                                              uct_dc_mlx5_ep_t *ep)
-{
-    uint8_t dci_index;
-
-    if (uct_dc_mlx5_iface_is_dci_rand(iface)) {
-        return;
+    if (ep->flags & UCT_DC_MLX5_EP_FLAG_INVALIDATED) {
+        (void)uct_dc_mlx5_ep_qp_to_err(ep);
     }
 
-    dci_index = ep->dci;
+    ucs_assertv(pool->stack_top > 0, "dci pool overflow, stack_top=%d",
+                (int)pool->stack_top);
+    ucs_debug("iface %p: allocate dci %d for ep %p", iface, ep->dci, ep);
+}
 
+static UCS_F_ALWAYS_INLINE void
+uct_dc_mlx5_iface_dci_schedule_release(uct_dc_mlx5_iface_t *iface, uint8_t dci)
+{
+    uint8_t pool_index = uct_dc_mlx5_iface_dci_pool_index(iface, dci);
+    uint8_t stack_top;
+
+    ucs_assert(!uct_dc_mlx5_iface_is_dci_shared(iface));
+
+    /* adding current DCI into release stack and mark pool for
+     * processing, see details in @ref uct_dc_mlx5_dci_pool_t description */
+    stack_top = ++iface->tx.dci_pool[pool_index].release_stack_top;
+    ucs_assert(stack_top < iface->tx.dci_pool[pool_index].stack_top);
+
+    iface->tx.dci_pool_release_bitmap              |= UCS_BIT(pool_index);
+    iface->tx.dci_pool[pool_index].stack[stack_top] = dci;
+
+    uct_worker_progress_register_safe(
+            &iface->super.super.super.super.worker->super,
+            uct_dc_mlx5_ep_dci_release_progress, iface,
+            UCS_CALLBACKQ_FLAG_ONESHOT, &iface->tx.dci_release_prog_id);
+}
+
+static UCS_F_ALWAYS_INLINE int
+uct_dc_mlx5_iface_dci_detach(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
+{
+    uint8_t dci_index = ep->dci;
+
+    ucs_assert(!uct_dc_mlx5_iface_is_dci_shared(iface));
     ucs_assert(dci_index != UCT_DC_MLX5_EP_NO_DCI);
     ucs_assert(iface->tx.dci_pool[uct_dc_mlx5_ep_pool_index(ep)].stack_top > 0);
 
     if (uct_dc_mlx5_iface_dci_has_outstanding(iface, dci_index)) {
-        return;
+        return 0;
     }
 
-    uct_dc_mlx5_iface_dci_release(iface, dci_index);
-
-    iface->tx.dcis[dci_index].ep = NULL;
     ep->dci                      = UCT_DC_MLX5_EP_NO_DCI;
     ep->flags                   &= ~UCT_DC_MLX5_EP_FLAG_TX_WAIT;
+    iface->tx.dcis[dci_index].ep = NULL;
+
+    uct_dc_mlx5_iface_dci_schedule_release(iface, dci_index);
+
+    return 1;
 }
 
 static UCS_F_ALWAYS_INLINE ucs_status_t
@@ -499,7 +578,7 @@ uct_dc_mlx5_iface_dci_get(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
 
     ucs_assert(!iface->super.super.config.tx_moderation);
 
-    if (uct_dc_mlx5_iface_is_dci_rand(iface)) {
+    if (uct_dc_mlx5_iface_is_dci_shared(iface)) {
         /* Silence Coverity - in random policy the endpoint always has an
          * assigned DCI */
         ucs_assert(ep->dci != UCT_DC_MLX5_EP_NO_DCI);
@@ -543,6 +622,9 @@ uct_dc_mlx5_iface_dci_get(uct_dc_mlx5_iface_t *iface, uct_dc_mlx5_ep_t *ep)
     }
 
     if (uct_dc_mlx5_iface_dci_can_alloc(iface, pool_index)) {
+        waitq = uct_dc_mlx5_iface_dci_waitq(iface, pool_index);
+        ucs_assert(ucs_arbiter_is_empty(waitq));
+
         uct_dc_mlx5_iface_dci_alloc(iface, ep);
         return UCS_OK;
     }
@@ -562,11 +644,6 @@ static inline struct mlx5_grh_av *uct_dc_mlx5_ep_get_grh(uct_dc_mlx5_ep_t *ep)
 }
 
 
-#define UCT_DC_MLX5_TXQP_DECL(_txqp, _txwq) \
-    uct_rc_txqp_t UCS_V_UNUSED *_txqp; \
-    uct_ib_mlx5_txwq_t UCS_V_UNUSED *_txwq;
-
-
 #define UCT_DC_MLX5_CHECK_DCI_RES(_iface, _ep) \
     { \
         ucs_status_t _status = uct_dc_mlx5_iface_dci_get(_iface, _ep); \
@@ -623,7 +700,7 @@ static inline struct mlx5_grh_av *uct_dc_mlx5_ep_get_grh(uct_dc_mlx5_ep_t *ep)
                 return _status; \
             } \
         } \
-        if (!uct_dc_mlx5_iface_is_dci_rand(_iface)) { \
+        if (!uct_dc_mlx5_iface_is_dci_shared(_iface)) { \
             uct_rc_iface_check_pending(&(_iface)->super.super, \
                                        &(_ep)->arb_group); \
         } \
