@@ -1,5 +1,5 @@
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2001-2015.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2015. ALL RIGHTS RESERVED.
 *
 * See file LICENSE for terms.
 */
@@ -11,6 +11,8 @@
 #include "ib_mlx5_log.h"
 
 #include <uct/ib/base/ib_device.h>
+#include <uct/ib/mlx5/ib_mlx5.inl>
+#include <uct/ib/rc/accel/rc_mlx5_common.h>
 #include <string.h>
 
 
@@ -19,6 +21,10 @@ static void uct_ib_mlx5_wqe_dump(uct_ib_iface_t *iface, void *wqe, void *qstart,
                                  uct_log_data_dump_func_t packet_dump_cb,
                                  char *buffer, size_t max, uct_ib_log_sge_t *log_sge);
 
+static void uct_ib_mlx5_resp_error_dump(uct_ib_iface_t *iface,
+                                        uct_ib_mlx5_err_cqe_t *ecqe, char *buf,
+                                        size_t max);
+
 static const char *uct_ib_mlx5_cqe_err_opcode(uct_ib_mlx5_err_cqe_t *ecqe)
 {
     uint8_t wqe_err_opcode = ntohl(ecqe->s_wqe_opcode_qpn) >> 24;
@@ -71,21 +77,17 @@ ucs_status_t uct_ib_mlx5_completion_with_err(uct_ib_iface_t *iface,
                                              uct_ib_mlx5_txwq_t *txwq,
                                              ucs_log_level_t log_level)
 {
-    ucs_status_t status = UCS_ERR_IO_ERROR;
-    char err_info[256]  = {};
-    char wqe_info[256]  = {};
-    char peer_info[128] = {};
-    uint16_t wqe_index;
-    uint32_t qp_num;
+    ucs_status_t err_status = UCS_ERR_IO_ERROR;
+    char err_info[256]      = {};
+    char wqe_info[256]      = {};
+    char peer_info[128]     = {};
+    uint16_t pi             = ntohs(ecqe->wqe_counter);
+    uint32_t qp_num         = ntohl(ecqe->s_wqe_opcode_qpn) &
+                              UCS_MASK(UCT_IB_QPN_ORDER);
     void *wqe;
-    unsigned dest_qpn;
     struct ibv_ah_attr ah_attr;
-
-    wqe_index = ntohs(ecqe->wqe_counter);
-    qp_num    = ntohl(ecqe->s_wqe_opcode_qpn) & UCS_MASK(UCT_IB_QPN_ORDER);
-    if (txwq != NULL) {
-        wqe_index %= UCS_PTR_BYTE_DIFF(txwq->qstart, txwq->qend) / MLX5_SEND_WQE_BB;
-    }
+    unsigned dest_qpn;
+    ucs_status_t status;
 
     switch (ecqe->syndrome) {
     case MLX5_CQE_SYNDROME_LOCAL_LENGTH_ERR:
@@ -100,8 +102,8 @@ ucs_status_t uct_ib_mlx5_completion_with_err(uct_ib_iface_t *iface,
     case MLX5_CQE_SYNDROME_WR_FLUSH_ERR:
         snprintf(err_info, sizeof(err_info),
                  "WR flushed because QP in error state");
-        log_level = UCS_LOG_LEVEL_TRACE;
-        status    = UCS_ERR_CANCELED;
+        log_level  = UCS_LOG_LEVEL_TRACE;
+        err_status = UCS_ERR_CANCELED;
         break;
     case MLX5_CQE_SYNDROME_MW_BIND_ERR:
         snprintf(err_info, sizeof(err_info), "Memory window bind");
@@ -117,23 +119,23 @@ ucs_status_t uct_ib_mlx5_completion_with_err(uct_ib_iface_t *iface,
         break;
     case MLX5_CQE_SYNDROME_REMOTE_ACCESS_ERR:
         snprintf(err_info, sizeof(err_info), "Remote access");
-        status = UCS_ERR_CONNECTION_RESET;
+        err_status = UCS_ERR_CONNECTION_RESET;
         break;
     case MLX5_CQE_SYNDROME_REMOTE_OP_ERR:
         snprintf(err_info, sizeof(err_info), "Remote OP");
-        status = UCS_ERR_CONNECTION_RESET;
+        err_status = UCS_ERR_CONNECTION_RESET;
         break;
     case MLX5_CQE_SYNDROME_TRANSPORT_RETRY_EXC_ERR:
         snprintf(err_info, sizeof(err_info), "Transport retry count exceeded");
-        status = UCS_ERR_ENDPOINT_TIMEOUT;
+        err_status = UCS_ERR_ENDPOINT_TIMEOUT;
         break;
     case MLX5_CQE_SYNDROME_RNR_RETRY_EXC_ERR:
         snprintf(err_info, sizeof(err_info), "Receive-no-ready retry count exceeded");
-        status = UCS_ERR_ENDPOINT_TIMEOUT;
+        err_status = UCS_ERR_ENDPOINT_TIMEOUT;
         break;
     case MLX5_CQE_SYNDROME_REMOTE_ABORTED_ERR:
         snprintf(err_info, sizeof(err_info), "Remote side aborted");
-        status = UCS_ERR_ENDPOINT_TIMEOUT;
+        err_status = UCS_ERR_ENDPOINT_TIMEOUT;
         break;
     default:
         snprintf(err_info, sizeof(err_info), "Generic");
@@ -145,7 +147,7 @@ ucs_status_t uct_ib_mlx5_completion_with_err(uct_ib_iface_t *iface,
     }
 
     if ((txwq != NULL) && ((ecqe->op_own >> 4) == MLX5_CQE_REQ_ERR)) {
-        wqe = UCS_PTR_BYTE_OFFSET(txwq->qstart, MLX5_SEND_WQE_BB * wqe_index);
+        wqe = uct_ib_mlx5_txwq_get_wqe(txwq, pi);
         uct_ib_mlx5_wqe_dump(iface, wqe, txwq->qstart, txwq->qend, INT_MAX, 0,
                              NULL, wqe_info, sizeof(wqe_info) - 1, NULL);
 
@@ -159,22 +161,23 @@ ucs_status_t uct_ib_mlx5_completion_with_err(uct_ib_iface_t *iface,
                                              peer_info, sizeof(peer_info));
             }
         }
+    } else if ((ecqe->op_own >> 4) == MLX5_CQE_RESP_ERR) {
+        uct_ib_mlx5_resp_error_dump(iface, ecqe, err_info, sizeof(err_info));
     } else {
         snprintf(wqe_info, sizeof(wqe_info) - 1, "opcode %s",
                  uct_ib_mlx5_cqe_err_opcode(ecqe));
     }
 
     ucs_log(log_level,
-            "%s on "UCT_IB_IFACE_FMT"/%s (synd 0x%x vend 0x%x hw_synd %d/%d)\n"
+            "%s on " UCT_IB_IFACE_FMT " (synd 0x%x vend 0x%x hw_synd %d/%d)\n"
             "%s QP 0x%x wqe[%d]: %s %s",
-            err_info, UCT_IB_IFACE_ARG(iface),
-            uct_ib_iface_is_roce(iface) ? "RoCE" : "IB",
-            ecqe->syndrome, ecqe->vendor_err_synd, ecqe->hw_synd_type >> 4,
-            ecqe->hw_err_synd, uct_ib_qp_type_str(iface->config.qp_type),
-            qp_num, wqe_index, wqe_info, peer_info);
+            err_info, UCT_IB_IFACE_ARG(iface), ecqe->syndrome,
+            ecqe->vendor_err_synd, ecqe->hw_synd_type >> 4, ecqe->hw_err_synd,
+            uct_ib_qp_type_str(iface->config.qp_type), qp_num, pi, wqe_info,
+            peer_info);
 
 out:
-    return status;
+    return err_status;
 }
 
 static unsigned uct_ib_mlx5_parse_dseg(void **dseg_p, void *qstart, void *qend,
@@ -413,6 +416,42 @@ static void uct_ib_mlx5_wqe_dump(uct_ib_iface_t *iface, void *wqe, void *qstart,
     }
 }
 
+static void uct_ib_mlx5_resp_error_dump(uct_ib_iface_t *iface,
+                                        uct_ib_mlx5_err_cqe_t *ecqe,
+                                        char *buffer, size_t max)
+{
+    uct_rc_mlx5_iface_common_t *mlx5_iface = ucs_derived_of(iface,
+                                                            uct_rc_mlx5_iface_common_t);
+    char *ends                             = buffer + max;
+    uct_ib_mlx5_srq_t *srq                 = &mlx5_iface->rx.srq;
+    uct_ib_mlx5_srq_seg_t *seg             = uct_ib_mlx5_srq_get_wqe(srq,
+                                                                     ecqe->wqe_counter);
+    int i;
+
+    snprintf(buffer, ends - buffer, "strides %d%s next wqe %u desc %p",
+             seg->srq.strides, seg->srq.free ? " F" : "",
+             htons(seg->srq.next_wqe_index), seg->srq.desc);
+    buffer += strlen(buffer);
+
+    if (seg->srq.strides > 1) {
+        snprintf(buffer, ends - buffer, " ptr_mask %d", seg->srq.ptr_mask);
+        buffer += strlen(buffer);
+    }
+
+    for (i = 0; i < mlx5_iface->tm.mp.num_strides; i++) {
+        snprintf(buffer, ends - buffer,
+                 " [seg %d bytes %d lkey 0x%x addr 0x%lx]", i,
+                 htobe32(seg->dptr[i].byte_count), htobe32(seg->dptr[i].lkey),
+                 htobe64(seg->dptr[i].addr));
+        buffer += strlen(buffer);
+
+        if (buffer == ends) {
+            break;
+        }
+    }
+}
+
+
 void __uct_ib_mlx5_log_tx(const char *file, int line, const char *function,
                           uct_ib_iface_t *iface, void *wqe, void *qstart,
                           void *qend, int max_sge, uct_ib_log_sge_t *log_sge,
@@ -451,6 +490,7 @@ void uct_ib_mlx5_av_dump(char *buf, size_t max,
     p    = buf;
     endp = buf + max - 1;
 
+    /* cppcheck-suppress[uninitvar] */
     snprintf(p, endp - p, " [rqpn 0x%x",
              ntohl(base_av->dqp_dct & ~UCT_IB_MLX5_EXTENDED_UD_AV));
     p += strlen(p);
@@ -460,6 +500,7 @@ void uct_ib_mlx5_av_dump(char *buf, size_t max,
         p += strlen(p);
     }
 
+    /* cppcheck-suppress[uninitvar] */
     if (base_av->dqp_dct & UCT_IB_MLX5_EXTENDED_UD_AV) {
         if (is_eth || (grh_av->grh_gid_fl & UCT_IB_MLX5_AV_GRH_PRESENT)) {
             if (is_eth) {
