@@ -1,5 +1,5 @@
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2001-2019.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2019. ALL RIGHTS RESERVED.
 * Copyright (C) UT-Battelle, LLC. 2015. ALL RIGHTS RESERVED.
 *
 * See file LICENSE for terms.
@@ -11,6 +11,8 @@
 extern "C" {
 #include <ucp/core/ucp_request.h>
 #include <ucp/core/ucp_types.h>
+#include <ucp/rndv/proto_rndv.h>
+#include <ucp/core/ucp_ep.inl>
 }
 
 using namespace ucs; /* For vector<char> serialization */
@@ -36,6 +38,10 @@ public:
             modify_config("PROTO_ENABLE", "y");
             modify_config("MAX_EAGER_LANES", "2");
         } else {
+            if (RUNNING_ON_VALGRIND) {
+                skip_external_protov2();
+            }
+
             // TODO:
             // 1. test offload and offload MP as different variants
             // 2. Enable offload for new protocols as well when it is fully
@@ -546,6 +552,7 @@ public:
         RNDV_SCHEME_PUT_ZCOPY,
         RNDV_SCHEME_GET_ZCOPY,
         RNDV_SCHEME_LAST,
+        RNDV_GET_ZCOPY_MANY_LANES,
         PUT_ZCOPY_FLUSH = ENABLE_PROTO << 1
     };
 
@@ -557,6 +564,9 @@ public:
         modify_config("RNDV_THRESH", "0");
         modify_config("RNDV_SCHEME", rndv_schemes[rndv_scheme()]);
         modify_config("RNDV_PUT_FORCE_FLUSH", force_flush() ? "y" : "n");
+        if (get_variant_value() & RNDV_GET_ZCOPY_MANY_LANES) {
+            modify_config("MAX_RNDV_LANES", "3");
+        }
         test_ucp_tag_match::init();
     }
 
@@ -574,6 +584,11 @@ public:
                                RNDV_SCHEME_PUT_ZCOPY | ENABLE_PROTO |
                                        PUT_ZCOPY_FLUSH,
                                "rndv_put_flush,proto");
+        // Add variant to check the RNDV lanes weight correctness for many lanes number
+        add_variant_with_value(variants, get_ctx_params(),
+                               RNDV_SCHEME_GET_ZCOPY | ENABLE_PROTO |
+                                       RNDV_GET_ZCOPY_MANY_LANES,
+                               "rndv_get_zcopy,proto,many_lanes");
     }
 
 protected:
@@ -818,8 +833,10 @@ UCS_TEST_P(test_ucp_tag_match_rndv, req_exp_auto_thresh, "RNDV_THRESH=auto") {
 }
 
 UCS_TEST_P(test_ucp_tag_match_rndv, exp_huge_mix) {
-    const std::vector<size_t> sizes = {1000, 2000, 8000, 2500ul * UCS_MBYTE,
-                                       UCS_GBYTE + 32};
+    const std::vector<size_t> sizes = {1000, 2000, 8000,
+                                       ucs::limit_buffer_size(2500ul *
+                                                              UCS_MBYTE),
+                                       ucs::limit_buffer_size(UCS_GBYTE + 32)};
 
     /* small sizes should warm-up tag cache */
     for (auto c_size : sizes) {
@@ -854,8 +871,9 @@ UCS_TEST_P(test_ucp_tag_match_rndv, exp_huge_mix) {
 
 UCS_TEST_P(test_ucp_tag_match_rndv, bidir_multi_exp_post)
 {
-    const size_t sizes[] = { 8 * UCS_KBYTE, 128 * UCS_KBYTE, 512 * UCS_KBYTE,
-                             8 * UCS_MBYTE, 128 * UCS_MBYTE, 512 * UCS_MBYTE };
+    const size_t sizes[] = {8 * UCS_KBYTE, 128 * UCS_KBYTE, 512 * UCS_KBYTE,
+                            8 * UCS_MBYTE, 128 * UCS_MBYTE, 512 * UCS_MBYTE};
+    const size_t max_total_size = limit_buffer_size();
 
     receiver().connect(&sender(), get_ep_params());
 
@@ -865,6 +883,15 @@ UCS_TEST_P(test_ucp_tag_match_rndv, bidir_multi_exp_post)
                             ucs::test_time_multiplier();
         const size_t count = ucs_max((size_t)(5000.0 / sqrt(sizes[i]) /
                                               ucs::test_time_multiplier()), 3lu);
+
+        size_t total_size = size * count * 2;
+        if (total_size > max_total_size) {
+            UCS_TEST_MESSAGE << "Total size (" << total_size
+                             << ") exceeds limit (" << max_total_size
+                             << "), stopping the test";
+            break;
+        }
+
         std::vector<request*> sreqs;
         std::vector<request*> rreqs;
         std::vector<std::vector<char> > sbufs;
@@ -918,3 +945,87 @@ UCS_TEST_P(test_ucp_tag_match_rndv, bidir_multi_exp_post)
 
 UCP_INSTANTIATE_TEST_CASE(test_ucp_tag_match_rndv)
 UCP_INSTANTIATE_TEST_CASE_TLS(test_ucp_tag_match_rndv, mm_tcp, "posix,sysv,tcp")
+
+class test_ucp_tag_match_rndv_align : public test_ucp_tag_match_rndv {
+public:
+    enum {
+        RNDV_SCHEME_GET_ZCOPY,
+        RNDV_SCHEME_PUT_ZCOPY,
+        RNDV_SCHEME_LAST
+    };
+
+    static const std::string rndv_schemes[];
+
+    void init()
+    {
+        modify_config("RNDV_THRESH", "0");
+        modify_config("RNDV_ALIGN_THRESH", "0");
+        modify_config("RNDV_SCHEME", rndv_schemes[rndv_scheme()]);
+        test_ucp_tag_match::init();
+    }
+
+    static void get_test_variants(std::vector<ucp_test_variant>& variants)
+    {
+        for (int rndv_scheme = 0; rndv_scheme < RNDV_SCHEME_LAST; ++rndv_scheme) {
+            add_variant_with_value(variants, get_ctx_params(),
+                                   rndv_scheme | ENABLE_PROTO,
+                                   rndv_schemes[rndv_scheme] + ",proto");
+        }
+    }
+protected:
+    size_t min_frag()
+    {
+        if (rndv_scheme() == RNDV_SCHEME_GET_ZCOPY) {
+            return ucp_ep_config(receiver().ep())->rndv.get_zcopy.min;
+        } else if (rndv_scheme() == RNDV_SCHEME_PUT_ZCOPY) {
+            return ucp_ep_config(receiver().ep())->rndv.put_zcopy.min;
+        }
+        ucs_assert_always(0);
+        return 0;
+    }
+};
+
+const std::string test_ucp_tag_match_rndv_align::rndv_schemes[] = { "get_zcopy",
+                                                                    "put_zcopy"};
+
+UCS_TEST_P(test_ucp_tag_match_rndv_align, recv_align)
+{
+    size_t recv_offsets[4];
+    size_t sizes[] = { UCS_SYS_PCI_MAX_PAYLOAD, 139 * UCS_KBYTE, 2000,
+                       8000, ucs::limit_buffer_size(2500ul * UCS_MBYTE) };
+
+    receiver().connect(&sender(), get_ep_params());
+
+    recv_offsets[0] = 0;
+    recv_offsets[1] = min_frag();
+    recv_offsets[2] = UCS_SYS_PCI_MAX_PAYLOAD - 1;
+    recv_offsets[3] = (ucs::rand() % UCS_SYS_PCI_MAX_PAYLOAD) + 1;
+
+    for (unsigned i = 0; i < ucs_static_array_size(recv_offsets); i++) {
+        for (unsigned j = 0; j < ucs_static_array_size(sizes); j++) {
+            const size_t recv_offset = recv_offsets[i];
+            const size_t size        = sizes[j] / ucs::test_time_multiplier() /
+                                        ucs::test_time_multiplier();
+            std::vector<char> sendbuf(size, 0);
+            std::vector<char> recvbuf(size + recv_offset +
+                                      UCS_SYS_PCI_MAX_PAYLOAD, 0);
+            size_t offset = UCS_SYS_PCI_MAX_PAYLOAD -
+                ((size_t)&recvbuf[0] % UCS_SYS_PCI_MAX_PAYLOAD) + recv_offset;
+            request *ucx_req = recv_nb(&recvbuf[offset], size, DATATYPE, 0x1337, 0xffff);
+
+            ASSERT_FALSE(UCS_PTR_IS_ERR(ucx_req));
+            ASSERT_NE(nullptr, ucx_req); /* Couldn't be completed because didn't send yet */
+
+            ucs::fill_random(sendbuf);
+            send_b(&sendbuf[0], size, DATATYPE, 0x111337);
+            wait(ucx_req);
+            request_free(ucx_req);
+
+            std::vector<char> recvbuf_offs(&recvbuf[offset],
+                                           &recvbuf[offset+size]);
+            EXPECT_EQ(sendbuf, recvbuf_offs);
+        }
+    }
+}
+
+UCP_INSTANTIATE_TEST_CASE(test_ucp_tag_match_rndv_align)
