@@ -1,6 +1,6 @@
 
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2001-2014.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2014. ALL RIGHTS RESERVED.
 * Copyright (C) UT-Battelle, LLC. 2014-2015. ALL RIGHTS RESERVED.
 * See file LICENSE for terms.
 */
@@ -94,8 +94,8 @@ public:
         size_t size = ucs_max(min, ucs_min(64ul, max));
         uint8_t mem_type_index;
 
-        ucs_assert(sender().md_attr().cap.access_mem_types != 0);
-        mem_type_index = ucs_ffs64(sender().md_attr().cap.access_mem_types);
+        ucs_assert(sender().md_attr().access_mem_types != 0);
+        mem_type_index = ucs_ffs64(sender().md_attr().access_mem_types);
 
         lbuf = new mapped_buffer(size, 0, sender(), 0, (ucs_memory_type_t)mem_type_index);
         rbuf = new mapped_buffer(size, 0, receiver(), 0, (ucs_memory_type_t)mem_type_index);
