@@ -28,7 +28,7 @@
 static int unpack_sub_hvector(const void *inbuf, uintptr_t insize, void *outbuf,
                               yaksi_type_s * type, uintptr_t outoffset,
                               uintptr_t * actual_unpack_bytes, yaksi_info_s * info,
-                              yaksi_request_s * request)
+                              yaksa_op_t op, yaksi_request_s * request)
 {
     int rc = YAKSA_SUCCESS;
 
@@ -60,14 +60,17 @@ static int unpack_sub_hvector(const void *inbuf, uintptr_t insize, void *outbuf,
         uintptr_t tmp_actual_unpack_bytes;
         rc = yaksi_iunpack(sbuf, tmp_unpack_bytes, dbuf, type->u.hvector.blocklength,
                            type->u.hvector.child, remoffset, &tmp_actual_unpack_bytes, info,
-                           request);
+                           op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         rem_unpack_bytes -= tmp_actual_unpack_bytes;
         *actual_unpack_bytes += tmp_actual_unpack_bytes;
 
-        if (rem_unpack_bytes == 0 || tmp_actual_unpack_bytes == 0) {
-            /* if we are out of unpack buffer space, return */
+        if (rem_unpack_bytes == 0 ||
+            tmp_actual_unpack_bytes <
+            type->u.hvector.child->size * type->u.hvector.blocklength - remoffset) {
+            /* if we are out of unpack buffer space or if we could not
+             * unpack fully, return */
             goto fn_exit;
         }
 
@@ -82,7 +85,7 @@ static int unpack_sub_hvector(const void *inbuf, uintptr_t insize, void *outbuf,
     numblocks = rem_unpack_bytes / bytes_in_block;
     for (int i = 0; i < numblocks; i++) {
         rc = yaksi_iunpack_backend(sbuf, dbuf, type->u.hvector.blocklength, type->u.hvector.child,
-                                   info, request);
+                                   info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         rem_unpack_bytes -= bytes_in_block;
@@ -99,7 +102,7 @@ static int unpack_sub_hvector(const void *inbuf, uintptr_t insize, void *outbuf,
 
         rc = yaksi_iunpack(sbuf, rem_unpack_bytes, dbuf, type->u.hvector.blocklength,
                            type->u.hvector.child, remoffset, &tmp_actual_unpack_bytes, info,
-                           request);
+                           op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         *actual_unpack_bytes += tmp_actual_unpack_bytes;
@@ -114,7 +117,7 @@ static int unpack_sub_hvector(const void *inbuf, uintptr_t insize, void *outbuf,
 static int unpack_sub_blkhindx(const void *inbuf, uintptr_t insize, void *outbuf,
                                yaksi_type_s * type, uintptr_t outoffset,
                                uintptr_t * actual_unpack_bytes, yaksi_info_s * info,
-                               yaksi_request_s * request)
+                               yaksa_op_t op, yaksi_request_s * request)
 {
     int rc = YAKSA_SUCCESS;
 
@@ -148,15 +151,18 @@ static int unpack_sub_blkhindx(const void *inbuf, uintptr_t insize, void *outbuf
         dbuf = (char *) outbuf + type->u.blkhindx.array_of_displs[blockid++];
         rc = yaksi_iunpack(sbuf, tmp_unpack_bytes, dbuf, type->u.blkhindx.blocklength,
                            type->u.blkhindx.child, remoffset, &tmp_actual_unpack_bytes, info,
-                           request);
+                           op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
         sbuf += tmp_actual_unpack_bytes;
 
         rem_unpack_bytes -= tmp_actual_unpack_bytes;
         *actual_unpack_bytes += tmp_actual_unpack_bytes;
 
-        if (rem_unpack_bytes == 0 || tmp_actual_unpack_bytes == 0) {
-            /* if we are out of unpack buffer space, return */
+        if (rem_unpack_bytes == 0 ||
+            tmp_actual_unpack_bytes <
+            type->u.blkhindx.child->size * type->u.blkhindx.blocklength - remoffset) {
+            /* if we are out of unpack buffer space or if we could not
+             * unpack fully, return */
             goto fn_exit;
         }
 
@@ -170,7 +176,7 @@ static int unpack_sub_blkhindx(const void *inbuf, uintptr_t insize, void *outbuf
     for (int i = 0; i < numblocks; i++) {
         dbuf = (char *) outbuf + type->u.blkhindx.array_of_displs[blockid++];
         rc = yaksi_iunpack_backend(sbuf, dbuf, type->u.blkhindx.blocklength, type->u.blkhindx.child,
-                                   info, request);
+                                   info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
         sbuf += bytes_in_block;
 
@@ -186,7 +192,7 @@ static int unpack_sub_blkhindx(const void *inbuf, uintptr_t insize, void *outbuf
         dbuf = (char *) outbuf + type->u.blkhindx.array_of_displs[blockid++];
         rc = yaksi_iunpack(sbuf, rem_unpack_bytes, dbuf, type->u.blkhindx.blocklength,
                            type->u.blkhindx.child, remoffset, &tmp_actual_unpack_bytes, info,
-                           request);
+                           op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         sbuf += tmp_actual_unpack_bytes;
@@ -202,7 +208,7 @@ static int unpack_sub_blkhindx(const void *inbuf, uintptr_t insize, void *outbuf
 static int unpack_sub_hindexed(const void *inbuf, uintptr_t insize, void *outbuf,
                                yaksi_type_s * type, uintptr_t outoffset,
                                uintptr_t * actual_unpack_bytes, yaksi_info_s * info,
-                               yaksi_request_s * request)
+                               yaksa_op_t op, yaksi_request_s * request)
 {
     int rc = YAKSA_SUCCESS;
 
@@ -246,15 +252,18 @@ static int unpack_sub_hindexed(const void *inbuf, uintptr_t insize, void *outbuf
         dbuf = (char *) outbuf + type->u.hindexed.array_of_displs[blockid];
         rc = yaksi_iunpack(sbuf, tmp_unpack_bytes, dbuf,
                            type->u.hindexed.array_of_blocklengths[blockid], type->u.hindexed.child,
-                           remoffset, &tmp_actual_unpack_bytes, info, request);
+                           remoffset, &tmp_actual_unpack_bytes, info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
         sbuf += tmp_actual_unpack_bytes;
 
         rem_unpack_bytes -= tmp_actual_unpack_bytes;
         *actual_unpack_bytes += tmp_actual_unpack_bytes;
 
-        if (rem_unpack_bytes == 0 || tmp_actual_unpack_bytes == 0) {
-            /* if we are out of unpack buffer space, return */
+        if (rem_unpack_bytes == 0 ||
+            tmp_actual_unpack_bytes < type->u.hindexed.child->size *
+            type->u.hindexed.array_of_blocklengths[blockid] - remoffset) {
+            /* if we are out of unpack buffer space or if we could not
+             * unpack fully, return */
             goto fn_exit;
         }
 
@@ -276,7 +285,7 @@ static int unpack_sub_hindexed(const void *inbuf, uintptr_t insize, void *outbuf
 
         dbuf = (char *) outbuf + type->u.hindexed.array_of_displs[blockid];
         rc = yaksi_iunpack_backend(sbuf, dbuf, type->u.hindexed.array_of_blocklengths[blockid],
-                                   type->u.hindexed.child, info, request);
+                                   type->u.hindexed.child, info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
         sbuf += bytes_in_block;
 
@@ -304,7 +313,7 @@ static int unpack_sub_hindexed(const void *inbuf, uintptr_t insize, void *outbuf
         dbuf = (char *) outbuf + type->u.hindexed.array_of_displs[blockid];
         rc = yaksi_iunpack(sbuf, rem_unpack_bytes, dbuf,
                            type->u.hindexed.array_of_blocklengths[blockid], type->u.hindexed.child,
-                           remoffset, &tmp_actual_unpack_bytes, info, request);
+                           remoffset, &tmp_actual_unpack_bytes, info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         sbuf += tmp_actual_unpack_bytes;
@@ -321,7 +330,7 @@ static int unpack_sub_hindexed(const void *inbuf, uintptr_t insize, void *outbuf
 
 static int unpack_sub_struct(const void *inbuf, uintptr_t insize, void *outbuf, yaksi_type_s * type,
                              uintptr_t outoffset, uintptr_t * actual_unpack_bytes,
-                             yaksi_info_s * info, yaksi_request_s * request)
+                             yaksi_info_s * info, yaksa_op_t op, yaksi_request_s * request)
 {
     int rc = YAKSA_SUCCESS;
 
@@ -366,15 +375,18 @@ static int unpack_sub_struct(const void *inbuf, uintptr_t insize, void *outbuf,
         rc = yaksi_iunpack(sbuf, tmp_unpack_bytes, dbuf,
                            type->u.str.array_of_blocklengths[blockid],
                            type->u.str.array_of_types[blockid], remoffset, &tmp_actual_unpack_bytes,
-                           info, request);
+                           info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         sbuf += tmp_actual_unpack_bytes;
         rem_unpack_bytes -= tmp_actual_unpack_bytes;
         *actual_unpack_bytes += tmp_actual_unpack_bytes;
 
-        if (rem_unpack_bytes == 0 || tmp_actual_unpack_bytes == 0) {
-            /* if we are out of unpack buffer space, return */
+        if (rem_unpack_bytes == 0 ||
+            tmp_actual_unpack_bytes < type->u.str.array_of_types[blockid]->size *
+            type->u.str.array_of_blocklengths[blockid] - remoffset) {
+            /* if we are out of unpack buffer space or if we could not
+             * unpack fully, return */
             goto fn_exit;
         }
 
@@ -396,7 +408,7 @@ static int unpack_sub_struct(const void *inbuf, uintptr_t insize, void *outbuf,
 
         dbuf = (char *) outbuf + type->u.str.array_of_displs[blockid];
         rc = yaksi_iunpack_backend(sbuf, dbuf, type->u.str.array_of_blocklengths[blockid],
-                                   type->u.str.array_of_types[blockid], info, request);
+                                   type->u.str.array_of_types[blockid], info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
         sbuf += bytes_in_block;
 
@@ -426,7 +438,7 @@ static int unpack_sub_struct(const void *inbuf, uintptr_t insize, void *outbuf,
         rc = yaksi_iunpack(sbuf, rem_unpack_bytes, dbuf,
                            type->u.str.array_of_blocklengths[blockid],
                            type->u.str.array_of_types[blockid], remoffset, &tmp_actual_unpack_bytes,
-                           info, request);
+                           info, op, request);
         YAKSU_ERR_CHECK(rc, fn_fail);
 
         sbuf += tmp_actual_unpack_bytes;
@@ -443,7 +455,7 @@ static int unpack_sub_struct(const void *inbuf, uintptr_t insize, void *outbuf,
 
 int yaksi_iunpack_element(const void *inbuf, uintptr_t insize, void *outbuf, yaksi_type_s * type,
                           uintptr_t outoffset, uintptr_t * actual_unpack_bytes,
-                          yaksi_info_s * info, yaksi_request_s * request)
+                          yaksi_info_s * info, yaksa_op_t op, yaksi_request_s * request)
 {
     int rc = YAKSA_SUCCESS;
 
@@ -458,37 +470,38 @@ int yaksi_iunpack_element(const void *inbuf, uintptr_t insize, void *outbuf, yak
     switch (type->kind) {
         case YAKSI_TYPE_KIND__HVECTOR:
             rc = unpack_sub_hvector(inbuf, insize, outbuf, type, outoffset, actual_unpack_bytes,
-                                    info, request);
+                                    info, op, request);
             YAKSU_ERR_CHECK(rc, fn_fail);
             break;
 
         case YAKSI_TYPE_KIND__BLKHINDX:
             rc = unpack_sub_blkhindx(inbuf, insize, outbuf, type, outoffset, actual_unpack_bytes,
-                                     info, request);
+                                     info, op, request);
             YAKSU_ERR_CHECK(rc, fn_fail);
             break;
 
         case YAKSI_TYPE_KIND__HINDEXED:
             rc = unpack_sub_hindexed(inbuf, insize, outbuf, type, outoffset, actual_unpack_bytes,
-                                     info, request);
+                                     info, op, request);
             YAKSU_ERR_CHECK(rc, fn_fail);
             break;
 
         case YAKSI_TYPE_KIND__STRUCT:
             rc = unpack_sub_struct(inbuf, insize, outbuf, type, outoffset, actual_unpack_bytes,
-                                   info, request);
+                                   info, op, request);
             YAKSU_ERR_CHECK(rc, fn_fail);
             break;
 
         case YAKSI_TYPE_KIND__RESIZED:
             rc = yaksi_iunpack_element(inbuf, insize, outbuf, type->u.resized.child, outoffset,
-                                       actual_unpack_bytes, info, request);
+                                       actual_unpack_bytes, info, op, request);
             YAKSU_ERR_CHECK(rc, fn_fail);
             break;
 
         case YAKSI_TYPE_KIND__CONTIG:
             rc = yaksi_iunpack(inbuf, insize, outbuf, type->u.contig.count,
-                               type->u.contig.child, outoffset, actual_unpack_bytes, info, request);
+                               type->u.contig.child, outoffset, actual_unpack_bytes, info, op,
+                               request);
             YAKSU_ERR_CHECK(rc, fn_fail);
             break;
 
@@ -497,7 +510,7 @@ int yaksi_iunpack_element(const void *inbuf, uintptr_t insize, void *outbuf, yak
                 yaksi_type_s *primary = type->u.subarray.primary;
                 char *dbuf = (char *) outbuf + type->true_lb - primary->true_lb;
                 rc = yaksi_iunpack_element(inbuf, insize, dbuf, primary, outoffset,
-                                           actual_unpack_bytes, info, request);
+                                           actual_unpack_bytes, info, op, request);
                 YAKSU_ERR_CHECK(rc, fn_fail);
                 break;
             }
