@@ -1,16 +1,27 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #ifndef MPIR_CONTEXTID_H_INCLUDED
 #define MPIR_CONTEXTID_H_INCLUDED
 
+#ifdef HAVE_EXTENDED_CONTEXT_BITS
+#define MPIR_CONTEXT_ID_T_DATATYPE MPI_UINT32_T
+typedef uint32_t MPIR_Context_id_t;
+#define MPIR_INVALID_CONTEXT_ID ((MPIR_Context_id_t)0xffffffff)
+#define MPIR_CONTEXT_ID_BITS (20)
+#define CONTEXT_ID_FMT PRIu32
+#else
+/* Default context id type is uint16_t. Instead of always using uint32_t, we take a conservative
+ * approach to ensure the smallest possible packet header size for ch3 and ch4 active messages.
+ */
 #define MPIR_CONTEXT_ID_T_DATATYPE MPI_UINT16_T
 typedef uint16_t MPIR_Context_id_t;
 #define MPIR_INVALID_CONTEXT_ID ((MPIR_Context_id_t)0xffff)
+#define MPIR_CONTEXT_ID_BITS (16)
+#define CONTEXT_ID_FMT PRIu16
+#endif
 
 /* The following preprocessor macros provide bitfield access information for
  * context ID values.  They follow a uniform naming pattern:
@@ -37,10 +48,8 @@ typedef uint16_t MPIR_Context_id_t;
 #define MPIR_CONTEXT_SUFFIX_WIDTH (1)
 #define MPIR_CONTEXT_SUFFIX_SHIFT (0)
 #define MPIR_CONTEXT_SUFFIX_MASK ((1 << MPIR_CONTEXT_SUFFIX_WIDTH) - 1)
-#define MPIR_CONTEXT_INTRA_PT2PT (0)
-#define MPIR_CONTEXT_INTRA_COLL  (1)
-#define MPIR_CONTEXT_INTER_PT2PT (0)
-#define MPIR_CONTEXT_INTER_COLL  (1)
+#define MPIR_CONTEXT_PT2PT_OFFSET 0
+#define MPIR_CONTEXT_COLL_OFFSET  1
 
 /* Used to derive context IDs for sub-communicators from a parent communicator's
    context ID value.  This field comes after the one bit suffix.
@@ -54,6 +63,7 @@ typedef uint16_t MPIR_Context_id_t;
 #define MPIR_CONTEXT_PARENT_OFFSET    (0 << MPIR_CONTEXT_SUBCOMM_SHIFT)
 #define MPIR_CONTEXT_INTRANODE_OFFSET (1 << MPIR_CONTEXT_SUBCOMM_SHIFT)
 #define MPIR_CONTEXT_INTERNODE_OFFSET (2 << MPIR_CONTEXT_SUBCOMM_SHIFT)
+#define MPIR_CONTEXT_MULTILEADS_OFFSET (3 << MPIR_CONTEXT_SUBCOMM_SHIFT)
 
 /* this field (IS_LOCALCOM) is used to derive a context ID for local
  * communicators of intercommunicators without communication */
@@ -86,10 +96,11 @@ typedef uint16_t MPIR_Context_id_t;
 
 /* should probably be (sizeof(int)*CHAR_BITS) once we make the code CHAR_BITS-clean */
 #define MPIR_CONTEXT_INT_BITS (32)
-#define MPIR_CONTEXT_ID_BITS (sizeof(MPIR_Context_id_t)*8)      /* 8 --> CHAR_BITS eventually */
 #define MPIR_MAX_CONTEXT_MASK \
     ((1 << (MPIR_CONTEXT_ID_BITS - (MPIR_CONTEXT_PREFIX_SHIFT + MPIR_CONTEXT_DYNAMIC_PROC_WIDTH))) / MPIR_CONTEXT_INT_BITS)
 
+void MPIR_context_id_init(void);
+
 /* Utility routines.  Where possible, these are kept in the source directory
    with the other comm routines (src/mpi/comm, in mpicomm.h).  However,
    to create a new communicator after a spawn or connect-accept operation,
