@@ -45,26 +45,41 @@ typedef enum MPIR_Lang_t {
 #endif
 } MPIR_Lang_t;
 
-extern const char MPII_Version_string[] MPICH_API_PUBLIC;
-extern const char MPII_Version_date[] MPICH_API_PUBLIC;
-extern const char MPII_Version_ABI[] MPICH_API_PUBLIC;
-extern const char MPII_Version_configure[] MPICH_API_PUBLIC;
-extern const char MPII_Version_device[] MPICH_API_PUBLIC;
-extern const char MPII_Version_CC[] MPICH_API_PUBLIC;
-extern const char MPII_Version_CXX[] MPICH_API_PUBLIC;
-extern const char MPII_Version_F77[] MPICH_API_PUBLIC;
-extern const char MPII_Version_FC[] MPICH_API_PUBLIC;
-extern const char MPII_Version_custom[] MPICH_API_PUBLIC;
-
 extern MPL_initlock_t MPIR_init_lock;
 
 #include "typerep_pre.h"        /* needed for MPIR_Typerep_req */
 
+typedef enum {
+    MPIR_NULL_REQUEST = 0,
+    MPIR_TYPEREP_REQUEST,
+    MPIR_GPU_REQUEST,
+} MPIR_request_type_t;
+
+typedef struct {
+    union {
+        MPIR_Typerep_req y_req;
+        MPL_gpu_request gpu_req;
+    } u;
+    MPIR_request_type_t type;
+} MPIR_gpu_req;
+
 int MPIR_Localcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
                    void *recvbuf, MPI_Aint recvcount, MPI_Datatype recvtype);
 int MPIR_Ilocalcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
                     void *recvbuf, MPI_Aint recvcount, MPI_Datatype recvtype,
-                    MPIR_Typerep_req * typereq_req);
+                    MPIR_Typerep_req * typerep_req);
+int MPIR_Localcopy_stream(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
+                          void *recvbuf, MPI_Aint recvcount, MPI_Datatype recvtype, void *stream);
+int MPIR_Localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
+                       MPI_Aint sendoffset, MPL_pointer_attr_t * sendattr, void *recvbuf,
+                       MPI_Aint recvcount, MPI_Datatype recvtype, MPI_Aint recvoffset,
+                       MPL_pointer_attr_t * recvattr, MPL_gpu_copy_direction_t dir,
+                       MPL_gpu_engine_type_t enginetype, bool commit);
+int MPIR_Ilocalcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
+                        MPI_Aint sendoffset, MPL_pointer_attr_t * sendattr, void *recvbuf,
+                        MPI_Aint recvcount, MPI_Datatype recvtype, MPI_Aint recvoffset,
+                        MPL_pointer_attr_t * recvattr, MPL_gpu_copy_direction_t dir,
+                        MPL_gpu_engine_type_t enginetype, bool commit, MPIR_gpu_req * req);
 
 /* Contiguous datatype calculates buffer address with `(char *) buf + dt_true_lb`.
  * However, dt_true_lb is treated as ptrdiff_t (signed), and when buf is MPI_BOTTOM
@@ -98,8 +113,19 @@ int MPIR_Find_external(struct MPIR_Comm *comm, int *external_size_p, int *extern
 int MPIR_Get_internode_rank(MPIR_Comm * comm_ptr, int r);
 int MPIR_Get_intranode_rank(MPIR_Comm * comm_ptr, int r);
 
-/* Default routines for asynchronous progress thread */
-int MPIR_Init_async_thread(void);
-int MPIR_Finalize_async_thread(void);
+#define MPIR_CAST(T, val) CAST_##T((val))
+#ifdef NDEBUG
+#define MPIR_CAST_int(val) ((int) (val))
+#define MPIR_CAST_Aint(val) ((MPI_Aint) (val))
+#else
+#define MPIR_CAST_int(val) \
+    (((val) > INT_MAX || ((val) < 0 && (val) < INT_MIN)) ? (assert(0), 0) : (int) (val))
+#define MPIR_CAST_Aint(val) \
+    (((val) > MPIR_AINT_MAX || ((val) < 0 && (val) < MPIR_AINT_MIN)) ? (assert(0), 0) \
+     : (MPI_Aint) (val))
+#endif
+
+int MPIR_get_supported_memory_kinds(char *requested_kinds, char **out_kinds);
+void MPIR_get_memory_kinds_from_comm(MPIR_Comm * comm_ptr, char **out_kinds);
 
 #endif /* MPIR_MISC_H_INCLUDED */
