@@ -23,13 +23,12 @@
  */
 
 int MPIR_Allgatherv_intra_ring(const void *sendbuf,
-                               int sendcount,
+                               MPI_Aint sendcount,
                                MPI_Datatype sendtype,
                                void *recvbuf,
-                               const int *recvcounts,
-                               const int *displs,
-                               MPI_Datatype recvtype,
-                               MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
+                               const MPI_Aint * recvcounts,
+                               const MPI_Aint * displs,
+                               MPI_Datatype recvtype, MPIR_Comm * comm_ptr, MPIR_Errflag_t errflag)
 {
     int comm_size, rank, i, left, right;
     int mpi_errno = MPI_SUCCESS;
@@ -50,12 +49,6 @@ int MPIR_Allgatherv_intra_ring(const void *sendbuf,
 
     MPIR_Datatype_get_extent_macro(recvtype, recvtype_extent);
 
-    char *sbuf = NULL, *rbuf = NULL;
-    int soffset, roffset;
-    int torecv, tosend, max, chunk_count = 0;
-    int sendnow, recvnow;
-    int sidx, ridx;
-
     if (sendbuf != MPI_IN_PLACE) {
         /* First, load the "local" version in the recvbuf. */
         mpi_errno = MPIR_Localcopy(sendbuf, sendcount, sendtype,
@@ -67,9 +60,11 @@ int MPIR_Allgatherv_intra_ring(const void *sendbuf,
     left = (comm_size + rank - 1) % comm_size;
     right = (rank + 1) % comm_size;
 
+    MPI_Aint torecv, tosend, max, chunk_count;
     torecv = total_count - recvcounts[rank];
     tosend = total_count - recvcounts[right];
 
+    chunk_count = 0;
     max = recvcounts[0];
     for (i = 1; i < comm_size; i++)
         if (max < recvcounts[i])
@@ -86,15 +81,20 @@ int MPIR_Allgatherv_intra_ring(const void *sendbuf,
     if (!chunk_count)
         chunk_count = max;
 
+    int soffset, roffset;
+    int sidx, ridx;
     sidx = rank;
     ridx = left;
     soffset = 0;
     roffset = 0;
     while (tosend || torecv) {  /* While we have data to send or receive */
+        MPI_Aint sendnow, recvnow;
         sendnow = ((recvcounts[sidx] - soffset) >
                    chunk_count) ? chunk_count : (recvcounts[sidx] - soffset);
         recvnow = ((recvcounts[ridx] - roffset) >
                    chunk_count) ? chunk_count : (recvcounts[ridx] - roffset);
+
+        char *sbuf, *rbuf;
         sbuf = (char *) recvbuf + ((displs[sidx] + soffset) * recvtype_extent);
         rbuf = (char *) recvbuf + ((displs[ridx] + roffset) * recvtype_extent);
 
@@ -110,41 +110,19 @@ int MPIR_Allgatherv_intra_ring(const void *sendbuf,
              * consecutive processes contribute 0 bytes each. */
         } else if (!sendnow) {  /* If there's no data to send, just do a recv call */
             mpi_errno =
-                MPIC_Recv(rbuf, recvnow, recvtype, left, MPIR_ALLGATHERV_TAG, comm_ptr, &status,
-                          errflag);
-            if (mpi_errno) {
-                /* for communication errors, just record the error but continue */
-                *errflag =
-                    MPIX_ERR_PROC_FAILED ==
-                    MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-                MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-            }
+                MPIC_Recv(rbuf, recvnow, recvtype, left, MPIR_ALLGATHERV_TAG, comm_ptr, &status);
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
             torecv -= recvnow;
         } else if (!recvnow) {  /* If there's no data to receive, just do a send call */
             mpi_errno =
                 MPIC_Send(sbuf, sendnow, recvtype, right, MPIR_ALLGATHERV_TAG, comm_ptr, errflag);
-            if (mpi_errno) {
-                /* for communication errors, just record the error but continue */
-                *errflag =
-                    MPIX_ERR_PROC_FAILED ==
-                    MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-                MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-            }
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
             tosend -= sendnow;
         } else {        /* There's data to be sent and received */
             mpi_errno = MPIC_Sendrecv(sbuf, sendnow, recvtype, right, MPIR_ALLGATHERV_TAG,
                                       rbuf, recvnow, recvtype, left, MPIR_ALLGATHERV_TAG,
                                       comm_ptr, &status, errflag);
-            if (mpi_errno) {
-                /* for communication errors, just record the error but continue */
-                *errflag =
-                    MPIX_ERR_PROC_FAILED ==
-                    MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-                MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-            }
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
             tosend -= sendnow;
             torecv -= recvnow;
         }
@@ -162,12 +140,8 @@ int MPIR_Allgatherv_intra_ring(const void *sendbuf,
     }
 
   fn_exit:
-    if (mpi_errno_ret)
-        mpi_errno = mpi_errno_ret;
-    else if (*errflag != MPIR_ERR_NONE)
-        MPIR_ERR_SET(mpi_errno, *errflag, "**coll_fail");
-
-    return mpi_errno;
+    return mpi_errno_ret;
   fn_fail:
+    mpi_errno_ret = mpi_errno;
     goto fn_exit;
 }
