@@ -37,8 +37,10 @@
 extern int MPIR_Nbc_progress_hook_id;
 
 extern MPIR_Tree_type_t MPIR_Iallreduce_tree_type;
+extern MPIR_Tree_type_t MPIR_Allreduce_tree_type;
 extern MPIR_Tree_type_t MPIR_Ireduce_tree_type;
 extern MPIR_Tree_type_t MPIR_Ibcast_tree_type;
+extern MPIR_Tree_type_t MPIR_Bcast_tree_type;
 extern void *MPIR_Csel_root;
 extern char MPII_coll_generic_json[];
 
