@@ -15,11 +15,10 @@ int MPIR_Ireduce_scatter_block_intra_sched_recursive_halving(const void *sendbuf
     int mpi_errno = MPI_SUCCESS;
     int rank, comm_size, i;
     MPI_Aint extent, true_extent, true_lb;
-    int *disps;
     void *tmp_recvbuf, *tmp_results;
-    int type_size ATTRIBUTE((unused)), total_count, dst;
+    int dst;
     int mask;
-    int *newcnts, *newdisps, rem, newdst, send_idx, recv_idx, last_idx, send_cnt, recv_cnt;
+    int rem, newdst, send_idx, recv_idx, last_idx, send_cnt, recv_cnt;
     int pof2, old_i, newrank;
 
     comm_size = comm_ptr->local_size;
@@ -32,17 +31,17 @@ int MPIR_Ireduce_scatter_block_intra_sched_recursive_halving(const void *sendbuf
     MPIR_Assert(MPIR_Op_is_commutative(op));
 #endif
 
-    disps = MPIR_Sched_alloc_state(s, comm_size * sizeof(int));
+    MPI_Aint *disps;
+    disps = MPIR_Sched_alloc_state(s, comm_size * sizeof(MPI_Aint));
     MPIR_ERR_CHKANDJUMP(!disps, mpi_errno, MPI_ERR_OTHER, "**nomem");
 
+    MPI_Aint total_count;
     total_count = 0;
     for (i = 0; i < comm_size; i++) {
         disps[i] = total_count;
         total_count += recvcount;
     }
 
-    MPIR_Datatype_get_size_macro(datatype, type_size);
-
     /* allocate temp. buffer to receive incoming data */
     tmp_recvbuf = MPIR_Sched_alloc_state(s, total_count * (MPL_MAX(extent, true_extent)));
     MPIR_ERR_CHKANDJUMP(!tmp_recvbuf, mpi_errno, MPI_ERR_OTHER, "**nomem");
@@ -66,11 +65,7 @@ int MPIR_Ireduce_scatter_block_intra_sched_recursive_halving(const void *sendbuf
     MPIR_ERR_CHECK(mpi_errno);
     MPIR_SCHED_BARRIER(s);
 
-    pof2 = 1;
-    while (pof2 <= comm_size)
-        pof2 <<= 1;
-    pof2 >>= 1;
-
+    pof2 = MPL_pof2(comm_size);
     rem = comm_size - pof2;
 
     /* In the non-power-of-two case, all even-numbered
@@ -112,10 +107,10 @@ int MPIR_Ireduce_scatter_block_intra_sched_recursive_halving(const void *sendbuf
          * even-numbered processes who no longer participate will
          * have their result calculated by the process to their
          * right (rank+1). */
-
-        newcnts = MPIR_Sched_alloc_state(s, pof2 * sizeof(int));
+        MPI_Aint *newcnts, *newdisps;
+        newcnts = MPIR_Sched_alloc_state(s, pof2 * sizeof(MPI_Aint));
         MPIR_ERR_CHKANDJUMP(!newcnts, mpi_errno, MPI_ERR_OTHER, "**nomem");
-        newdisps = MPIR_Sched_alloc_state(s, pof2 * sizeof(int));
+        newdisps = MPIR_Sched_alloc_state(s, pof2 * sizeof(MPI_Aint));
         MPIR_ERR_CHKANDJUMP(!newdisps, mpi_errno, MPI_ERR_OTHER, "**nomem");
 
         for (i = 0; i < pof2; i++) {
