@@ -12,14 +12,11 @@
 
 /* This is the utility file for comm that contains the basic comm items
    and storage management */
-#ifndef MPID_COMM_PREALLOC
-#define MPID_COMM_PREALLOC 8
-#endif
 
 /* Preallocated comm objects */
 /* initialized in initthread.c */
 MPIR_Comm MPIR_Comm_builtin[MPIR_COMM_N_BUILTIN];
-MPIR_Comm MPIR_Comm_direct[MPID_COMM_PREALLOC];
+MPIR_Comm MPIR_Comm_direct[MPIR_COMM_PREALLOC];
 
 MPIR_Object_alloc_t MPIR_Comm_mem = {
     0,
@@ -31,7 +28,7 @@ MPIR_Object_alloc_t MPIR_Comm_mem = {
     MPIR_COMM,
     sizeof(MPIR_Comm),
     MPIR_Comm_direct,
-    MPID_COMM_PREALLOC,
+    MPIR_COMM_PREALLOC,
     NULL, {0}
 };
 
@@ -92,7 +89,7 @@ static int get_string_value(char *s, int type, int val)
     if (type == MPIR_COMM_HINT_TYPE_BOOL) {
         strncpy(s, val ? "true" : "false", MPI_MAX_INFO_VAL);
     } else if (type == MPIR_COMM_HINT_TYPE_INT) {
-        MPL_snprintf(s, MPI_MAX_INFO_VAL, "%d", val);
+        snprintf(s, MPI_MAX_INFO_VAL, "%d", val);
     } else {
         return -1;
     }
@@ -106,32 +103,35 @@ static int get_string_value(char *s, int type, int val)
  * If the hint is registered with callback function, it can be used for customization
  * at both creation time and run-time.
  */
-int MPII_Comm_set_hints(MPIR_Comm * comm_ptr, MPIR_Info * info)
+int MPII_Comm_set_hints(MPIR_Comm * comm_ptr, MPIR_Info * info, bool in_comm_creation)
 {
     int mpi_errno = MPI_SUCCESS;
 
-    MPIR_Info *curr_info;
-    LL_FOREACH(info, curr_info) {
-        if (curr_info->key == NULL)
+    for (int i = 0; i < next_comm_hint_index; i++) {
+        if (!MPIR_comm_hint_list[i].key) {
             continue;
-        for (int i = 0; i < next_comm_hint_index; i++) {
-            if (MPIR_comm_hint_list[i].key &&
-                strcmp(curr_info->key, MPIR_comm_hint_list[i].key) == 0) {
-                int val;
-                int ret = parse_string_value(curr_info->value, MPIR_comm_hint_list[i].type, &val);
-                if (ret == 0) {
-                    if (MPIR_comm_hint_list[i].fn) {
-                        MPIR_comm_hint_list[i].fn(comm_ptr, i, val);
-                    } else {
-                        comm_ptr->hints[i] = val;
-                    }
+        }
+
+        const char *str_val;
+        str_val = MPIR_Info_lookup(info, MPIR_comm_hint_list[i].key);
+        if (str_val) {
+            int val;
+            int rc = parse_string_value(str_val, MPIR_comm_hint_list[i].type, &val);
+            if (rc == 0) {
+                if (MPIR_comm_hint_list[i].fn) {
+                    MPIR_comm_hint_list[i].fn(comm_ptr, i, val);
+                } else {
+                    comm_ptr->hints[i] = val;
                 }
             }
         }
     }
 
-    mpi_errno = MPID_Comm_set_hints(comm_ptr, info);
-    MPIR_ERR_CHECK(mpi_errno);
+    /* Device can process hints during commit stage if in_comm_creation */
+    if (!in_comm_creation) {
+        mpi_errno = MPID_Comm_set_hints(comm_ptr, info);
+        MPIR_ERR_CHECK(mpi_errno);
+    }
 
     /* FIXME: run collective to ensure hints consistency */
   fn_exit:
@@ -153,6 +153,10 @@ int MPII_Comm_get_hints(MPIR_Comm * comm_ptr, MPIR_Info * info)
         }
     }
 
+    char *memory_alloc_kinds;
+    MPIR_get_memory_kinds_from_comm(comm_ptr, &memory_alloc_kinds);
+    MPIR_Info_set_impl(info, "mpi_memory_alloc_kinds", memory_alloc_kinds);
+
   fn_exit:
     return mpi_errno;
   fn_fail:
@@ -167,6 +171,63 @@ int MPII_Comm_check_hints(MPIR_Comm * comm)
     return MPI_SUCCESS;
 }
 
+/*
+=== BEGIN_INFO_HINT_BLOCK ===
+hints:
+    - name        : mpi_assert_no_any_tag
+      functions   : MPI_Comm_dup_with_info, MPI_Comm_idup_with_info, MPI_Comm_set_info
+      type        : boolean
+      default     : false
+      description : >-
+        If set to true, user promises that MPI_ANY_TAG will not be used with
+        the communicator. This potentially allows MPICH to treat messages with
+        different tags independent and seek to improve performance, e.g. by
+        employ multiple network context.
+
+    - name        : mpi_assert_no_any_source
+      functions   : MPI_Comm_dup_with_info, MPI_Comm_idup_with_info, MPI_Comm_set_info
+      type        : boolean
+      default     : false
+      description : >-
+        If set to true, user promises that MPI_ANY_SOURCE will not be used with
+        the communicator. This potentially allows MPICH to treat messages send
+        to different ranks or receive from different ranks independent and
+        seek to improve performance, e.g. by employ multiple network context.
+
+    - name        : mpi_assert_exact_length
+      functions   : MPI_Comm_dup_with_info, MPI_Comm_idup_with_info, MPI_Comm_set_info
+      type        : boolean
+      default     : false
+      description : >-
+        If set to true, user promises that the lengths of messages received
+        by the process will always equal to the size of the corresponding
+        receive buffers.
+
+    - name        : mpi_assert_allow_overtaking
+      functions   : MPI_Comm_dup_with_info, MPI_Comm_idup_with_info, MPI_Comm_set_info
+      type        : boolean
+      default     : false
+      description : >-
+        If set to true, user asserts that send operations are not required
+        to be matched at the receiver in the order in which the send operations
+        were performed by the sender, and receive operations are not required
+        to be matched in the order in which they were performed by the
+        receivers.
+
+    - name        : mpi_assert_strict_persistent_collective_ordering
+      functions   : MPI_Comm_dup_with_info, MPI_Comm_idup_with_info, MPI_Comm_set_info
+      type        : boolean
+      default     : false
+      description : >-
+      If set to true, then the implementation may assume that all the persistent
+      collective operations are started in the same order across all MPI processes
+      in the group of the communicator. It is required that if this assertion is made
+      on one member of the communicator's group, then it must be made on all members
+      of that communicator's group with the same value.
+
+=== END_INFO_HINT_BLOCK ===
+*/
+
 void MPIR_Comm_hint_init(void)
 {
     MPIR_Comm_register_hint(MPIR_COMM_HINT_NO_ANY_TAG, "mpi_assert_no_any_tag",
@@ -177,6 +238,9 @@ void MPIR_Comm_hint_init(void)
                             NULL, MPIR_COMM_HINT_TYPE_BOOL, 0, 0);
     MPIR_Comm_register_hint(MPIR_COMM_HINT_ALLOW_OVERTAKING, "mpi_assert_allow_overtaking",
                             NULL, MPIR_COMM_HINT_TYPE_BOOL, 0, 0);
+    MPIR_Comm_register_hint(MPIR_COMM_HINT_STRICT_PCOLL_ORDERING,
+                            "mpi_assert_strict_persistent_collective_ordering", NULL,
+                            MPIR_COMM_HINT_TYPE_BOOL, 0, 0);
     /* Used by ch4:ofi, but needs to be initialized early to get the default value. */
     MPIR_Comm_register_hint(MPIR_COMM_HINT_ENABLE_MULTI_NIC_STRIPING, "enable_multi_nic_striping",
                             NULL, MPIR_COMM_HINT_TYPE_BOOL, 0, -1);
@@ -229,6 +293,7 @@ int MPII_Comm_init(MPIR_Comm * comm_p)
     comm_p->remote_group = NULL;
     comm_p->local_group = NULL;
     comm_p->topo_fns = NULL;
+    comm_p->bsendbuffer = NULL;
     comm_p->name[0] = '\0';
     comm_p->seq = 0;    /* default to 0, to be updated at Comm_commit */
     comm_p->tainted = 0;
@@ -253,7 +318,12 @@ int MPII_Comm_init(MPIR_Comm * comm_p)
     comm_p->mapper_head = NULL;
     comm_p->mapper_tail = NULL;
 
-    /* mutex is only used in POBJ or VCI granularity. But the overhead of
+    comm_p->threadcomm = NULL;
+    MPIR_stream_comm_init(comm_p);
+
+    comm_p->persistent_requests = NULL;
+
+    /* mutex is only used in VCI granularity. But the overhead of
      * creation is low, so we always create it. */
     {
         int thr_err;
@@ -261,6 +331,9 @@ int MPII_Comm_init(MPIR_Comm * comm_p)
         MPIR_Assert(thr_err == 0);
     }
 
+    /* Initialize the session_ptr to NULL; for non-session-related communicators it will remain NULL */
+    comm_p->session_ptr = NULL;
+
     /* Fields not set include context_id, remote and local size, and
      * kind, since different communicator construction routines need
      * different values */
@@ -315,6 +388,8 @@ int MPII_Setup_intercomm_localcomm(MPIR_Comm * intercomm_ptr)
     mpi_errno = MPII_Comm_init(localcomm_ptr);
     MPIR_ERR_CHECK(mpi_errno);
 
+    MPIR_Comm_set_session_ptr(localcomm_ptr, intercomm_ptr->session_ptr);
+
     /* use the parent intercomm's recv ctx as the basis for our ctx */
     localcomm_ptr->recvcontext_id =
         MPIR_CONTEXT_SET_FIELD(IS_LOCALCOMM, intercomm_ptr->recvcontext_id, 1);
@@ -613,6 +688,8 @@ int MPIR_Comm_create_subcomms(MPIR_Comm * comm)
         comm->node_comm->local_size = num_local;
         comm->node_comm->remote_size = num_local;
 
+        MPIR_Comm_set_session_ptr(comm->node_comm, comm->session_ptr);
+
         /* Copy relevant hints to node_comm */
         propagate_hints_to_subcomm(comm, comm->node_comm);
 
@@ -638,6 +715,8 @@ int MPIR_Comm_create_subcomms(MPIR_Comm * comm)
         comm->node_roots_comm->local_size = num_external;
         comm->node_roots_comm->remote_size = num_external;
 
+        MPIR_Comm_set_session_ptr(comm->node_roots_comm, comm->session_ptr);
+
         /* Copy relevant hints to node_roots_comm */
         propagate_hints_to_subcomm(comm, comm->node_roots_comm);
 
@@ -679,8 +758,7 @@ static int init_comm_seq(MPIR_Comm * comm)
         /* Every rank need share the same seq from root. NOTE: it is possible for
          * different communicators to have the same seq. It is only used as an
          * opportunistic optimization */
-        MPIR_Errflag_t errflag = MPIR_ERR_NONE;
-        mpi_errno = MPIR_Bcast_allcomm_auto(&tmp, 1, MPI_INT, 0, comm, &errflag);
+        mpi_errno = MPIR_Bcast_allcomm_auto(&tmp, 1, MPI_INT, 0, comm, MPIR_ERR_NONE);
         MPIR_ERR_CHECK(mpi_errno);
 
         comm->seq = tmp;
@@ -795,6 +873,41 @@ int MPII_Comm_is_node_consecutive(MPIR_Comm * comm)
     return 1;
 }
 
+/* Duplicate a communicator without copying the streams. This is the common
+ * part called by both MPIR_Comm_dup_with_info_impl and MPIR_Stream_comm_create_impl.
+ */
+int MPII_Comm_dup(MPIR_Comm * comm_ptr, MPIR_Info * info, MPIR_Comm ** newcomm_ptr)
+{
+    int mpi_errno = MPI_SUCCESS;
+    MPIR_Attribute *new_attributes = 0;
+
+    /* Copy attributes, executing the attribute copy functions */
+    /* This accesses the attribute dup function through the perprocess
+     * structure to prevent comm_dup from forcing the linking of the
+     * attribute functions.  The actual function is (by default)
+     * MPIR_Attr_dup_list
+     */
+    if (MPIR_Process.attr_dup) {
+        mpi_errno = MPIR_Process.attr_dup(comm_ptr->handle, comm_ptr->attributes, &new_attributes);
+        MPIR_ERR_CHECK(mpi_errno);
+    }
+
+
+    /* Generate a new context value and a new communicator structure */
+    /* We must use the local size, because this is compared to the
+     * rank of the process in the communicator.  For intercomms,
+     * this must be the local size */
+    mpi_errno = MPII_Comm_copy(comm_ptr, comm_ptr->local_size, info, newcomm_ptr);
+    MPIR_ERR_CHECK(mpi_errno);
+
+    (*newcomm_ptr)->attributes = new_attributes;
+
+  fn_exit:
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
+
 /*
  * Copy a communicator, including creating a new context and copying the
  * virtual connection tables and clearing the various fields.
@@ -854,6 +967,8 @@ int MPII_Comm_copy(MPIR_Comm * comm_ptr, int size, MPIR_Info * info, MPIR_Comm *
     newcomm_ptr->comm_kind = comm_ptr->comm_kind;
     newcomm_ptr->local_comm = 0;
 
+    MPIR_Comm_set_session_ptr(newcomm_ptr, comm_ptr->session_ptr);
+
     /* There are two cases here - size is the same as the old communicator,
      * or it is smaller.  If the size is the same, we can just add a reference.
      * Otherwise, we need to create a new network address mapping.  Note that this is the
@@ -897,15 +1012,15 @@ int MPII_Comm_copy(MPIR_Comm * comm_ptr, int size, MPIR_Info * info, MPIR_Comm *
     }
 
     /* Inherit the error handler (if any) */
-    MPID_THREAD_CS_ENTER(POBJ, comm_ptr->mutex);
+    MPID_THREAD_CS_ENTER(VCI, comm_ptr->mutex);
     newcomm_ptr->errhandler = comm_ptr->errhandler;
     if (comm_ptr->errhandler) {
         MPIR_Errhandler_add_ref(comm_ptr->errhandler);
     }
-    MPID_THREAD_CS_EXIT(POBJ, comm_ptr->mutex);
+    MPID_THREAD_CS_EXIT(VCI, comm_ptr->mutex);
 
     if (info) {
-        MPII_Comm_set_hints(newcomm_ptr, info);
+        MPII_Comm_set_hints(newcomm_ptr, info, true);
     }
 
     newcomm_ptr->tainted = comm_ptr->tainted;
@@ -966,16 +1081,18 @@ int MPII_Comm_copy_data(MPIR_Comm * comm_ptr, MPIR_Info * info, MPIR_Comm ** out
     newcomm_ptr->remote_size = comm_ptr->remote_size;
     newcomm_ptr->is_low_group = comm_ptr->is_low_group; /* only relevant for intercomms */
 
+    MPIR_Comm_set_session_ptr(newcomm_ptr, comm_ptr->session_ptr);
+
     /* Inherit the error handler (if any) */
-    MPID_THREAD_CS_ENTER(POBJ, comm_ptr->mutex);
+    MPID_THREAD_CS_ENTER(VCI, comm_ptr->mutex);
     newcomm_ptr->errhandler = comm_ptr->errhandler;
     if (comm_ptr->errhandler) {
         MPIR_Errhandler_add_ref(comm_ptr->errhandler);
     }
-    MPID_THREAD_CS_EXIT(POBJ, comm_ptr->mutex);
+    MPID_THREAD_CS_EXIT(VCI, comm_ptr->mutex);
 
     if (info) {
-        MPII_Comm_set_hints(newcomm_ptr, info);
+        MPII_Comm_set_hints(newcomm_ptr, info, true);
     }
 
     /* Start with no attributes on this communicator */
@@ -1003,6 +1120,8 @@ int MPIR_Comm_delete_internal(MPIR_Comm * comm_ptr)
 {
     int in_use;
     int mpi_errno = MPI_SUCCESS;
+    int unmatched_messages = 0;
+    MPI_Comm comm_handle ATTRIBUTE((unused)) = comm_ptr->handle;
 
     MPIR_FUNC_ENTER;
 
@@ -1020,12 +1139,48 @@ int MPIR_Comm_delete_internal(MPIR_Comm * comm_ptr)
         /* Release the temporary reference added before the call to
          * attr_free */
         MPIR_Object_release_ref(comm_ptr, &in_use);
+        MPIR_Assertp(in_use == 0);
     }
 
     /* If the attribute delete functions return failure, the
      * communicator must not be freed.  That is the reason for the
      * test on mpi_errno here. */
     if (mpi_errno == MPI_SUCCESS) {
+        bool do_message_check = false;
+#if defined(HAVE_ERROR_CHECKING)
+        do_message_check = true;
+#endif
+#if defined(MPICH_IS_THREADED) && MPICH_THREAD_GRANULARITY == MPICH_THREAD_GRANULARITY__VCI
+        /* anysrc/anytag probe is extremely messy with pervci thread-cs. Avoid for for now. */
+        if (MPIR_ThreadInfo.isThreaded) {
+            do_message_check = false;
+        }
+#endif
+        if (do_message_check) {
+            /* receive any unmatched messages to clear the queue and avoid context_id
+             * reuse issues. unmatched messages could be the result of user error,
+             * or send operations that were unable to be canceled by the device layer */
+            MPIR_Object_add_ref(comm_ptr);
+            int flag;
+            MPI_Status status;
+            do {
+                mpi_errno = MPID_Iprobe(MPI_ANY_SOURCE, MPI_ANY_TAG, comm_ptr, 0, &flag, &status);
+                MPIR_ERR_CHECK(mpi_errno);
+                if (flag) {
+                    /* receive the message, ignore truncation errors */
+                    MPIR_Request *request;
+                    MPID_Recv(NULL, 0, MPI_DATATYPE_NULL, status.MPI_SOURCE, status.MPI_TAG,
+                              comm_ptr, 0, MPI_STATUS_IGNORE, &request);
+                    if (request != NULL) {
+                        MPID_Wait(request, MPI_STATUS_IGNORE);
+                        MPIR_Request_free(request);
+                    }
+                    unmatched_messages++;
+                }
+            } while (flag);
+            MPIR_Object_release_ref(comm_ptr, &in_use);
+        }
+
         /* If this communicator is our parent, and we're disconnecting
          * from the parent, mark that fact */
         if (MPIR_Process.comm_parent == comm_ptr)
@@ -1040,6 +1195,14 @@ int MPIR_Comm_delete_internal(MPIR_Comm * comm_ptr)
         mpi_errno = MPID_Comm_free_hook(comm_ptr);
         MPIR_ERR_CHECK(mpi_errno);
 
+        mpi_errno = MPIR_Comm_bsend_finalize(comm_ptr);
+        MPIR_ERR_CHECK(mpi_errno);
+
+        if (comm_ptr->session_ptr != NULL) {
+            /* Release session */
+            MPIR_Session_release(comm_ptr->session_ptr);
+        }
+
         if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTERCOMM && comm_ptr->local_comm)
             MPIR_Comm_release(comm_ptr->local_comm);
 
@@ -1057,6 +1220,8 @@ int MPIR_Comm_delete_internal(MPIR_Comm * comm_ptr)
         MPL_free(comm_ptr->intranode_table);
         MPL_free(comm_ptr->internode_table);
 
+        MPIR_stream_comm_free(comm_ptr);
+
         /* Free the context value.  This should come after freeing the
          * intra/inter-node communicators since those free calls won't
          * release this context ID and releasing this before then could lead
@@ -1098,94 +1263,173 @@ int MPIR_Comm_delete_internal(MPIR_Comm * comm_ptr)
     }
 
   fn_exit:
+    if (unmatched_messages > 0) {
+        MPIR_ERR_SET2(mpi_errno, MPI_ERR_OTHER, "**commhasunmatched", "**commhasunmatched %x %d",
+                      comm_handle, unmatched_messages);
+    }
+
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
     goto fn_exit;
 }
 
-/* Release a reference to a communicator.  If there are no pending
-   references, delete the communicator and recover all storage and
-   context ids.  This version of the function always manipulates the reference
-   counts, even for predefined objects. */
-int MPIR_Comm_release_always(MPIR_Comm * comm_ptr)
+/* This function retrieves info key and collectively compares hint_str to see
+ * hether all processes are having the same string.
+ */
+int MPII_collect_info_key(MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, const char *key,
+                          const char **value_ptr)
 {
     int mpi_errno = MPI_SUCCESS;
-    int in_use;
 
-    MPIR_FUNC_ENTER;
+    const char *hint_str = NULL;
+    if (info_ptr) {
+        hint_str = MPIR_Info_lookup(info_ptr, key);
+    }
 
-    /* we want to short-circuit any optimization that avoids reference counting
-     * predefined communicators, such as MPI_COMM_WORLD or MPI_COMM_SELF. */
-    MPIR_Object_release_ref_always(comm_ptr, &in_use);
-    if (!in_use) {
-        mpi_errno = MPIR_Comm_delete_internal(comm_ptr);
+    int hint_str_size;
+    if (!hint_str) {
+        hint_str_size = 0;
+    } else {
+        hint_str_size = strlen(hint_str);
+        if (hint_str_size == 0) {
+            hint_str = NULL;
+        }
+    }
+
+    if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTERCOMM) {
+        /* just ensure consistency on the local group */
+        /* TODO: check consistency between local and remote processes */
+        if (!comm_ptr->local_comm) {
+            MPII_Setup_intercomm_localcomm(comm_ptr);
+        }
+        comm_ptr = comm_ptr->local_comm;
+    }
+
+    int is_equal;
+    mpi_errno = MPIR_Allreduce_equal(&hint_str_size, 1, MPI_INT, &is_equal, comm_ptr);
+    MPIR_ERR_CHECK(mpi_errno);
+
+    if (is_equal && hint_str_size > 0) {
+        mpi_errno = MPIR_Allreduce_equal(hint_str, hint_str_size, MPI_CHAR, &is_equal, comm_ptr);
         MPIR_ERR_CHECK(mpi_errno);
     }
 
+    MPIR_ERR_CHKANDJUMP1(!is_equal, mpi_errno, MPI_ERR_OTHER, "**infonoteq", "**infonoteq %s", key);
+
+    *value_ptr = hint_str;
+
   fn_exit:
-    MPIR_FUNC_EXIT;
-    return mpi_errno;
+    return MPI_SUCCESS;
   fn_fail:
+    /* inconsistent info keys are ignored */
+    *value_ptr = NULL;
     goto fn_exit;
 }
 
-/* This function collectively compares hint_str to see whether all processes are having
- * the same string. The result is set in output pointer info_args_are_equal.
- */
-/* TODO: it is certainly not ideal that we have to run 4 Allreduce to do this check.
- * Once we have an OP_EQUAL operator, a single Allreduce would suffice.
- */
-int MPII_compare_info_hint(const char *hint_str, MPIR_Comm * comm_ptr, int *info_args_are_equal)
+/* Returns true if the communicator is node-aware and the number of processes in all the nodes are
+ * same */
+int MPII_Comm_is_node_balanced(MPIR_Comm * comm, int *num_nodes, bool * node_balanced)
 {
-    int hint_str_size = strlen(hint_str);
-    int hint_str_size_max;
-    int hint_str_equal;
-    int hint_str_equal_global = 0;
-    char *hint_str_global = NULL;
+    int i = 0;
     int mpi_errno = MPI_SUCCESS;
-    MPIR_Errflag_t errflag = MPIR_ERR_NONE;
+    int *ranks_per_node;
+    *num_nodes = 0;
 
-    /* Find the maximum hint_str size.  Each process locally compares
-     * its hint_str size to the global max, and makes sure that this
-     * comparison is successful on all processes. */
-    mpi_errno =
-        MPID_Allreduce(&hint_str_size, &hint_str_size_max, 1, MPI_INT, MPI_MAX, comm_ptr, &errflag);
-    MPIR_ERR_CHECK(mpi_errno);
+    MPIR_CHKPMEM_DECL(1);
 
-    hint_str_equal = (hint_str_size == hint_str_size_max);
-
-    mpi_errno =
-        MPID_Allreduce(&hint_str_equal, &hint_str_equal_global, 1, MPI_INT, MPI_LAND,
-                       comm_ptr, &errflag);
-    MPIR_ERR_CHECK(mpi_errno);
-
-    if (!hint_str_equal_global)
+    if (!MPIR_Comm_is_parent_comm(comm)) {
+        *node_balanced = false;
         goto fn_exit;
+    }
 
+    /* Find maximum value in the internode_table */
+    for (i = 0; i < comm->local_size; i++) {
+        if (comm->internode_table[i] > *num_nodes) {
+            *num_nodes = comm->internode_table[i];
+        }
+    }
+    /* number of nodes is max_node_id + 1 */
+    (*num_nodes)++;
 
-    /* Now that the sizes of the hint_strs match, check to make sure
-     * the actual hint_strs themselves are the equal */
-    hint_str_global = (char *) MPL_malloc(strlen(hint_str), MPL_MEM_OTHER);
+    MPIR_CHKPMEM_CALLOC(ranks_per_node, int *,
+                        *num_nodes * sizeof(int), mpi_errno, "ranks per node", MPL_MEM_OTHER);
 
-    mpi_errno =
-        MPID_Allreduce(hint_str, hint_str_global, strlen(hint_str), MPI_CHAR,
-                       MPI_MAX, comm_ptr, &errflag);
-    MPIR_ERR_CHECK(mpi_errno);
+    for (i = 0; i < comm->local_size; i++) {
+        ranks_per_node[comm->internode_table[i]]++;
+    }
 
-    hint_str_equal = !memcmp(hint_str, hint_str_global, strlen(hint_str));
-
-    mpi_errno =
-        MPID_Allreduce(&hint_str_equal, &hint_str_equal_global, 1, MPI_INT, MPI_LAND,
-                       comm_ptr, &errflag);
-    MPIR_ERR_CHECK(mpi_errno);
+    for (i = 1; i < *num_nodes; i++) {
+        if (ranks_per_node[i - 1] != ranks_per_node[i]) {
+            *node_balanced = false;
+            goto fn_exit;
+        }
+    }
 
+    *node_balanced = true;
   fn_exit:
-    MPL_free(hint_str_global);
-
-    *info_args_are_equal = hint_str_equal_global;
+    MPIR_CHKPMEM_REAP();
     return mpi_errno;
-
   fn_fail:
     goto fn_exit;
 }
+
+int MPIR_Comm_save_inactive_request(MPIR_Comm * comm, MPIR_Request * request)
+{
+    MPID_THREAD_CS_ENTER(VCI, comm->mutex);
+    HASH_ADD_INT(comm->persistent_requests, handle, request, MPL_MEM_COMM);
+    MPID_THREAD_CS_EXIT(VCI, comm->mutex);
+
+    return MPI_SUCCESS;
+}
+
+int MPIR_Comm_delete_inactive_request(MPIR_Comm * comm, MPIR_Request * request)
+{
+    MPID_THREAD_CS_ENTER(VCI, comm->mutex);
+    HASH_DEL(comm->persistent_requests, request);
+    MPID_THREAD_CS_EXIT(VCI, comm->mutex);
+
+    return MPI_SUCCESS;
+}
+
+int MPIR_Comm_free_inactive_requests(MPIR_Comm * comm)
+{
+    MPIR_Request *request, *tmp;
+    MPID_THREAD_CS_ENTER(VCI, comm->mutex);
+    HASH_ITER(hh, comm->persistent_requests, request, tmp) {
+        if (!MPIR_Request_is_active(request)) {
+            HASH_DEL(comm->persistent_requests, request);
+            /* reset request->comm so it won't trigger MPIR_Comm_delete_inactive_request,
+             * which will recursively entering the lock. */
+            if (request->comm) {
+                MPIR_Comm_release(request->comm);
+                request->comm = NULL;
+            }
+            MPL_internal_error_printf
+                ("WARNING: freeing inactive persistent request %x on communicator %x.\n",
+                 request->handle, comm->handle);
+            MPIR_Request_free_impl(request);
+        }
+    }
+    MPID_THREAD_CS_EXIT(VCI, comm->mutex);
+
+    return MPI_SUCCESS;
+}
+
+void MPIR_Comm_set_session_ptr(MPIR_Comm * comm_ptr, MPIR_Session * session_ptr)
+{
+    if (session_ptr != NULL) {
+        comm_ptr->session_ptr = session_ptr;
+        /* Add ref counter of session */
+        MPIR_Session_add_ref(session_ptr);
+    }
+}
+
+void MPIR_get_memory_kinds_from_comm(MPIR_Comm * comm_ptr, char **kinds)
+{
+    if (comm_ptr->session_ptr) {
+        *kinds = comm_ptr->session_ptr->memory_alloc_kinds;
+    } else {
+        *kinds = MPIR_Process.memory_alloc_kinds;
+    }
+}
