@@ -11,12 +11,18 @@
 
 #if (MPICH_DATATYPE_ENGINE == MPICH_DATATYPE_ENGINE_YAKSA)
 #include "yaksa.h"
-typedef yaksa_request_t MPIR_Typerep_req;
+typedef struct {
+    yaksa_request_t req;
+    yaksa_info_t info;
+} MPIR_Typerep_req;
 #define MPIR_TYPEREP_REQ_NULL YAKSA_REQUEST__NULL
 #define MPIR_TYPEREP_HANDLE_TYPE yaksa_type_t
 #define MPIR_TYPEREP_HANDLE_NULL YAKSA_TYPE__NULL
 #else
-typedef int MPIR_Typerep_req;   /* unused in dataloop */
+typedef struct {
+    int req;
+    int info;
+} MPIR_Typerep_req;             /* unused in dataloop */
 #define MPIR_TYPEREP_REQ_NULL 0
 #define MPIR_TYPEREP_HANDLE_TYPE struct MPII_Dataloop *
 #define MPIR_TYPEREP_HANDLE_NULL NULL
