@@ -1,9 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *   ad_panfs_open.c
- *
- *   Copyright (C) 2001 University of Chicago.
- *   See COPYRIGHT notice in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "ad_panfs.h"
@@ -286,48 +283,47 @@ void ADIOI_PANFS_Open(ADIO_File fd, int *error_code)
             /* Error - set layout type to unknown */
             ADIOI_Info_set(fd->info, "panfs_layout_type", "PAN_FS_CLIENT_LAYOUT_TYPE__INVALID");
         } else {
-            MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u", file_query_args.layout.agg_type);
+            snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u", file_query_args.layout.agg_type);
             ADIOI_Info_set(fd->info, "panfs_layout_type", temp_buffer);
             if (file_query_args.layout.layout_is_valid == 1) {
                 switch (file_query_args.layout.agg_type) {
                     case PAN_FS_CLIENT_LAYOUT_TYPE__RAID0:
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.raid0.stripe_unit);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid0.stripe_unit);
                         ADIOI_Info_set(fd->info, "panfs_layout_stripe_unit", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.raid0.total_num_comps);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid0.total_num_comps);
                         ADIOI_Info_set(fd->info, "panfs_layout_total_num_comps", temp_buffer);
                         break;
                     case PAN_FS_CLIENT_LAYOUT_TYPE__RAID1_5_PARITY_STRIPE:
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.raid1_5_parity_stripe.stripe_unit);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid1_5_parity_stripe.stripe_unit);
                         ADIOI_Info_set(fd->info, "panfs_layout_stripe_unit", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.
-                                     raid1_5_parity_stripe.parity_stripe_width);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.
+                                 raid1_5_parity_stripe.parity_stripe_width);
                         ADIOI_Info_set(fd->info, "panfs_layout_parity_stripe_width", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.
-                                     raid1_5_parity_stripe.parity_stripe_depth);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.
+                                 raid1_5_parity_stripe.parity_stripe_depth);
                         ADIOI_Info_set(fd->info, "panfs_layout_parity_stripe_depth", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.
-                                     raid1_5_parity_stripe.total_num_comps);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid1_5_parity_stripe.total_num_comps);
                         ADIOI_Info_set(fd->info, "panfs_layout_total_num_comps", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.
-                                     raid1_5_parity_stripe.layout_visit_policy);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.
+                                 raid1_5_parity_stripe.layout_visit_policy);
                         ADIOI_Info_set(fd->info, "panfs_layout_visit_policy", temp_buffer);
                         break;
                     case PAN_FS_CLIENT_LAYOUT_TYPE__RAID10:
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.raid10.stripe_unit);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid10.stripe_unit);
                         ADIOI_Info_set(fd->info, "panfs_layout_stripe_unit", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.raid10.total_num_comps);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid10.total_num_comps);
                         ADIOI_Info_set(fd->info, "panfs_layout_total_num_comps", temp_buffer);
-                        MPL_snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
-                                     file_query_args.layout.u.raid10.layout_visit_policy);
+                        snprintf(temp_buffer, TEMP_BUFFER_SIZE, "%u",
+                                 file_query_args.layout.u.raid10.layout_visit_policy);
                         ADIOI_Info_set(fd->info, "panfs_layout_visit_policy", temp_buffer);
                         break;
                     case PAN_FS_CLIENT_LAYOUT_TYPE__INVALID:
