@@ -73,7 +73,7 @@ void ADIOI_GEN_SetInfo(ADIO_File fd, MPI_Info users_info, int *error_code)
         fd->hints->cb_config_list = NULL;
 
         /* number of processes that perform I/O in collective I/O */
-        MPL_snprintf(value, MPI_MAX_INFO_VAL + 1, "%d", nprocs);
+        snprintf(value, MPI_MAX_INFO_VAL + 1, "%d", nprocs);
         ADIOI_Info_set(info, "cb_nodes", value);
         fd->hints->cb_nodes = nprocs;
 
@@ -132,6 +132,14 @@ void ADIOI_GEN_SetInfo(ADIO_File fd, MPI_Info users_info, int *error_code)
         ADIOI_Info_set(info, "romio_synchronized_flush", "disabled");
         fd->hints->synchronizing_flush = 0;
 
+        /* While MPI-IO rules say a write from one process is not visible until
+         * sync or close, many file systems implement the more restrictive
+         * POSIX semantics:  under POSIX semantics a write from one process is
+         * visible to everyone.  For counter-example, Unify, NFS and PVFS do
+         * not support this semantic */
+        ADIOI_Info_set(info, "romio_visibility_immediate", "true");
+        fd->hints->visibility_immediate = 1;
+
         fd->hints->initialized = 1;
 
         /* ADIO_Open sets up collective buffering arrays.  If we are in this
@@ -222,7 +230,7 @@ void ADIOI_GEN_SetInfo(ADIO_File fd, MPI_Info users_info, int *error_code)
                 /* can't ask for more aggregators than mpi processes, though it
                  * might be interesting to think what such oversubscription
                  * might mean... someday */
-                MPL_snprintf(value, MPI_MAX_INFO_VAL + 1, "%d", nprocs);
+                snprintf(value, MPI_MAX_INFO_VAL + 1, "%d", nprocs);
                 ADIOI_Info_set(info, "cb_nodes", value);
                 fd->hints->cb_nodes = nprocs;
             }
@@ -249,11 +257,17 @@ void ADIOI_GEN_SetInfo(ADIO_File fd, MPI_Info users_info, int *error_code)
         ADIOI_Info_check_and_install_int(fd, users_info, "romio_min_fdomain_size",
                                          &(fd->hints->min_fdomain_size), myname, error_code);
 
-        /* Now we use striping unit in common code so we should
+        /* Now we use striping information in common code so we should
          * process hints for it. */
         ADIOI_Info_check_and_install_int(fd, users_info, "striping_unit",
                                          &(fd->hints->striping_unit), myname, error_code);
 
+        ADIOI_Info_check_and_install_int(fd, users_info, "striping_factor",
+                                         &(fd->hints->striping_factor), myname, error_code);
+
+        ADIOI_Info_check_and_install_int(fd, users_info, "start_iodevice",
+                                         &(fd->hints->start_iodevice), myname, error_code);
+
         ADIOI_Info_check_and_install_enabled(fd, users_info, "romio_synchronized_flush",
                                              &(fd->hints->synchronizing_flush), myname, error_code);
     }
