@@ -107,7 +107,6 @@ Output Parameters:
 int MPI_File_iwrite_c(MPI_File fh, ROMIO_CONST void *buf, MPI_Count count,
                       MPI_Datatype datatype, MPI_Request * request)
 {
-    assert(count <= INT_MAX);
     int error_code = MPI_SUCCESS;
     static char myname[] = "MPI_FILE_IWRITE";
 #ifdef MPI_hpux
@@ -138,9 +137,8 @@ int MPIOI_File_iwrite(MPI_File fh,
                       MPI_Offset offset,
                       int file_ptr_type,
                       const void *buf,
-                      int count, MPI_Datatype datatype, char *myname, MPI_Request * request)
+                      MPI_Aint count, MPI_Datatype datatype, char *myname, MPI_Request * request)
 {
-    assert(count <= INT_MAX);
     int error_code, buftype_is_contig, filetype_is_contig;
     MPI_Count datatype_size;
     ADIO_Status status;
