@@ -14,14 +14,13 @@ int MPID_nem_tcp_ckpt_pause_send_vc(MPIDI_VC_t * vc)
 {
     int mpi_errno = MPI_SUCCESS;
     MPID_nem_tcp_vc_area *vc_tcp = VC_TCP(vc);
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_NEM_TCP_CKPT_PAUSE_SEND_VC);
 
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_NEM_TCP_CKPT_PAUSE_SEND_VC);
+    MPIR_FUNC_ENTER;
 
     vc_tcp->send_paused = TRUE;
 
   fn_exit:
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_NEM_TCP_CKPT_PAUSE_SEND_VC);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
 
@@ -34,9 +33,8 @@ int MPID_nem_tcp_pkt_unpause_handler(MPIDI_VC_t * vc, MPIDI_CH3_Pkt_t * pkt,
 {
     int mpi_errno = MPI_SUCCESS;
     MPID_nem_tcp_vc_area *vc_tcp = VC_TCP(vc);
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_NEM_TCP_CKPT_UNPAUSE_HANDLER);
 
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_NEM_TCP_CKPT_UNPAUSE_HANDLER);
+    MPIR_FUNC_ENTER;
 
     vc_tcp->send_paused = FALSE;
 
@@ -56,7 +54,7 @@ int MPID_nem_tcp_pkt_unpause_handler(MPIDI_VC_t * vc, MPIDI_CH3_Pkt_t * pkt,
   fn_exit:
     *buflen = 0;
 
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_NEM_TCP_CKPT_UNPAUSE_HANDLER);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
 
@@ -68,9 +66,8 @@ int MPID_nem_tcp_ckpt_continue_vc(MPIDI_VC_t * vc)
     int mpi_errno = MPI_SUCCESS;
     MPID_PKT_DECL_CAST(upkt, MPIDI_nem_tcp_pkt_unpause_t, unpause_pkt);
     MPIR_Request *unpause_req;
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_NEM_TCP_CKPT_CONTINUE_VC);
 
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_NEM_TCP_CKPT_CONTINUE_VC);
+    MPIR_FUNC_ENTER;
 
     unpause_pkt->type = MPIDI_NEM_PKT_NETMOD;
     unpause_pkt->subtype = MPIDI_NEM_TCP_PKT_UNPAUSE;
@@ -88,7 +85,7 @@ int MPID_nem_tcp_ckpt_continue_vc(MPIDI_VC_t * vc)
     }
 
   fn_exit:
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_NEM_TCP_CKPT_CONTINUE_VC);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
 
@@ -103,9 +100,8 @@ int MPID_nem_tcp_ckpt_restart_vc(MPIDI_VC_t * vc)
     MPIDI_CH3_Pkt_t upkt;
     MPIDI_nem_tcp_pkt_unpause_t *const pkt = (MPIDI_nem_tcp_pkt_unpause_t *) & upkt;
     MPIR_Request *sreq;
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_NEM_TCP_CKPT_RESTART_VC);
 
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_NEM_TCP_CKPT_RESTART_VC);
+    MPIR_FUNC_ENTER;
 
     pkt->type = MPIDI_NEM_PKT_NETMOD;
     pkt->subtype = MPIDI_NEM_TCP_PKT_UNPAUSE;
@@ -123,7 +119,7 @@ int MPID_nem_tcp_ckpt_restart_vc(MPIDI_VC_t * vc)
     }
 
   fn_exit:
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_NEM_TCP_CKPT_RESTART_VC);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
 
