@@ -8,9 +8,8 @@
 static MPIR_Request *create_request(void *hdr, intptr_t hdr_sz, size_t nb)
 {
     MPIR_Request *sreq;
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_CREATE_REQUEST);
 
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_CREATE_REQUEST);
+    MPIR_FUNC_ENTER;
 
     sreq = MPIR_Request_create(MPIR_REQUEST_KIND__UNDEFINED);
     /* --BEGIN ERROR HANDLING-- */
@@ -26,7 +25,7 @@ static MPIR_Request *create_request(void *hdr, intptr_t hdr_sz, size_t nb)
     sreq->dev.iov_count = 1;
     sreq->dev.OnDataAvail = 0;
 
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_CREATE_REQUEST);
+    MPIR_FUNC_EXIT;
     return sreq;
 }
 
@@ -35,7 +34,7 @@ static MPIR_Request *create_request(void *hdr, intptr_t hdr_sz, size_t nb)
  * entire message is successfully sent, then NULL is
  * returned.  Otherwise a request is allocated, the header is copied into the
  * request, and a pointer to the request is returned.
- * An error condition also results in a request be allocated and the errror
+ * An error condition also results in a request be allocated and the error
  * being returned in the status field of the request.
  */
 int MPIDI_CH3_iStartMsg(MPIDI_VC_t * vc, void *hdr, intptr_t hdr_sz, MPIR_Request ** sreq_ptr)
@@ -43,9 +42,8 @@ int MPIDI_CH3_iStartMsg(MPIDI_VC_t * vc, void *hdr, intptr_t hdr_sz, MPIR_Reques
     MPIR_Request *sreq = NULL;
     MPIDI_CH3I_VC *vcch = &vc->ch;
     int mpi_errno = MPI_SUCCESS;
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_CH3_ISTARTMSG);
 
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_CH3_ISTARTMSG);
+    MPIR_FUNC_ENTER;
 
     MPIR_Assert(hdr_sz <= sizeof(MPIDI_CH3_Pkt_t));
 
@@ -185,6 +183,6 @@ int MPIDI_CH3_iStartMsg(MPIDI_VC_t * vc, void *hdr, intptr_t hdr_sz, MPIR_Reques
 
   fn_fail:
     *sreq_ptr = sreq;
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_CH3_ISTARTMSG);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 }
