@@ -308,7 +308,8 @@ int MPIDI_CH3_ReqHandler_GaccumRecvComplete(MPIDI_VC_t * vc, MPIR_Request * rreq
     else {
         MPI_Aint actual_pack_bytes;
         MPIR_Typerep_pack(rreq->dev.real_user_buf, rreq->dev.user_count, rreq->dev.datatype,
-                       stream_offset, resp_req->dev.user_buf, stream_data_len, &actual_pack_bytes);
+                       stream_offset, resp_req->dev.user_buf, stream_data_len, &actual_pack_bytes,
+                       MPIR_TYPEREP_FLAG_NONE);
         MPIR_Assert(actual_pack_bytes == stream_data_len);
     }
 
@@ -339,9 +340,7 @@ int MPIDI_CH3_ReqHandler_GaccumRecvComplete(MPIDI_VC_t * vc, MPIR_Request * rreq
     iov[1].iov_len = stream_data_len;
     iovcnt = 2;
 
-    MPID_THREAD_CS_ENTER(POBJ, vc->pobj_mutex);
     mpi_errno = MPIDI_CH3_iSendv(vc, resp_req, iov, iovcnt);
-    MPID_THREAD_CS_EXIT(POBJ, vc->pobj_mutex);
 
     MPIR_ERR_CHKANDJUMP(mpi_errno != MPI_SUCCESS, mpi_errno, MPI_ERR_OTHER, "**ch3|rmamsg");
 
@@ -433,7 +432,7 @@ int MPIDI_CH3_ReqHandler_FOPRecvComplete(MPIDI_VC_t * vc, MPIR_Request * rreq, i
     else {
         MPI_Aint actual_pack_bytes;
         MPIR_Typerep_pack(rreq->dev.real_user_buf, 1, rreq->dev.datatype, 0, resp_req->dev.user_buf,
-                       type_size, &actual_pack_bytes);
+                       type_size, &actual_pack_bytes, MPIR_TYPEREP_FLAG_NONE);
         MPIR_Assert(actual_pack_bytes == type_size);
     }
 
@@ -466,9 +465,7 @@ int MPIDI_CH3_ReqHandler_FOPRecvComplete(MPIDI_VC_t * vc, MPIR_Request * rreq, i
     iov[1].iov_len = type_size;
     iovcnt = 2;
 
-    MPID_THREAD_CS_ENTER(POBJ, vc->pobj_mutex);
     mpi_errno = MPIDI_CH3_iSendv(vc, resp_req, iov, iovcnt);
-    MPID_THREAD_CS_EXIT(POBJ, vc->pobj_mutex);
 
     MPIR_ERR_CHKANDJUMP(mpi_errno != MPI_SUCCESS, mpi_errno, MPI_ERR_OTHER, "**ch3|rmamsg");
 
@@ -804,11 +801,8 @@ int MPIDI_CH3_ReqHandler_GetDerivedDTRecvComplete(MPIDI_VC_t * vc,
     sreq->dev.msg_offset = 0;
     sreq->dev.msgsize = new_dtp->size * sreq->dev.user_count;
 
-    /* Because this is in a packet handler, it is already within a critical section */
-    /* MPID_THREAD_CS_ENTER(POBJ, vc->pobj_mutex); */
     mpi_errno = vc->sendNoncontig_fn(vc, sreq, get_resp_pkt, sizeof(*get_resp_pkt),
                                      NULL, 0);
-    /* MPID_THREAD_CS_EXIT(POBJ, vc->pobj_mutex); */
     /* --BEGIN ERROR HANDLING-- */
     if (mpi_errno != MPI_SUCCESS) {
         MPIR_Request_free(sreq);
@@ -1283,7 +1277,8 @@ static inline int perform_get_acc_in_lock_queue(MPIR_Win * win_ptr,
     else {
         MPI_Aint actual_pack_bytes;
         MPIR_Typerep_pack(get_accum_pkt->addr, get_accum_pkt->count, get_accum_pkt->datatype,
-                       0, sreq->dev.user_buf, type_size * recv_count, &actual_pack_bytes);
+                       0, sreq->dev.user_buf, type_size * recv_count, &actual_pack_bytes,
+                       MPIR_TYPEREP_FLAG_NONE);
         MPIR_Assert(actual_pack_bytes == type_size * recv_count);
     }
 
@@ -1422,7 +1417,7 @@ static inline int perform_fop_in_lock_queue(MPIR_Win * win_ptr,
     else {
         MPI_Aint actual_pack_bytes;
         MPIR_Typerep_pack(fop_pkt->addr, 1, fop_pkt->datatype, 0, resp_req->dev.user_buf,
-                       type_size, &actual_pack_bytes);
+                       type_size, &actual_pack_bytes, MPIR_TYPEREP_FLAG_NONE);
         MPIR_Assert(actual_pack_bytes == type_size);
     }
 
@@ -1446,11 +1441,9 @@ static inline int perform_fop_in_lock_queue(MPIR_Win * win_ptr,
 
     if (fop_pkt->type == MPIDI_CH3_PKT_FOP_IMMED) {
         /* send back the original data */
-        MPID_THREAD_CS_ENTER(POBJ, target_lock_entry->vc->pobj_mutex);
         mpi_errno =
             MPIDI_CH3_iStartMsg(target_lock_entry->vc, fop_resp_pkt, sizeof(*fop_resp_pkt),
                                 &resp_req);
-        MPID_THREAD_CS_EXIT(POBJ, target_lock_entry->vc->pobj_mutex);
         MPIR_ERR_CHKANDJUMP(mpi_errno != MPI_SUCCESS, mpi_errno, MPI_ERR_OTHER, "**ch3|rmamsg");
 
         if (resp_req != NULL) {
@@ -1547,10 +1540,8 @@ static inline int perform_cas_in_lock_queue(MPIR_Win * win_ptr,
     }
 
     /* Send the response packet */
-    MPID_THREAD_CS_ENTER(POBJ, target_lock_entry->vc->pobj_mutex);
     mpi_errno =
         MPIDI_CH3_iStartMsg(target_lock_entry->vc, cas_resp_pkt, sizeof(*cas_resp_pkt), &send_req);
-    MPID_THREAD_CS_EXIT(POBJ, target_lock_entry->vc->pobj_mutex);
 
     MPIR_ERR_CHKANDJUMP(mpi_errno != MPI_SUCCESS, mpi_errno, MPI_ERR_OTHER, "**ch3|rmamsg");
 
