@@ -4,11 +4,6 @@
  */
 
 #include "mpidimpl.h"
-#ifdef USE_PMI2_API
-#include "pmi2.h"
-#else
-#include "pmi.h"
-#endif
 
 /*
 === BEGIN_MPI_T_CVAR_INFO_BLOCK ===
@@ -52,6 +47,7 @@ int MPIDI_PG_Init(MPIDI_PG_Compare_ids_fn_t compare_ids_fn,
     return mpi_errno;
 }
 
+extern int MPIDI_lpid_counter;
 /*@ 
    MPIDI_PG_Finalize - Finalize the process groups, including freeing all
    process group structures
@@ -68,7 +64,7 @@ int MPIDI_PG_Finalize(void)
 	MPIU_PG_Printall( stdout );
     }
 
-    /* FIXME - straighten out the use of PMI_Finalize - no use after 
+    /* FIXME - straighten out the use of MPIR_pmi_finalize - no use after 
        PG_Finalize */
     if (pg_world->connData) {
         MPIR_pmi_finalize();
@@ -103,6 +99,7 @@ int MPIDI_PG_Finalize(void)
 	MPIDI_PG_Destroy(MPIDI_Process.my_pg);
     } 
     MPIDI_Process.my_pg = NULL;
+    MPIDI_lpid_counter = 0;
 
     /* ifdefing out this check because the list will not be NULL in 
        Ch3_finalize because
@@ -507,64 +504,24 @@ void MPIDI_PG_IdToNum( MPIDI_PG_t *pg, int *id )
 */
 int MPIDI_PG_SetConnInfo( int rank, const char *connString )
 {
-#ifdef USE_PMI2_API
     int mpi_errno = MPI_SUCCESS;
+
     int len;
-    char key[PMI2_MAX_KEYLEN];
-
-    MPIR_FUNC_ENTER;
-
-    len = MPL_snprintf(key, sizeof(key), "P%d-businesscard", rank);
+    char key[40];
+    len = snprintf(key, sizeof(key), "P%d-businesscard", rank);
     MPIR_ERR_CHKANDJUMP1(len < 0 || len > sizeof(key), mpi_errno, MPI_ERR_OTHER, "**snprintf", "**snprintf %d", len);
 
-    mpi_errno = PMI2_KVS_Put(key, connString);
+    mpi_errno = MPIR_pmi_kvs_put(key, connString);
     MPIR_ERR_CHECK(mpi_errno);
 
-    mpi_errno = PMI2_KVS_Fence();
+    mpi_errno = MPIR_pmi_barrier();
     MPIR_ERR_CHECK(mpi_errno);
-    
+
  fn_exit:
     MPIR_FUNC_EXIT;
     return mpi_errno;
  fn_fail:
     goto fn_exit;
-#else
-    int mpi_errno = MPI_SUCCESS;
-    int pmi_errno;
-    int len;
-    char key[128];
-
-    MPIR_FUNC_ENTER;
-
-    MPIR_Assert(pg_world->connData);
-    
-    len = MPL_snprintf(key, sizeof(key), "P%d-businesscard", rank);
-    if (len < 0 || len > sizeof(key)) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**snprintf",
-			     "**snprintf %d", len);
-    }
-    pmi_errno = PMI_KVS_Put(pg_world->connData, key, connString );
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**pmi_kvs_put",
-			     "**pmi_kvs_put %d", pmi_errno);
-    }
-    pmi_errno = PMI_KVS_Commit(pg_world->connData);
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**pmi_kvs_commit",
-			     "**pmi_kvs_commit %d", pmi_errno);
-    }
-    
-    pmi_errno = PMI_Barrier();
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**pmi_barrier",
-			     "**pmi_barrier %d", pmi_errno);
-    }
- fn_exit:
-    MPIR_FUNC_EXIT;
-    return mpi_errno;
- fn_fail:
-    goto fn_exit;
-#endif
 }
 
 /* For all of these routines, the format of the process group description
@@ -591,51 +548,20 @@ int MPIDI_PG_SetConnInfo( int rank, const char *connString )
    process information */
 static int getConnInfoKVS( int rank, char *buf, int bufsize, MPIDI_PG_t *pg )
 {
-#ifdef USE_PMI2_API
-    char key[MPIDI_MAX_KVS_KEY_LEN];
-    int  mpi_errno = MPI_SUCCESS, rc;
-    int vallen;
+    int mpi_errno = MPI_SUCCESS;
 
-    rc = MPL_snprintf(key, MPIDI_MAX_KVS_KEY_LEN, "P%d-businesscard", rank );
-    if (rc < 0 || rc > MPIDI_MAX_KVS_KEY_LEN) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**nomem");
-    }
+    int len;
+    char key[40];
+    len = snprintf(key, sizeof(key), "P%d-businesscard", rank);
+    MPIR_ERR_CHKANDJUMP1(len < 0 || len > sizeof(key), mpi_errno, MPI_ERR_OTHER, "**snprintf", "**snprintf %d", len);
 
-    mpi_errno = PMI2_KVS_Get(pg->connData, PMI2_ID_NULL, key, buf, bufsize, &vallen);
-    if (mpi_errno) {
-	MPIDI_PG_CheckForSingleton();
-	mpi_errno = PMI2_KVS_Get(pg->connData, PMI2_ID_NULL, key, buf, bufsize, &vallen);
-        MPIR_ERR_CHECK(mpi_errno);
-    }
- fn_exit:
-    return mpi_errno;
- fn_fail:
-    goto fn_exit;
-#else
-    char key[MPIDI_MAX_KVS_KEY_LEN];
-    int  mpi_errno = MPI_SUCCESS, rc, pmi_errno;
-
-    rc = MPL_snprintf(key, MPIDI_MAX_KVS_KEY_LEN, "P%d-businesscard", rank );
-    if (rc < 0 || rc > MPIDI_MAX_KVS_KEY_LEN) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**nomem");
-    }
-
-    MPID_THREAD_CS_ENTER(POBJ, MPIR_THREAD_POBJ_PMI_MUTEX);
-    pmi_errno = PMI_KVS_Get(pg->connData, key, buf, bufsize );
-    if (pmi_errno) {
-	MPIDI_PG_CheckForSingleton();
-	pmi_errno = PMI_KVS_Get(pg->connData, key, buf, bufsize );
-    }
-    MPID_THREAD_CS_EXIT(POBJ, MPIR_THREAD_POBJ_PMI_MUTEX);
-    if (pmi_errno) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**pmi_kvs_get");
-    }
+    mpi_errno = MPIR_pmi_kvs_get(rank, key, buf, bufsize);
+    MPIR_ERR_CHECK(mpi_errno);
 
  fn_exit:
     return mpi_errno;
  fn_fail:
     goto fn_exit;
-#endif
 }
 
 /* *slen is the length of the string, including the null terminator.  So if the
@@ -661,7 +587,7 @@ static int connToStringKVS( char **buf_p, int *slen, MPIDI_PG_t *pg )
     string[len++] = 0;
     
     /* Add the size of the pg */
-    MPL_snprintf( &string[len], curSlen - len, "%d", pg->size );
+    snprintf( &string[len], curSlen - len, "%d", pg->size );
     while (string[len]) len++;
     len++;
 
@@ -734,49 +660,15 @@ static int connFreeKVS( MPIDI_PG_t *pg )
 
 int MPIDI_PG_InitConnKVS( MPIDI_PG_t *pg )
 {
-#ifdef USE_PMI2_API
-    int mpi_errno = MPI_SUCCESS;
-    
-    pg->connData = (char *)MPL_malloc(MPIDI_MAX_JOBID_LEN, MPL_MEM_STRINGS);
-    if (pg->connData == NULL) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER, "**nomem");
-    }
-    
-    mpi_errno = PMI2_Job_GetId(pg->connData, MPIDI_MAX_JOBID_LEN);
-    MPIR_ERR_CHECK(mpi_errno);
-#else
-    int pmi_errno, kvs_name_sz;
     int mpi_errno = MPI_SUCCESS;
 
-    pmi_errno = PMI_KVS_Get_name_length_max( &kvs_name_sz );
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, 
-			     "**pmi_kvs_get_name_length_max", 
-			     "**pmi_kvs_get_name_length_max %d", pmi_errno);
-    }
-    
-    pg->connData = (char *)MPL_malloc(kvs_name_sz + 1, MPL_MEM_STRINGS);
-    if (pg->connData == NULL) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER, "**nomem");
-    }
-    
-    pmi_errno = PMI_KVS_Get_my_name(pg->connData, kvs_name_sz);
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, 
-			     "**pmi_kvs_get_my_name", 
-			     "**pmi_kvs_get_my_name %d", pmi_errno);
-    }
-#endif
+    pg->connData = MPL_strdup(MPIR_pmi_job_id());
     pg->getConnInfo        = getConnInfoKVS;
     pg->connInfoToString   = connToStringKVS;
     pg->connInfoFromString = connFromStringKVS;
     pg->freeConnInfo       = connFreeKVS;
 
- fn_exit:
     return mpi_errno;
- fn_fail:
-    MPL_free(pg->connData);
-    goto fn_exit;
 }
 
 /* Return the kvsname associated with the MPI_COMM_WORLD of this process. */
@@ -833,24 +725,11 @@ static int connToString( char **buf_p, int *slen, MPIDI_PG_t *pg )
 #endif
 
     pg_id = pg->id;
-    /* FIXME: This is a hack, and it doesn't even work */
-    /*    MPIDI_PrintConnStrToFile( stdout, __FILE__, __LINE__, 
-	  "connToString: pg id is", (char *)pg_id );*/
-    /* This is intended to cause a process to transition from a singleton
-       to a non-singleton. */
-    /* XXX DJG TODO figure out what this little bit is all about. */
-    if (strstr( pg_id, "singinit_kvs" ) == pg_id) {
-#ifdef USE_PMI2_API
-        MPIR_Assertp(0); /* don't know what to do here for pmi2 yet.  DARIUS */
-#else
-	PMI_KVS_Get_my_name( pg->id, 256 );
-#endif
-    }
     
     while (*pg_id) str[len++] = *pg_id++;
     str[len++] = 0;
     
-    MPL_snprintf( &str[len], 20, "%d", pg->size);
+    snprintf( &str[len], 20, "%d", pg->size);
     /* Skip over the length */
     while (str[len++]);
 
@@ -1158,17 +1037,5 @@ int MPIU_PG_Printall( FILE *fp )
 
 int MPIDI_PG_CheckForSingleton( void )
 {
-
-#ifdef USE_PMI2_API
-    /* PMI2 FIXME for now we just always assume we aren't doing singleton init */
-#else
-    if (strstr((char*)pg_world->id,"singinit_kvs") == (char *)pg_world->id) {
-	char buf[256];
-	/* Force an enroll */
-	PMI_KVS_Get( "foobar", "foobar", buf, sizeof(buf) );
-	PMI_KVS_Get_my_name( pg_world->id, 256 );
-	PMI_KVS_Get_my_name( pg_world->connData, 256 );
-    }
-#endif
     return MPI_SUCCESS;
 }
