@@ -9,20 +9,34 @@
 MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_improbe(int source,
                                                   int tag,
                                                   MPIR_Comm * comm,
-                                                  int context_offset, MPIDI_av_entry_t * addr,
+                                                  int attr, MPIDI_av_entry_t * addr,
                                                   int *flag, MPIR_Request ** message,
                                                   MPI_Status * status)
 {
-    return MPIDIG_mpi_improbe(source, tag, comm, context_offset, 0, flag, message, status);
+    int mpi_errno = MPI_SUCCESS;
+    int context_offset = MPIR_PT2PT_ATTR_CONTEXT_OFFSET(attr);
+
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(0).lock);
+    mpi_errno = MPIDIG_mpi_improbe(source, tag, comm, context_offset, 0, flag, message, status);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(0).lock);
+
+    return mpi_errno;
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_iprobe(int source,
                                                  int tag,
                                                  MPIR_Comm * comm,
-                                                 int context_offset, MPIDI_av_entry_t * addr,
+                                                 int attr, MPIDI_av_entry_t * addr,
                                                  int *flag, MPI_Status * status)
 {
-    return MPIDIG_mpi_iprobe(source, tag, comm, context_offset, 0, flag, status);
+    int mpi_errno = MPI_SUCCESS;
+    int context_offset = MPIR_PT2PT_ATTR_CONTEXT_OFFSET(attr);
+
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(0).lock);
+    mpi_errno = MPIDIG_mpi_iprobe(source, tag, comm, context_offset, 0, flag, status);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(0).lock);
+
+    return mpi_errno;
 }
 
 #endif /* NETMOD_AM_FALLBACK_PROBE_H_INCLUDED */
