@@ -19,7 +19,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_handle_short_am(MPIDI_OFI_am_header_t * m
     MPIR_FUNC_ENTER;
 
     int attr = 0;               /* is_local = 0, is_async = 0 */
-    MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vni_src, msg_hdr->vni_dst);
+    MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vci_src, msg_hdr->vci_dst);
     MPIDIG_global.target_msg_cbs[msg_hdr->handler_id] (am_hdr,
                                                        p_data, msg_hdr->payload_sz, attr, NULL);
 
@@ -38,7 +38,11 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_handle_pipeline(MPIDI_OFI_am_header_t * m
 
     MPIR_FUNC_ENTER;
 
-    cache_rreq = MPIDIG_req_cache_lookup(MPIDI_OFI_global.req_map, (uint64_t) msg_hdr->fi_src_addr);
+    int vci = msg_hdr->vci_dst;
+    void *req_map = MPIDI_OFI_global.per_vci[vci].req_map;
+    uint64_t remote_id = msg_hdr->src_id;
+
+    cache_rreq = MPIDIG_req_cache_lookup(req_map, remote_id);
     MPL_DBG_MSG_FMT(MPIDI_CH4_DBG_GENERAL, VERBOSE,
                     (MPL_DBG_FDEST, "cached req %p handle=0x%x", cache_rreq,
                      cache_rreq ? cache_rreq->handle : 0));
@@ -47,17 +51,17 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_handle_pipeline(MPIDI_OFI_am_header_t * m
 
     if (!rreq) {
         int attr = MPIDIG_AM_ATTR__IS_ASYNC;
-        MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vni_src, msg_hdr->vni_dst);
+        MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vci_src, msg_hdr->vci_dst);
         MPIDIG_global.target_msg_cbs[msg_hdr->handler_id] (am_hdr, p_data, msg_hdr->payload_sz,
                                                            attr, &rreq);
         MPIDIG_recv_setup(rreq);
-        MPIDIG_req_cache_add(MPIDI_OFI_global.req_map, (uint64_t) msg_hdr->fi_src_addr, rreq);
+        MPIDIG_req_cache_add(req_map, remote_id, rreq);
     }
 
     is_done = MPIDIG_recv_copy_seg(p_data, msg_hdr->payload_sz, rreq);
     if (is_done) {
         MPIDIG_REQUEST(rreq, req->target_cmpl_cb) (rreq);
-        MPIDIG_req_cache_remove(MPIDI_OFI_global.req_map, (uint64_t) msg_hdr->fi_src_addr);
+        MPIDIG_req_cache_remove(req_map, remote_id);
     }
 
     MPIR_FUNC_EXIT;
@@ -72,7 +76,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_handle_short_am_hdr(MPIDI_OFI_am_header_t
     MPIR_FUNC_ENTER;
 
     int attr = 0;
-    MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vni_src, msg_hdr->vni_dst);
+    MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vci_src, msg_hdr->vci_dst);
     MPIDIG_global.target_msg_cbs[msg_hdr->handler_id] (am_hdr, NULL, 0, attr, NULL);
 
     MPIR_FUNC_EXIT;
@@ -93,13 +97,13 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_do_rdma_read(void *dst,
 
     rem = data_sz;
 
-    int vni_local = MPIDIG_REQUEST(rreq, req->local_vci);
-    int vni_remote = MPIDIG_REQUEST(rreq, req->remote_vci);
+    int vci_local = MPIDIG_REQUEST(rreq, req->local_vci);
+    int vci_remote = MPIDIG_REQUEST(rreq, req->remote_vci);
     while (done != data_sz) {
         curr_len = MPL_MIN(rem, MPIDI_OFI_global.max_msg_size);
 
         MPIR_Assert(sizeof(MPIDI_OFI_am_request_t) <= MPIDI_OFI_AM_HDR_POOL_CELL_SIZE);
-        MPIDU_genq_private_pool_alloc_cell(MPIDI_OFI_global.per_vni[vni_local].am_hdr_buf_pool,
+        MPIDU_genq_private_pool_alloc_cell(MPIDI_OFI_global.per_vci[vci_local].am_hdr_buf_pool,
                                            (void **) &am_req);
         MPIR_Assert(am_req);
 
@@ -114,7 +118,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_do_rdma_read(void *dst,
 
         /* am uses nic 0 */
         int nic = 0;
-        MPIDI_OFI_cntr_incr(comm, vni_local, nic);
+        MPIDI_OFI_cntr_incr(vci_local, nic);
 
         struct iovec iov = {
             .iov_base = (char *) dst + done,
@@ -129,14 +133,14 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_do_rdma_read(void *dst,
             .msg_iov = &iov,
             .desc = NULL,
             .iov_count = 1,
-            .addr = MPIDI_OFI_comm_to_phys(comm, src_rank, nic, vni_local, vni_remote),
+            .addr = MPIDI_OFI_comm_to_phys(comm, src_rank, nic, vci_remote),
             .rma_iov = &rma_iov,
             .rma_iov_count = 1,
             .context = &am_req->context,
             .data = 0
         };
 
-        int ctx_idx = MPIDI_OFI_get_ctx_index(comm, vni_local, nic);
+        int ctx_idx = MPIDI_OFI_get_ctx_index(vci_local, nic);
         MPIDI_OFI_CALL_RETRY_AM(fi_readmsg(MPIDI_OFI_global.ctx[ctx_idx].tx, &msg, FI_COMPLETION),
                                 rdma_readfrom);
 
@@ -163,7 +167,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_handle_rdma_read(MPIDI_OFI_am_header_t *
     MPIR_FUNC_ENTER;
 
     int attr = MPIDIG_AM_ATTR__IS_ASYNC | MPIDIG_AM_ATTR__IS_RNDV | MPIDI_OFI_AM_ATTR__RDMA;
-    MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vni_src, msg_hdr->vni_dst);
+    MPIDIG_AM_ATTR_SET_VCIS(attr, msg_hdr->vci_src, msg_hdr->vci_dst);
     MPIDIG_global.target_msg_cbs[msg_hdr->handler_id] (am_hdr, NULL, 0, attr, &rreq);
 
     if (!rreq)
@@ -308,7 +312,7 @@ MPL_STATIC_INLINE_PREFIX int do_long_am_recv_unpack(MPI_Aint in_data_sz, MPIR_Re
     p->context_id = lmt_msg->context_id;
     p->src_offset = lmt_msg->src_offset;
 
-    mpi_errno = MPIR_gpu_malloc_host(&p->unpack_buffer, pack_size);
+    p->unpack_buffer = MPL_malloc(pack_size, MPL_MEM_OTHER);
     MPIR_ERR_CHECK(mpi_errno);
 
     MPI_Aint remain = MPIDIG_REQUEST(rreq, req->recv_async).in_data_sz;
@@ -345,7 +349,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_OFI_am_lmt_unpack_event(MPIR_Request * rreq)
         return FALSE;
     } else {
         /* all done. */
-        MPIR_gpu_free_host(p->unpack_buffer);
+        MPL_free(p->unpack_buffer);
         return TRUE;
     }
 }
