@@ -12,28 +12,28 @@
  * value that is generally common across the different providers. */
 #define MPIDI_OFI_MAX_ERR_DATA_SIZE 64
 
-static int peek_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int peek_empty_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int send_huge_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int ssend_ack_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int chunk_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
-static int inject_emu_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
-static int accept_probe_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int dynproc_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int am_isend_rdma_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
+static int peek_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int peek_empty_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int send_huge_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int ssend_ack_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int chunk_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
+static int inject_emu_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
+static int accept_probe_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int dynproc_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int am_isend_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int am_isend_rdma_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int am_isend_pipeline_event(int vci, struct fi_cq_tagged_entry *wc,
                                    MPIR_Request * dont_use_me);
-static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me);
+static int am_recv_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int am_read_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me);
 
-static int peek_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int peek_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_ENTER;
 
     if (MPIDI_OFI_HUGE_SEND & wc->tag) {
-        mpi_errno = MPIDI_OFI_peek_huge_event(vni, wc, rreq);
+        mpi_errno = MPIDI_OFI_peek_huge_event(vci, wc, rreq);
         goto fn_exit;
     }
 
@@ -51,7 +51,7 @@ static int peek_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rre
     return mpi_errno;
 }
 
-static int peek_empty_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int peek_empty_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     MPIR_FUNC_ENTER;
     MPIDI_OFI_dynamic_process_request_t *ctrl;
@@ -80,7 +80,163 @@ static int peek_empty_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     return MPI_SUCCESS;
 }
 
-static int send_huge_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+/* GPU pipeline events */
+static int pipeline_send_event(struct fi_cq_tagged_entry *wc, MPIR_Request * r)
+{
+    int mpi_errno = MPI_SUCCESS;
+    int c;
+    MPIDI_OFI_gpu_pipeline_request *req;
+    MPIR_Request *sreq;
+    void *wc_buf = NULL;
+    MPIR_FUNC_ENTER;
+
+    req = (MPIDI_OFI_gpu_pipeline_request *) r;
+    /* get original mpi request */
+    sreq = req->parent;
+    wc_buf = req->buf;
+    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.gpu_pipeline_send_pool, wc_buf);
+
+    MPIR_cc_decr(sreq->cc_ptr, &c);
+    if (c == 0) {
+        MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(sreq, datatype));
+        MPIR_Request_free(sreq);
+    }
+    MPL_free(r);
+
+    MPIR_FUNC_EXIT;
+    return mpi_errno;
+}
+
+static int pipeline_recv_event(struct fi_cq_tagged_entry *wc, MPIR_Request * r, int event_id)
+{
+    int mpi_errno = MPI_SUCCESS;
+    int vci_local, i;
+    MPIDI_OFI_gpu_pipeline_request *req;
+    MPIR_Request *rreq;
+    void *wc_buf = NULL;
+    int in_use MPL_UNUSED;
+    MPIDI_OFI_gpu_task_t *task = NULL;
+    int engine_type = MPIR_CVAR_CH4_OFI_GPU_PIPELINE_H2D_ENGINE_TYPE;
+
+    MPIR_FUNC_ENTER;
+
+    req = (MPIDI_OFI_gpu_pipeline_request *) r;
+    rreq = req->parent;
+    wc_buf = req->buf;
+    MPL_free(r);
+
+    void *recv_buf = MPIDI_OFI_REQUEST(rreq, noncontig.pack.buf);
+    size_t recv_count = MPIDI_OFI_REQUEST(rreq, noncontig.pack.count);
+    MPI_Datatype datatype = MPIDI_OFI_REQUEST(rreq, noncontig.pack.datatype);
+
+    fi_addr_t remote_addr = MPIDI_OFI_REQUEST(rreq, pipeline_info.remote_addr);
+    vci_local = MPIDI_OFI_REQUEST(rreq, pipeline_info.vci_local);
+
+    if (event_id == MPIDI_OFI_EVENT_RECV_GPU_PIPELINE_INIT) {
+        rreq->status.MPI_SOURCE = MPIDI_OFI_cqe_get_source(wc, true);
+        rreq->status.MPI_ERROR = MPIDI_OFI_idata_get_error_bits(wc->data);
+        rreq->status.MPI_TAG = MPIDI_OFI_init_get_tag(wc->tag);
+
+        if (unlikely(MPIDI_OFI_is_tag_sync(wc->tag))) {
+            MPIDI_OFI_REQUEST(rreq, pipeline_info.is_sync) = true;
+        }
+
+        uint32_t packed = MPIDI_OFI_idata_get_gpu_packed_bit(wc->data);
+        uint32_t n_chunks = MPIDI_OFI_idata_get_gpuchunk_bits(wc->data);
+        if (likely(packed == 0)) {
+            if (wc->len > 0) {
+                /* First chunk arrives. */
+                MPI_Aint actual_unpack_bytes;
+                MPIR_gpu_req yreq;
+                mpi_errno =
+                    MPIR_Ilocalcopy_gpu(wc_buf, wc->len, MPI_BYTE, 0, NULL, recv_buf, recv_count,
+                                        datatype, 0, NULL, MPL_GPU_COPY_H2D, engine_type, 1, &yreq);
+                MPIR_ERR_CHECK(mpi_errno);
+                actual_unpack_bytes = wc->len;
+                task =
+                    MPIDI_OFI_create_gpu_task(MPIDI_OFI_PIPELINE_RECV, wc_buf,
+                                              actual_unpack_bytes, rreq, yreq);
+                DL_APPEND(MPIDI_OFI_global.gpu_recv_task_queue[vci_local], task);
+                MPIDI_OFI_REQUEST(rreq, pipeline_info.offset) += (size_t) actual_unpack_bytes;
+            } else {
+                /* free this chunk */
+                MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.gpu_pipeline_recv_pool, wc_buf);
+            }
+            /* Post recv for remaining chunks. */
+            MPIR_cc_dec(rreq->cc_ptr);
+            MPIR_Assert(n_chunks > 0);
+            for (i = 0; i < n_chunks; i++) {
+                int c;
+                MPIR_cc_incr(rreq->cc_ptr, &c);
+
+                size_t chunk_sz = MPIR_CVAR_CH4_OFI_GPU_PIPELINE_BUFFER_SZ;
+
+                char *host_buf = NULL;
+                MPIDU_genq_private_pool_alloc_cell(MPIDI_OFI_global.gpu_pipeline_recv_pool,
+                                                   (void **) &host_buf);
+
+                MPIDI_OFI_REQUEST(rreq, event_id) = MPIDI_OFI_EVENT_RECV_GPU_PIPELINE;
+
+                MPIDI_OFI_gpu_pipeline_request *chunk_req = NULL;
+                chunk_req = (MPIDI_OFI_gpu_pipeline_request *)
+                    MPL_malloc(sizeof(MPIDI_OFI_gpu_pipeline_request), MPL_MEM_BUFFER);
+                if (chunk_req == NULL) {
+                    mpi_errno = MPIR_ERR_OTHER;
+                    goto fn_fail;
+                }
+                chunk_req->event_id = MPIDI_OFI_EVENT_RECV_GPU_PIPELINE;
+                chunk_req->parent = rreq;
+                chunk_req->buf = host_buf;
+                int ret = 0;
+                if (!MPIDI_OFI_global.gpu_recv_queue && host_buf) {
+                    ret = fi_trecv
+                        (MPIDI_OFI_global.ctx
+                         [MPIDI_OFI_REQUEST(rreq, pipeline_info.ctx_idx)].rx,
+                         host_buf, chunk_sz, NULL, remote_addr,
+                         MPIDI_OFI_REQUEST(rreq,
+                                           pipeline_info.match_bits) | MPIDI_OFI_GPU_PIPELINE_SEND,
+                         MPIDI_OFI_REQUEST(rreq, pipeline_info.mask_bits),
+                         (void *) &chunk_req->context);
+                }
+                if (MPIDI_OFI_global.gpu_recv_queue || !host_buf || ret != 0) {
+                    MPIDI_OFI_gpu_pending_recv_t *recv_task =
+                        MPIDI_OFI_create_recv_task(chunk_req, i, n_chunks);
+                    DL_APPEND(MPIDI_OFI_global.gpu_recv_queue, recv_task);
+                }
+            }
+        } else {
+            MPIR_ERR_CHKANDJUMP(true, mpi_errno, MPI_ERR_OTHER, "**gpu_pipeline_packed");
+        }
+    } else {
+        if (likely(event_id == MPIDI_OFI_EVENT_RECV_GPU_PIPELINE)) {
+            /* FIXME: current design unpacks all bytes from host buffer, overflow check is missing. */
+            MPI_Aint actual_unpack_bytes;
+            MPIR_gpu_req yreq;
+            mpi_errno =
+                MPIR_Ilocalcopy_gpu(wc_buf, (MPI_Aint) wc->len, MPI_BYTE, 0, NULL,
+                                    (char *) recv_buf, (MPI_Aint) recv_count, datatype,
+                                    MPIDI_OFI_REQUEST(rreq, pipeline_info.offset), NULL,
+                                    MPL_GPU_COPY_H2D, engine_type, 1, &yreq);
+            MPIR_ERR_CHECK(mpi_errno);
+            actual_unpack_bytes = wc->len;
+            MPIDI_OFI_REQUEST(rreq, pipeline_info.offset) += (size_t) actual_unpack_bytes;
+            task =
+                MPIDI_OFI_create_gpu_task(MPIDI_OFI_PIPELINE_RECV, wc_buf, actual_unpack_bytes,
+                                          rreq, yreq);
+            DL_APPEND(MPIDI_OFI_global.gpu_recv_task_queue[vci_local], task);
+        } else {
+            MPIR_ERR_CHKANDJUMP(true, mpi_errno, MPI_ERR_OTHER, "**gpu_pipeline_packed");
+        }
+    }
+  fn_exit:
+    MPIR_FUNC_EXIT;
+    return mpi_errno;
+  fn_fail:
+    rreq->status.MPI_ERROR = mpi_errno;
+    goto fn_exit;
+}
+
+static int send_huge_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
     int mpi_errno = MPI_SUCCESS;
     int c, num_nics;
@@ -97,20 +253,17 @@ static int send_huge_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
         huge_send_mrs = MPIDI_OFI_REQUEST(sreq, huge.send_mrs);
 
         /* Clean up the memory region */
-        if (!MPIDI_OFI_ENABLE_MR_PROV_KEY) {
-            for (int i = 0; i < num_nics; i++) {
-                uint64_t key = fi_mr_key(huge_send_mrs[i]);
+        for (int i = 0; i < num_nics; i++) {
+            uint64_t key = fi_mr_key(huge_send_mrs[i]);
+            MPIDI_OFI_CALL(fi_close(&huge_send_mrs[i]->fid), mr_unreg);
+            if (!MPIDI_OFI_ENABLE_MR_PROV_KEY) {
                 MPIDI_OFI_mr_key_free(MPIDI_OFI_LOCAL_MR_KEY, key);
             }
         }
-
-        for (int i = 0; i < num_nics; i++) {
-            MPIDI_OFI_CALL(fi_close(&huge_send_mrs[i]->fid), mr_unreg);
-        }
         MPL_free(huge_send_mrs);
 
         if (MPIDI_OFI_REQUEST(sreq, noncontig.pack.pack_buffer)) {
-            MPIR_gpu_free_host(MPIDI_OFI_REQUEST(sreq, noncontig.pack.pack_buffer));
+            MPL_free(MPIDI_OFI_REQUEST(sreq, noncontig.pack.pack_buffer));
         }
 
         MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(sreq, datatype));
@@ -124,21 +277,18 @@ static int send_huge_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     goto fn_exit;
 }
 
-static int ssend_ack_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+static int ssend_ack_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
-    int mpi_errno;
     MPIDI_OFI_ssendack_request_t *req = (MPIDI_OFI_ssendack_request_t *) sreq;
     MPIR_FUNC_ENTER;
-    mpi_errno =
-        MPIDI_OFI_send_event(vni, NULL, req->signal_req,
-                             MPIDI_OFI_REQUEST(req->signal_req, event_id));
-
+    /* no need to cleanup the packed data buffer, it's done when the main send completes */
+    MPIDI_Request_complete_fast(req->signal_req);
     MPL_free(req);
     MPIR_FUNC_EXIT;
-    return mpi_errno;
+    return MPI_SUCCESS;
 }
 
-static int chunk_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
+static int chunk_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
 {
     int c;
     MPIR_FUNC_ENTER;
@@ -154,7 +304,7 @@ static int chunk_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     return MPI_SUCCESS;
 }
 
-static int inject_emu_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
+static int inject_emu_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
 {
     int incomplete;
     MPIR_FUNC_ENTER;
@@ -164,14 +314,14 @@ static int inject_emu_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     if (!incomplete) {
         MPL_free(MPIDI_OFI_REQUEST(req, util.inject_buf));
         MPIDI_CH4_REQUEST_FREE(req);
-        MPIDI_OFI_global.per_vni[vni].am_inflight_inject_emus -= 1;
+        MPIDI_OFI_global.per_vci[vci].am_inflight_inject_emus -= 1;
     }
 
     MPIR_FUNC_EXIT;
     return MPI_SUCCESS;
 }
 
-static int accept_probe_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int accept_probe_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     MPIR_FUNC_ENTER;
     MPIDI_OFI_dynamic_process_request_t *ctrl = (MPIDI_OFI_dynamic_process_request_t *) rreq;
@@ -183,7 +333,7 @@ static int accept_probe_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Reque
     return MPI_SUCCESS;
 }
 
-static int dynproc_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int dynproc_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     MPIR_FUNC_ENTER;
     MPIDI_OFI_dynamic_process_request_t *ctrl = (MPIDI_OFI_dynamic_process_request_t *) rreq;
@@ -192,7 +342,7 @@ static int dynproc_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Reque
     return MPI_SUCCESS;
 }
 
-static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+static int am_isend_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIDI_OFI_am_header_t *msg_hdr;
@@ -202,7 +352,7 @@ static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     msg_hdr = &MPIDI_OFI_AM_SREQ_HDR(sreq, msg_hdr);
     MPID_Request_complete(sreq);
 
-    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vni[vni].pack_buf_pool,
+    MPIDU_genq_private_pool_free_cell(MPIDI_global.per_vci[vci].pack_buf_pool,
                                       MPIDI_OFI_AM_SREQ_HDR(sreq, pack_buffer));
     MPIDI_OFI_AM_SREQ_HDR(sreq, pack_buffer) = NULL;
     mpi_errno = MPIDIG_global.origin_cbs[msg_hdr->handler_id] (sreq);
@@ -215,7 +365,7 @@ static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     goto fn_exit;
 }
 
-static int am_isend_rdma_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+static int am_isend_rdma_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
     int mpi_errno = MPI_SUCCESS;
 
@@ -229,30 +379,29 @@ static int am_isend_rdma_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Requ
     return mpi_errno;
 }
 
-static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
+static int am_isend_pipeline_event(int vci, struct fi_cq_tagged_entry *wc,
                                    MPIR_Request * dont_use_me)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_OFI_am_header_t *msg_hdr;
     MPIDI_OFI_am_send_pipeline_request_t *ofi_req;
     MPIR_Request *sreq = NULL;
 
     MPIR_FUNC_ENTER;
 
     ofi_req = MPL_container_of(wc->op_context, MPIDI_OFI_am_send_pipeline_request_t, context);
-    msg_hdr = ofi_req->msg_hdr;
+    int handler_id = ((MPIDI_OFI_am_header_t *) ofi_req->msg_hdr)->handler_id;
     sreq = ofi_req->sreq;
     MPID_Request_complete(sreq);        /* FIXME: Should not call MPIDI in NM ? */
 
-    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vni[vni].pack_buf_pool,
+    MPIDU_genq_private_pool_free_cell(MPIDI_global.per_vci[vci].pack_buf_pool,
                                       ofi_req->pack_buffer);
 
-    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vni[vni].am_hdr_buf_pool, ofi_req);
+    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vci[vci].am_hdr_buf_pool, ofi_req);
 
     int is_done = MPIDIG_am_send_async_finish_seg(sreq);
 
     if (is_done) {
-        mpi_errno = MPIDIG_global.origin_cbs[msg_hdr->handler_id] (sreq);
+        mpi_errno = MPIDIG_global.origin_cbs[handler_id] (sreq);
     }
 
     MPIR_ERR_CHECK(mpi_errno);
@@ -264,12 +413,11 @@ static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
     goto fn_exit;
 }
 
-static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int am_recv_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIDI_OFI_am_header_t *am_hdr;
     MPIDI_OFI_am_unordered_msg_t *uo_msg = NULL;
-    fi_addr_t fi_src_addr;
     uint16_t expected_seqno, next_seqno;
     MPIR_FUNC_ENTER;
 
@@ -298,23 +446,24 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     }
 #endif
 
-    expected_seqno = MPIDI_OFI_am_get_next_recv_seqno(vni, am_hdr->fi_src_addr);
+    expected_seqno = MPIDI_OFI_am_get_next_recv_seqno(vci, am_hdr->src_id);
     if (am_hdr->seqno != expected_seqno) {
         /* This message came earlier than the one that we were expecting.
          * Put it in the queue to process it later. */
         MPL_DBG_MSG_FMT(MPIDI_CH4_DBG_GENERAL, TERSE,
                         (MPL_DBG_FDEST,
-                         "Expected seqno=%d but got %d (am_type=%d addr=%" PRIx64 "). "
+                         "Expected seqno=%d but got %d (am_type=%d src_rank=%d). "
                          "Enqueueing it to the queue.\n",
-                         expected_seqno, am_hdr->seqno, am_hdr->am_type, am_hdr->fi_src_addr));
-        mpi_errno = MPIDI_OFI_am_enqueue_unordered_msg(vni, orig_buf);
+                         expected_seqno, am_hdr->seqno, am_hdr->am_type, am_hdr->src_rank));
+        mpi_errno = MPIDI_OFI_am_enqueue_unordered_msg(vci, orig_buf);
         MPIR_ERR_CHECK(mpi_errno);
         goto fn_exit;
     }
 
     /* Received an expected message */
+    uint64_t remote_id;
   fn_repeat:
-    fi_src_addr = am_hdr->fi_src_addr;
+    remote_id = am_hdr->src_id;
     next_seqno = am_hdr->seqno + 1;
 
     void *p_data;
@@ -372,7 +521,8 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     MPL_free(uo_msg);
 
     /* See if we can process other messages in the queue */
-    if ((uo_msg = MPIDI_OFI_am_claim_unordered_msg(vni, fi_src_addr, next_seqno)) != NULL) {
+    uo_msg = MPIDI_OFI_am_claim_unordered_msg(vci, remote_id, next_seqno);
+    if (uo_msg != NULL) {
         am_hdr = &uo_msg->am_hdr;
         orig_buf = am_hdr;
 #ifdef NEEDS_STRICT_ALIGNMENT
@@ -383,8 +533,8 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
         goto fn_repeat;
     }
 
-    /* Record the next expected sequence number from fi_src_addr */
-    MPIDI_OFI_am_set_next_recv_seqno(vni, fi_src_addr, next_seqno);
+    /* Record the next expected sequence number */
+    MPIDI_OFI_am_set_next_recv_seqno(vci, remote_id, next_seqno);
 
   fn_exit:
     MPIR_FUNC_EXIT;
@@ -393,7 +543,7 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     goto fn_exit;
 }
 
-static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me)
+static int am_read_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_Request *rreq;
@@ -433,105 +583,114 @@ static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     MPIDIG_REQUEST(rreq, req->target_cmpl_cb) (rreq);
     MPID_Request_complete(rreq);
   fn_exit:
-    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vni[vni].am_hdr_buf_pool, ofi_req);
+    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vci[vci].am_hdr_buf_pool, ofi_req);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
     goto fn_exit;
 }
 
-int MPIDI_OFI_dispatch_function(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
+int MPIDI_OFI_dispatch_function(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
 {
     int mpi_errno = MPI_SUCCESS;
 
     if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_SEND) {
         /* Passing the event_id as a parameter; do not need to load it from the
          * request object each time the send_event handler is invoked */
-        mpi_errno = MPIDI_OFI_send_event(vni, wc, req, MPIDI_OFI_EVENT_SEND);
+        mpi_errno = MPIDI_OFI_send_event(vci, wc, req, MPIDI_OFI_EVENT_SEND);
         goto fn_exit;
     } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_RECV) {
         /* Passing the event_id as a parameter; do not need to load it from the
          * request object each time the send_event handler is invoked */
-        mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV);
+        mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND)) {
-        mpi_errno = am_isend_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND) {
+        mpi_errno = am_isend_event(vci, wc, req);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_RDMA)) {
-        mpi_errno = am_isend_rdma_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_RDMA) {
+        mpi_errno = am_isend_rdma_event(vci, wc, req);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_PIPELINE)) {
-        mpi_errno = am_isend_pipeline_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_PIPELINE) {
+        mpi_errno = am_isend_pipeline_event(vci, wc, req);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_RECV)) {
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_RECV) {
         if (wc->flags & FI_RECV)
-            mpi_errno = am_recv_event(vni, wc, req);
+            mpi_errno = am_recv_event(vci, wc, req);
 
         if (unlikely(wc->flags & FI_MULTI_RECV)) {
             MPIDI_OFI_am_repost_request_t *am = (MPIDI_OFI_am_repost_request_t *) req;
-            mpi_errno = MPIDI_OFI_am_repost_buffer(vni, am->index);
+            mpi_errno = MPIDI_OFI_am_repost_buffer(vci, am->index);
         }
 
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_READ)) {
-        mpi_errno = am_read_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_READ) {
+        mpi_errno = am_read_event(vci, wc, req);
+        goto fn_exit;
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_SEND_GPU_PIPELINE) {
+        mpi_errno = pipeline_send_event(wc, req);
+        goto fn_exit;
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_RECV_GPU_PIPELINE_INIT) {
+        mpi_errno = pipeline_recv_event(wc, req, MPIDI_OFI_EVENT_RECV_GPU_PIPELINE_INIT);
+        goto fn_exit;
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_RECV_GPU_PIPELINE) {
+        mpi_errno = pipeline_recv_event(wc, req, MPIDI_OFI_EVENT_RECV_GPU_PIPELINE);
         goto fn_exit;
     } else if (unlikely(1)) {
         switch (MPIDI_OFI_REQUEST(req, event_id)) {
             case MPIDI_OFI_EVENT_PEEK:
-                mpi_errno = peek_event(vni, wc, req);
+                mpi_errno = peek_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_RECV_HUGE:
                 if (wc->tag & MPIDI_OFI_HUGE_SEND) {
-                    mpi_errno = MPIDI_OFI_recv_huge_event(vni, wc, req);
+                    mpi_errno = MPIDI_OFI_recv_huge_event(vci, wc, req);
                 } else {
-                    mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV_HUGE);
+                    mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV_HUGE);
                 }
                 break;
 
             case MPIDI_OFI_EVENT_RECV_PACK:
-                mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV_PACK);
+                mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV_PACK);
                 break;
 
             case MPIDI_OFI_EVENT_RECV_NOPACK:
-                mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV_NOPACK);
+                mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV_NOPACK);
                 break;
 
             case MPIDI_OFI_EVENT_SEND_HUGE:
-                mpi_errno = send_huge_event(vni, wc, req);
+                mpi_errno = send_huge_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_SEND_PACK:
-                mpi_errno = MPIDI_OFI_send_event(vni, wc, req, MPIDI_OFI_EVENT_SEND_PACK);
+                mpi_errno = MPIDI_OFI_send_event(vci, wc, req, MPIDI_OFI_EVENT_SEND_PACK);
                 break;
 
             case MPIDI_OFI_EVENT_SEND_NOPACK:
-                mpi_errno = MPIDI_OFI_send_event(vni, wc, req, MPIDI_OFI_EVENT_SEND_NOPACK);
+                mpi_errno = MPIDI_OFI_send_event(vci, wc, req, MPIDI_OFI_EVENT_SEND_NOPACK);
                 break;
 
             case MPIDI_OFI_EVENT_SSEND_ACK:
-                mpi_errno = ssend_ack_event(vni, wc, req);
+                mpi_errno = ssend_ack_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_CHUNK_DONE:
-                mpi_errno = chunk_done_event(vni, wc, req);
+                mpi_errno = chunk_done_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_HUGE_CHUNK_DONE:
-                mpi_errno = MPIDI_OFI_huge_chunk_done_event(vni, wc, req);
+                mpi_errno = MPIDI_OFI_huge_chunk_done_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_INJECT_EMU:
-                mpi_errno = inject_emu_event(vni, wc, req);
+                mpi_errno = inject_emu_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_DYNPROC_DONE:
-                mpi_errno = dynproc_done_event(vni, wc, req);
+                mpi_errno = dynproc_done_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_ACCEPT_PROBE:
-                mpi_errno = accept_probe_event(vni, wc, req);
+                mpi_errno = accept_probe_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_ABORT:
@@ -546,7 +705,7 @@ int MPIDI_OFI_dispatch_function(int vni, struct fi_cq_tagged_entry *wc, MPIR_Req
     return mpi_errno;
 }
 
-int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
+int MPIDI_OFI_handle_cq_error(int vci, int nic, ssize_t ret)
 {
     int mpi_errno = MPI_SUCCESS;
     struct fi_cq_err_entry e;
@@ -555,7 +714,7 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
     ssize_t ret_cqerr;
     MPIR_FUNC_ENTER;
 
-    int ctx_idx = MPIDI_OFI_get_ctx_index(NULL, vni, nic);
+    int ctx_idx = MPIDI_OFI_get_ctx_index(vci, nic);
     switch (ret) {
         case -FI_EAVAIL:
             /* Provide separate error buffer for each thread. This makes the
@@ -576,13 +735,13 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
 
                     switch (req->kind) {
                         case MPIR_REQUEST_KIND__SEND:
-                            mpi_errno = MPIDI_OFI_dispatch_function(vni, NULL, req);
+                            mpi_errno = MPIDI_OFI_dispatch_function(vci, NULL, req);
                             break;
 
                         case MPIR_REQUEST_KIND__RECV:
                             req->status.MPI_ERROR = MPI_ERR_TRUNCATE;
                             mpi_errno =
-                                MPIDI_OFI_dispatch_function(vni, (struct fi_cq_tagged_entry *) &e,
+                                MPIDI_OFI_dispatch_function(vci, (struct fi_cq_tagged_entry *) &e,
                                                             req);
                             break;
 
@@ -596,20 +755,39 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
 
                 case FI_ECANCELED:
                     req = MPIDI_OFI_context_to_request(e.op_context);
-                    MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(req, datatype));
-                    MPIR_STATUS_SET_CANCEL_BIT(req->status, TRUE);
+                    /* Clean up the request. Reference MPIDI_OFI_recv_event.
+                     * NOTE: assuming only the receive request can be cancelled and reach here
+                     */
+                    int event_id = MPIDI_OFI_REQUEST(req, event_id);
+                    if (event_id == MPIDI_OFI_EVENT_DYNPROC_DONE) {
+                        dynproc_done_event(vci, e.op_context, req);
+                    } else {
+                        /* assume it is a pending recv */
+                        MPIR_STATUS_SET_CANCEL_BIT(req->status, TRUE);
+                        MPIR_STATUS_SET_COUNT(req->status, 0);
+                        if ((event_id == MPIDI_OFI_EVENT_RECV_PACK ||
+                             event_id == MPIDI_OFI_EVENT_GET_HUGE) &&
+                            MPIDI_OFI_REQUEST(req, noncontig.pack.pack_buffer)) {
+                            MPL_free(MPIDI_OFI_REQUEST(req, noncontig.pack.pack_buffer));
+                        } else if (MPIDI_OFI_ENABLE_PT2PT_NOPACK &&
+                                   event_id == MPIDI_OFI_EVENT_RECV_NOPACK &&
+                                   MPIDI_OFI_REQUEST(req, noncontig.nopack)) {
+                            MPL_free(MPIDI_OFI_REQUEST(req, noncontig.nopack));
+                        }
+                        MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(req, datatype));
+                        MPIDI_Request_complete_fast(req);
+                    }
                     break;
 
                 case FI_ENOMSG:
                     req = MPIDI_OFI_context_to_request(e.op_context);
-                    mpi_errno = peek_empty_event(vni, NULL, req);
+                    mpi_errno = peek_empty_event(vci, NULL, req);
                     break;
 
                 default:
                     MPIR_ERR_SETFATALANDJUMP4(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
                                               "**ofid_poll %s %d %s %s", __SHORT_FILE__,
                                               __LINE__, __func__, fi_strerror(e.err));
-                    break;
             }
 
             break;
@@ -618,7 +796,6 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
             MPIR_ERR_SETFATALANDJUMP4(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
                                       "**ofid_poll %s %d %s %s", __SHORT_FILE__, __LINE__,
                                       __func__, fi_strerror(errno));
-            break;
     }
 
   fn_exit:
