@@ -20,7 +20,7 @@ MPL_STATIC_INLINE_PREFIX void MPIDI_UCX_send_cmpl_cb(void *request, ucs_status_t
 
     if (unlikely(status == UCS_ERR_CANCELED))
         MPIR_STATUS_SET_CANCEL_BIT(req->status, TRUE);
-    MPIDIU_request_complete(req);
+    MPIDI_Request_complete_fast(req);
     ucp_request->req = NULL;
     ucp_request_release(ucp_request);
 
@@ -36,7 +36,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_send(const void *buf,
                                             int context_offset,
                                             MPIDI_av_entry_t * addr,
                                             MPIR_Request ** request,
-                                            int vni_src, int vni_dst, int have_request, int is_sync)
+                                            int vci_src, int vci_dst, int have_request, int is_sync)
 {
     int dt_contig;
     size_t data_sz;
@@ -55,7 +55,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_send(const void *buf,
         .cb.send = MPIDI_UCX_send_cmpl_cb,
     };
 
-    ep = MPIDI_UCX_AV_TO_EP(addr, vni_src, vni_dst);
+    ep = MPIDI_UCX_AV_TO_EP(addr, vci_src, vci_dst);
     ucx_tag = MPIDI_UCX_init_tag(comm->context_id + context_offset, comm->rank, tag);
     MPIDI_Datatype_get_info(count, datatype, dt_contig, data_sz, dt_ptr, dt_true_lb);
 
@@ -84,12 +84,15 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_send(const void *buf,
 
     if (ucp_request) {
         if (req == NULL) {
-            req = MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__SEND, vni_src, 2);
+            req = MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__SEND, vci_src, 2);
+            MPIR_ERR_CHKANDSTMT(!req, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
+            req->comm = comm;
+            MPIR_Comm_add_ref(comm);
         } else {
             MPIR_Request_add_ref(req);
         }
         ucp_request->req = req;
-        MPIDI_UCX_REQ(req).ucp_request = ucp_request;
+        MPIDI_UCX_REQ(req).s.ucp_request = ucp_request;
     } else if (req != NULL) {
         MPIR_cc_set(&req->cc, 0);
     } else if (have_request) {
@@ -104,20 +107,34 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_send(const void *buf,
     goto fn_exit;
 }
 
-MPL_STATIC_INLINE_PREFIX int MPIDI_NM_isend_coll(const void *buf, MPI_Aint count,
-                                                 MPI_Datatype datatype, int rank, int tag,
-                                                 MPIR_Comm * comm, int context_offset,
-                                                 MPIDI_av_entry_t * addr, MPIR_Request ** request,
-                                                 MPIR_Errflag_t * errflag)
+#define MPIDI_UCX_SEND_VNIS(vci_src_, vci_dst_) \
+    do { \
+        MPIDI_EXPLICIT_VCIS(comm, attr, comm->rank, rank, vci_src_, vci_dst_); \
+        if (vci_src_ == 0 && vci_dst_ == 0) { \
+            vci_src = MPIDI_get_vci(SRC_VCI_FROM_SENDER, comm, comm->rank, rank, tag); \
+            vci_dst = MPIDI_get_vci(DST_VCI_FROM_SENDER, comm, comm->rank, rank, tag); \
+        } \
+    } while (0)
+
+MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_isend(const void *buf,
+                                                MPI_Aint count,
+                                                MPI_Datatype datatype,
+                                                int rank,
+                                                int tag,
+                                                MPIR_Comm * comm, int attr,
+                                                MPIDI_av_entry_t * addr, MPIR_Request ** request)
 {
-    int mpi_errno = MPI_SUCCESS;
-
+    int mpi_errno;
     MPIR_FUNC_ENTER;
-    int vni_src = MPIDI_UCX_get_vni(SRC_VCI_FROM_SENDER, comm, comm->rank, rank, tag);
-    int vni_dst = MPIDI_UCX_get_vni(DST_VCI_FROM_SENDER, comm, comm->rank, rank, tag);
-    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vni_src).lock);
 
-    switch (*errflag) {
+    int context_offset = MPIR_PT2PT_ATTR_CONTEXT_OFFSET(attr);
+    int is_sync = MPIR_PT2PT_ATTR_GET_SYNCFLAG(attr) ? 1 : 0;
+
+    int vci_src, vci_dst;
+    MPIDI_UCX_SEND_VNIS(vci_src, vci_dst);
+
+    MPIR_Errflag_t errflag = MPIR_PT2PT_ATTR_GET_ERRFLAG(attr);
+    switch (errflag) {
         case MPIR_ERR_NONE:
             break;
         case MPIR_ERR_PROC_FAILED:
@@ -127,56 +144,10 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_NM_isend_coll(const void *buf, MPI_Aint count
             MPIR_TAG_SET_ERROR_BIT(tag);
     }
 
+    MPIDI_UCX_THREAD_CS_ENTER_VCI(vci_src);
     mpi_errno = MPIDI_UCX_send(buf, count, datatype, rank, tag, comm, context_offset,
-                               addr, request, vni_src, vni_dst, 1, 0);
-
-    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vni_src).lock);
-    MPIR_FUNC_EXIT;
-
-    return mpi_errno;
-}
-
-MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_isend(const void *buf,
-                                                MPI_Aint count,
-                                                MPI_Datatype datatype,
-                                                int rank,
-                                                int tag,
-                                                MPIR_Comm * comm, int context_offset,
-                                                MPIDI_av_entry_t * addr, MPIR_Request ** request)
-{
-    int mpi_errno;
-    int vni_src = MPIDI_UCX_get_vni(SRC_VCI_FROM_SENDER, comm, comm->rank, rank, tag);
-    int vni_dst = MPIDI_UCX_get_vni(DST_VCI_FROM_SENDER, comm, comm->rank, rank, tag);
-
-    MPIR_FUNC_ENTER;
-
-    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vni_src).lock);
-    mpi_errno = MPIDI_UCX_send(buf, count, datatype, rank, tag, comm, context_offset,
-                               addr, request, vni_src, vni_dst, 1, 0);
-    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vni_src).lock);
-
-    MPIR_FUNC_EXIT;
-    return mpi_errno;
-}
-
-MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_issend(const void *buf,
-                                                 MPI_Aint count,
-                                                 MPI_Datatype datatype,
-                                                 int rank,
-                                                 int tag,
-                                                 MPIR_Comm * comm, int context_offset,
-                                                 MPIDI_av_entry_t * addr, MPIR_Request ** request)
-{
-    int mpi_errno;
-    int vni_src = MPIDI_UCX_get_vni(SRC_VCI_FROM_SENDER, comm, comm->rank, rank, tag);
-    int vni_dst = MPIDI_UCX_get_vni(DST_VCI_FROM_SENDER, comm, comm->rank, rank, tag);
-
-    MPIR_FUNC_ENTER;
-
-    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vni_src).lock);
-    mpi_errno = MPIDI_UCX_send(buf, count, datatype, rank, tag, comm, context_offset,
-                               addr, request, vni_src, vni_dst, 1, 1);
-    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vni_src).lock);
+                               addr, request, vci_src, vci_dst, 1, is_sync);
+    MPIDI_UCX_THREAD_CS_EXIT_VCI(vci_src);
 
     MPIR_FUNC_EXIT;
     return mpi_errno;
@@ -188,9 +159,9 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_cancel_send(MPIR_Request * sreq)
 
     if (!MPIR_Request_is_complete(sreq)) {
         int vci = MPIDI_Request_get_vci(sreq);
-        MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vci).lock);
-        ucp_request_cancel(MPIDI_UCX_global.ctx[vci].worker, MPIDI_UCX_REQ(sreq).ucp_request);
-        MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vci).lock);
+        MPIDI_UCX_THREAD_CS_ENTER_VCI(vci);
+        ucp_request_cancel(MPIDI_UCX_global.ctx[vci].worker, MPIDI_UCX_REQ(sreq).s.ucp_request);
+        MPIDI_UCX_THREAD_CS_EXIT_VCI(vci);
     }
 
     MPIR_FUNC_EXIT;
