@@ -6,7 +6,12 @@
 #include "mpidimpl.h"
 #include "xpmem_post.h"
 
-int MPIDI_XPMEM_mpi_init_hook(int rank, int size, int *tag_bits)
+int MPIDI_XPMEM_init_local(void)
+{
+    return MPI_SUCCESS;
+}
+
+int MPIDI_XPMEM_init_world(void)
 {
     return MPI_SUCCESS;
 }
