@@ -20,7 +20,8 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_pready_range(int partition_low,
     return MPIDIG_mpi_pready_range(partition_low, partition_high, request);
 }
 
-MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_pready_list(int length, int array_of_partitions[],
+MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_pready_list(int length,
+                                                         const int array_of_partitions[],
                                                          MPIR_Request * request)
 {
     return MPIDIG_mpi_pready_list(length, array_of_partitions, request);
