@@ -10,10 +10,10 @@
 #include "posix_impl.h"
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_compute_accumulate(void *origin_addr,
-                                                            int origin_count,
+                                                            MPI_Aint origin_count,
                                                             MPI_Datatype origin_datatype,
                                                             void *target_addr,
-                                                            int target_count,
+                                                            MPI_Aint target_count,
                                                             MPI_Datatype target_datatype, MPI_Op op,
                                                             int mapped_device)
 {
@@ -63,8 +63,8 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_compute_accumulate(void *origin_addr,
     MPIR_ERR_CHKANDJUMP(packed_buf == NULL, mpi_errno, MPI_ERR_NO_MEM, "**nomem");
 
     MPI_Aint actual_pack_bytes;
-    mpi_errno = MPIR_Typerep_pack(origin_addr, origin_count, origin_datatype, 0,
-                                  packed_buf, total_len, &actual_pack_bytes);
+    mpi_errno = MPIR_Typerep_pack(origin_addr, origin_count, origin_datatype, 0, packed_buf,
+                                  total_len, &actual_pack_bytes, MPIR_TYPEREP_FLAG_NONE);
     MPIR_ERR_CHECK(mpi_errno);
     MPIR_Assert(actual_pack_bytes == total_len);
 
@@ -82,11 +82,11 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_compute_accumulate(void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_put(const void *origin_addr,
-                                                int origin_count,
+                                                MPI_Aint origin_count,
                                                 MPI_Datatype origin_datatype,
                                                 int target_rank,
                                                 MPI_Aint target_disp,
-                                                int target_count, MPI_Datatype target_datatype,
+                                                MPI_Aint target_count, MPI_Datatype target_datatype,
                                                 MPIR_Win * win, MPIDI_winattr_t winattr)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -116,7 +116,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_put(const void *origin_addr,
     if (winattr & MPIDI_WINATTR_MR_PREFERRED) {
         /* If MR-preferred is set, switch to nonblocking version which may slightly
          * increase per-op+flush overhead. */
-        MPIR_Typerep_req typerep_req = MPIR_TYPEREP_REQ_NULL;
+        MPIR_Typerep_req typerep_req = { MPIR_TYPEREP_REQ_NULL, NULL };
         mpi_errno = MPIR_Ilocalcopy(origin_addr, origin_count, origin_datatype,
                                     (char *) base + disp_unit * target_disp, target_count,
                                     target_datatype, &typerep_req);
@@ -136,11 +136,11 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_put(const void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_get(void *origin_addr,
-                                                int origin_count,
+                                                MPI_Aint origin_count,
                                                 MPI_Datatype origin_datatype,
                                                 int target_rank,
                                                 MPI_Aint target_disp,
-                                                int target_count, MPI_Datatype target_datatype,
+                                                MPI_Aint target_count, MPI_Datatype target_datatype,
                                                 MPIR_Win * win, MPIDI_winattr_t winattr)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -170,7 +170,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_get(void *origin_addr,
     if (winattr & MPIDI_WINATTR_MR_PREFERRED) {
         /* If MR-preferred is set, switch to nonblocking version which may slightly
          * increase per-op+flush overhead. */
-        MPIR_Typerep_req typerep_req = MPIR_TYPEREP_REQ_NULL;
+        MPIR_Typerep_req typerep_req = { MPIR_TYPEREP_REQ_NULL, NULL };
         mpi_errno = MPIR_Ilocalcopy((char *) base + disp_unit * target_disp, target_count,
                                     target_datatype, origin_addr, origin_count, origin_datatype,
                                     &typerep_req);
@@ -189,14 +189,14 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_get(void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_get_accumulate(const void *origin_addr,
-                                                           int origin_count,
+                                                           MPI_Aint origin_count,
                                                            MPI_Datatype origin_datatype,
                                                            void *result_addr,
-                                                           int result_count,
+                                                           MPI_Aint result_count,
                                                            MPI_Datatype result_datatype,
                                                            int target_rank,
                                                            MPI_Aint target_disp,
-                                                           int target_count,
+                                                           MPI_Aint target_count,
                                                            MPI_Datatype target_datatype, MPI_Op op,
                                                            MPIR_Win * win, MPIDI_winattr_t winattr)
 {
@@ -261,11 +261,11 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_get_accumulate(const void *origin_ad
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_accumulate(const void *origin_addr,
-                                                       int origin_count,
+                                                       MPI_Aint origin_count,
                                                        MPI_Datatype origin_datatype,
                                                        int target_rank,
                                                        MPI_Aint target_disp,
-                                                       int target_count,
+                                                       MPI_Aint target_count,
                                                        MPI_Datatype target_datatype, MPI_Op op,
                                                        MPIR_Win * win, MPIDI_winattr_t winattr)
 {
@@ -321,12 +321,13 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_do_accumulate(const void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_put(const void *origin_addr,
-                                                 int origin_count,
+                                                 MPI_Aint origin_count,
                                                  MPI_Datatype origin_datatype,
                                                  int target_rank,
                                                  MPI_Aint target_disp,
-                                                 int target_count, MPI_Datatype target_datatype,
-                                                 MPIR_Win * win, MPIDI_winattr_t winattr)
+                                                 MPI_Aint target_count,
+                                                 MPI_Datatype target_datatype, MPIR_Win * win,
+                                                 MPIDI_winattr_t winattr)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_ENTER;
@@ -352,12 +353,13 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_put(const void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_get(void *origin_addr,
-                                                 int origin_count,
+                                                 MPI_Aint origin_count,
                                                  MPI_Datatype origin_datatype,
                                                  int target_rank,
                                                  MPI_Aint target_disp,
-                                                 int target_count, MPI_Datatype target_datatype,
-                                                 MPIR_Win * win, MPIDI_winattr_t winattr)
+                                                 MPI_Aint target_count,
+                                                 MPI_Datatype target_datatype, MPIR_Win * win,
+                                                 MPIDI_winattr_t winattr)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_ENTER;
@@ -383,17 +385,16 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_get(void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rput(const void *origin_addr,
-                                                  int origin_count,
+                                                  MPI_Aint origin_count,
                                                   MPI_Datatype origin_datatype,
                                                   int target_rank,
                                                   MPI_Aint target_disp,
-                                                  int target_count,
+                                                  MPI_Aint target_count,
                                                   MPI_Datatype target_datatype,
                                                   MPIR_Win * win, MPIDI_winattr_t winattr,
                                                   MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIR_Request *sreq = NULL;
     MPIR_FUNC_ENTER;
 
     /* CH4 schedules operation only based on process locality.
@@ -412,12 +413,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rput(const void *origin_addr,
                                    target_rank, target_disp, target_count, target_datatype, win,
                                    winattr);
     if (mpi_errno == MPI_SUCCESS) {
-        /* create a completed request for user. */
-        MPIDI_CH4_REQUEST_CREATE(sreq, MPIR_REQUEST_KIND__RMA, vci, 2);
-        MPIR_Assert(sreq);
-
-        MPID_Request_complete(sreq);
-        *request = sreq;
+        *request = MPIR_Request_create_complete(MPIR_REQUEST_KIND__RMA);
     }
     MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vci).lock);
 
@@ -479,9 +475,9 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_compare_and_swap(const void *origin
         MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vci).lock);
     }
 
-    MPIR_Typerep_copy(result_addr, target_addr, dtype_sz);
+    MPIR_Typerep_copy(result_addr, target_addr, dtype_sz, MPIR_TYPEREP_FLAG_NONE);
     if (MPIR_Compare_equal(compare_addr, target_addr, datatype)) {
-        MPIR_Typerep_copy(target_addr, origin_addr, dtype_sz);
+        MPIR_Typerep_copy(target_addr, origin_addr, dtype_sz, MPIR_TYPEREP_FLAG_NONE);
     }
 
     if (winattr & MPIDI_WINATTR_SHM_ALLOCATED) {
@@ -498,18 +494,17 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_compare_and_swap(const void *origin
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_raccumulate(const void *origin_addr,
-                                                         int origin_count,
+                                                         MPI_Aint origin_count,
                                                          MPI_Datatype origin_datatype,
                                                          int target_rank,
                                                          MPI_Aint target_disp,
-                                                         int target_count,
+                                                         MPI_Aint target_count,
                                                          MPI_Datatype target_datatype,
                                                          MPI_Op op, MPIR_Win * win,
                                                          MPIDI_winattr_t winattr,
                                                          MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIR_Request *sreq = NULL;
     MPIR_FUNC_ENTER;
 
     /* CH4 schedules operation only based on process locality.
@@ -522,20 +517,12 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_raccumulate(const void *origin_addr
         goto fn_exit;
     }
 
-    int vci = MPIDI_WIN(win, am_vci);
     mpi_errno = MPIDI_POSIX_do_accumulate(origin_addr, origin_count, origin_datatype,
                                           target_rank, target_disp, target_count,
                                           target_datatype, op, win, winattr);
     MPIR_ERR_CHECK(mpi_errno);
 
-    /* create a completed request for user. */
-    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vci).lock);
-    MPIDI_CH4_REQUEST_CREATE(sreq, MPIR_REQUEST_KIND__RMA, vci, 2);
-    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vci).lock);
-    MPIR_Assert(sreq);
-
-    MPID_Request_complete(sreq);
-    *request = sreq;
+    *request = MPIR_Request_create_complete(MPIR_REQUEST_KIND__RMA);
 
   fn_exit:
     MPIR_FUNC_EXIT;
@@ -545,21 +532,20 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_raccumulate(const void *origin_addr
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rget_accumulate(const void *origin_addr,
-                                                             int origin_count,
+                                                             MPI_Aint origin_count,
                                                              MPI_Datatype origin_datatype,
                                                              void *result_addr,
-                                                             int result_count,
+                                                             MPI_Aint result_count,
                                                              MPI_Datatype result_datatype,
                                                              int target_rank,
                                                              MPI_Aint target_disp,
-                                                             int target_count,
+                                                             MPI_Aint target_count,
                                                              MPI_Datatype target_datatype,
                                                              MPI_Op op, MPIR_Win * win,
                                                              MPIDI_winattr_t winattr,
                                                              MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIR_Request *sreq = NULL;
     MPIR_FUNC_ENTER;
 
     /* CH4 schedules operation only based on process locality.
@@ -573,22 +559,13 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rget_accumulate(const void *origin_
         goto fn_exit;
     }
 
-    int vci = MPIDI_WIN(win, am_vci);
-
     mpi_errno = MPIDI_POSIX_do_get_accumulate(origin_addr, origin_count, origin_datatype,
                                               result_addr, result_count, result_datatype,
                                               target_rank, target_disp, target_count,
                                               target_datatype, op, win, winattr);
     MPIR_ERR_CHECK(mpi_errno);
 
-    /* create a completed request for user. */
-    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vci).lock);
-    MPIDI_CH4_REQUEST_CREATE(sreq, MPIR_REQUEST_KIND__RMA, vci, 2);
-    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vci).lock);
-    MPIR_Assert(sreq);
-
-    MPID_Request_complete(sreq);
-    *request = sreq;
+    *request = MPIR_Request_create_complete(MPIR_REQUEST_KIND__RMA);
 
   fn_exit:
     MPIR_FUNC_EXIT;
@@ -649,7 +626,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_fetch_and_op(const void *origin_add
         MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vci).lock);
     }
 
-    MPIR_Typerep_copy(result_addr, target_addr, dtype_sz);
+    MPIR_Typerep_copy(result_addr, target_addr, dtype_sz, MPIR_TYPEREP_FLAG_NONE);
 
     if (op != MPI_NO_OP) {
         /* We need to make sure op is valid here.
@@ -676,17 +653,16 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_fetch_and_op(const void *origin_add
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rget(void *origin_addr,
-                                                  int origin_count,
+                                                  MPI_Aint origin_count,
                                                   MPI_Datatype origin_datatype,
                                                   int target_rank,
                                                   MPI_Aint target_disp,
-                                                  int target_count,
+                                                  MPI_Aint target_count,
                                                   MPI_Datatype target_datatype,
                                                   MPIR_Win * win, MPIDI_winattr_t winattr,
                                                   MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIR_Request *sreq = NULL;
     MPIR_FUNC_ENTER;
 
     /* CH4 schedules operation only based on process locality.
@@ -705,12 +681,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rget(void *origin_addr,
                                    target_rank, target_disp, target_count, target_datatype, win,
                                    winattr);
     if (mpi_errno == MPI_SUCCESS) {
-        /* create a completed request for user. */
-        MPIDI_CH4_REQUEST_CREATE(sreq, MPIR_REQUEST_KIND__RMA, vci, 2);
-        MPIR_Assert(sreq);
-
-        MPID_Request_complete(sreq);
-        *request = sreq;
+        *request = MPIR_Request_create_complete(MPIR_REQUEST_KIND__RMA);
     }
     MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vci).lock);
 
@@ -720,14 +691,14 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_rget(void *origin_addr,
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_get_accumulate(const void *origin_addr,
-                                                            int origin_count,
+                                                            MPI_Aint origin_count,
                                                             MPI_Datatype origin_datatype,
                                                             void *result_addr,
-                                                            int result_count,
+                                                            MPI_Aint result_count,
                                                             MPI_Datatype result_datatype,
                                                             int target_rank,
                                                             MPI_Aint target_disp,
-                                                            int target_count,
+                                                            MPI_Aint target_count,
                                                             MPI_Datatype target_datatype, MPI_Op op,
                                                             MPIR_Win * win, MPIDI_winattr_t winattr)
 {
@@ -754,11 +725,11 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_get_accumulate(const void *origin_a
 }
 
 MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_mpi_accumulate(const void *origin_addr,
-                                                        int origin_count,
+                                                        MPI_Aint origin_count,
                                                         MPI_Datatype origin_datatype,
                                                         int target_rank,
                                                         MPI_Aint target_disp,
-                                                        int target_count,
+                                                        MPI_Aint target_count,
                                                         MPI_Datatype target_datatype, MPI_Op op,
                                                         MPIR_Win * win, MPIDI_winattr_t winattr)
 {
