@@ -79,6 +79,7 @@ cvars:
         kary      - kary tree type
         knomial_1 - knomial_1 tree type (ranks are added in order from the left side)
         knomial_2 - knomial_2 tree type (ranks are added in order from the right side)
+        	    knomial_2 is only supported with non topology aware trees.
 
     - name        : MPIR_CVAR_REDUCE_INTRANODE_TREE_KVAL
       category    : COLLECTIVE
@@ -102,6 +103,7 @@ cvars:
         kary      - kary tree type
         knomial_1 - knomial_1 tree type (ranks are added in order from the left side)
         knomial_2 - knomial_2 tree type (ranks are added in order from the right side)
+        	    knomial_2 is only supported with non topology aware trees.
 
     - name        : MPIR_CVAR_ENABLE_INTRANODE_TOPOLOGY_AWARE_TREES
       category    : COLLECTIVE
@@ -125,8 +127,8 @@ cvars:
         order, first child to be added is the last one to be processed in traversal)
         The tree radix and tree type of package_leaders and per_package tree is
         MPIR_CVAR_BCAST{REDUCE}_INTRANODE_TREE_KVAL and MPIR_CVAR_BCAST{REDUCE}_INTRANODE_TREE_TYPE
-        respectively for bast and reduce. But of as now topology aware trees are only kary. knomial
-        is to be implemented.
+        respectively for bast and reduce. But of as now topology aware trees are only kary and knomial_1.
+        knomial_2 is not implemented.
 
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
@@ -136,39 +138,33 @@ cvars:
 #include "topotree.h"
 #include "topotree_util.h"
 
-#define RELEASE_GATHER_FIELD(comm, field)                   \
-    MPIDI_POSIX_COMM(comm, release_gather).field
-
 
 MPIDI_POSIX_release_gather_tree_type_t MPIDI_POSIX_Bcast_tree_type, MPIDI_POSIX_Reduce_tree_type;
 
+static int get_tree_type(const char *tree_type_name)
+{
+    if (0 == strcmp(tree_type_name, "kary"))
+        return MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KARY;
+    else if (0 == strcmp(tree_type_name, "knomial_1"))
+        return MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KNOMIAL_1;
+    else if (0 == strcmp(tree_type_name, "knomial_2"))
+        return MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KNOMIAL_2;
+    else
+        return MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KARY;
+}
+
 /* Initialize the release_gather struct to NULL */
 int MPIDI_POSIX_mpi_release_gather_comm_init_null(MPIR_Comm * comm_ptr)
 {
+
     MPIR_FUNC_ENTER;
 
     RELEASE_GATHER_FIELD(comm_ptr, num_collective_calls) = 0;
-
-    if (0 == strcmp(MPIR_CVAR_BCAST_INTRANODE_TREE_TYPE, "kary"))
-        MPIDI_POSIX_Bcast_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KARY;
-    else if (0 == strcmp(MPIR_CVAR_BCAST_INTRANODE_TREE_TYPE, "knomial_1"))
-        MPIDI_POSIX_Bcast_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KNOMIAL_1;
-    else if (0 == strcmp(MPIR_CVAR_BCAST_INTRANODE_TREE_TYPE, "knomial_2"))
-        MPIDI_POSIX_Bcast_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KNOMIAL_2;
-    else
-        MPIDI_POSIX_Bcast_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KARY;
-
-    if (0 == strcmp(MPIR_CVAR_REDUCE_INTRANODE_TREE_TYPE, "kary"))
-        MPIDI_POSIX_Reduce_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KARY;
-    else if (0 == strcmp(MPIR_CVAR_REDUCE_INTRANODE_TREE_TYPE, "knomial_1"))
-        MPIDI_POSIX_Reduce_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KNOMIAL_1;
-    else if (0 == strcmp(MPIR_CVAR_REDUCE_INTRANODE_TREE_TYPE, "knomial_2"))
-        MPIDI_POSIX_Reduce_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KNOMIAL_2;
-    else
-        MPIDI_POSIX_Reduce_tree_type = MPIDI_POSIX_RELEASE_GATHER_TREE_TYPE_KARY;
-
     RELEASE_GATHER_FIELD(comm_ptr, is_initialized) = 0;
 
+    NB_RELEASE_GATHER_FIELD(comm_ptr, num_collective_calls) = 0;
+    NB_RELEASE_GATHER_FIELD(comm_ptr, is_initialized) = 0;
+
     MPIR_FUNC_EXIT;
     return MPI_SUCCESS;
 }
@@ -188,7 +184,6 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
     size_t flags_shm_size = 0;
     const long pg_sz = sysconf(_SC_PAGESIZE);
     MPIR_Errflag_t errflag = MPIR_ERR_NONE;
-    bool mapfail_flag = false;
     int topotree_fail[2] = { -1, -1 };  /* -1 means topo trees not created due to reasons like not
                                          * specifying binding, no hwloc etc. 0 means topo trees were
                                          * created successfully. 1 means topo trees were created
@@ -202,6 +197,20 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
      * reduce buffer (divided into multiple cells) per rank. */
 
     if (RELEASE_GATHER_FIELD(comm_ptr, is_initialized) == 0) {
+        RELEASE_GATHER_FIELD(comm_ptr, is_initialized) = 1;
+        /* CVARs may get updated. Turn them into per-comm settings */
+        RELEASE_GATHER_FIELD(comm_ptr, bcast_tree_type) =
+            get_tree_type(MPIR_CVAR_BCAST_INTRANODE_TREE_TYPE);
+        RELEASE_GATHER_FIELD(comm_ptr, bcast_tree_kval) = MPIR_CVAR_BCAST_INTRANODE_TREE_KVAL;
+        RELEASE_GATHER_FIELD(comm_ptr, bcast_shm_size) =
+            MPIR_CVAR_BCAST_INTRANODE_BUFFER_TOTAL_SIZE;
+        RELEASE_GATHER_FIELD(comm_ptr, bcast_num_cells) = MPIR_CVAR_BCAST_INTRANODE_NUM_CELLS;
+        RELEASE_GATHER_FIELD(comm_ptr, reduce_tree_type) =
+            get_tree_type(MPIR_CVAR_REDUCE_INTRANODE_TREE_TYPE);
+        RELEASE_GATHER_FIELD(comm_ptr, reduce_tree_kval) = MPIR_CVAR_REDUCE_INTRANODE_TREE_KVAL;
+        RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size) =
+            MPIR_CVAR_REDUCE_INTRANODE_BUFFER_TOTAL_SIZE;
+        RELEASE_GATHER_FIELD(comm_ptr, reduce_num_cells) = MPIR_CVAR_REDUCE_INTRANODE_NUM_CELLS;
         /* release_gather based collectives have not been used before on this comm */
         initialize_flags = true;
         if (operation == MPIDI_POSIX_RELEASE_GATHER_OPCODE_BCAST) {
@@ -234,7 +243,7 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
             /* Calculate the amount of memory that would be allocated for flags */
             flags_shm_size = MPIDI_POSIX_RELEASE_GATHER_FLAG_SPACE_PER_RANK * num_ranks;
             /* Reset flags_shm_size so that the data buffers are aligned to the system pages */
-            flags_num_pages = flags_shm_size / (int) (pg_sz);
+            flags_num_pages = (int) (flags_shm_size / pg_sz);
             if (flags_shm_size % pg_sz != 0) {
                 flags_num_pages++;
             }
@@ -245,31 +254,41 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
         }
 
         if (initialize_bcast_buf) {
-            memory_to_be_allocated += MPIR_CVAR_BCAST_INTRANODE_BUFFER_TOTAL_SIZE;
+            memory_to_be_allocated += RELEASE_GATHER_FIELD(comm_ptr, bcast_shm_size);
         }
         if (initialize_reduce_buf) {
-            memory_to_be_allocated += (num_ranks * MPIR_CVAR_REDUCE_INTRANODE_BUFFER_TOTAL_SIZE);
+            memory_to_be_allocated += (num_ranks * RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size));
         }
 
         if (rank == 0) {
             /* rank 0 decides if more memory can be created and broadcasts the decision to other ranks */
-            tmp_shm_counter = MPL_atomic_acquire_load_uint64(MPIDI_POSIX_shm_limit_counter);
+            tmp_shm_counter =
+                (size_t) MPL_atomic_acquire_load_uint64(MPIDI_POSIX_shm_limit_counter);
 
             /* Check if it is allowed to create more shm on this node */
             if ((tmp_shm_counter + memory_to_be_allocated) >
                 (MPIR_CVAR_COLL_SHM_LIMIT_PER_NODE * 1024)) {
                 /* cannot create more shm, fallback to MPIR level algorithms, and broadcast the decision to other ranks */
+                if (MPIR_CVAR_COLLECTIVE_FALLBACK == MPIR_CVAR_COLLECTIVE_FALLBACK_print) {
+                    fprintf(stderr,
+                            "Intra-node collectives about to allocate more shared memory than \
+                            the specified limit through MPIR_CVAR_COLL_SHM_LIMIT_PER_NODE. Fallback \
+                            to other algorithms.\n");
+                }
+
                 fallback = 1;
-                MPIR_Bcast_impl(&fallback, 1, MPI_INT, 0, comm_ptr, &errflag);
+                MPIR_Bcast_impl(&fallback, 1, MPI_INT, 0, comm_ptr, errflag);
                 MPIR_ERR_SETANDJUMP(mpi_errno_ret, MPI_ERR_NO_MEM, "**nomem");
             } else {
+                /* More shm can be created, update the shared counter */
+                MPL_atomic_fetch_add_uint64(MPIDI_POSIX_shm_limit_counter, memory_to_be_allocated);
                 fallback = 0;
-                mpi_errno = MPIR_Bcast_impl(&fallback, 1, MPI_INT, 0, comm_ptr, &errflag);
-                MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+                mpi_errno = MPIR_Bcast_impl(&fallback, 1, MPI_INT, 0, comm_ptr, errflag);
+                MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
             }
         } else {
-            mpi_errno = MPIR_Bcast_impl(&fallback, 1, MPI_INT, 0, comm_ptr, &errflag);
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            mpi_errno = MPIR_Bcast_impl(&fallback, 1, MPI_INT, 0, comm_ptr, errflag);
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
             if (fallback) {
                 MPIR_ERR_SETANDJUMP(mpi_errno_ret, MPI_ERR_NO_MEM, "**nomem");
             }
@@ -290,28 +309,24 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
             /* Topology aware trees are created only when the user has specified process binding */
             if (MPIR_hwtopo_is_initialized()) {
                 mpi_errno =
-                    MPIDI_SHM_topology_tree_init(comm_ptr, 0, MPIR_CVAR_BCAST_INTRANODE_TREE_KVAL,
+                    MPIDI_SHM_topology_tree_init(comm_ptr, 0,
+                                                 RELEASE_GATHER_FIELD(comm_ptr, bcast_tree_kval),
+                                                 RELEASE_GATHER_FIELD(comm_ptr, bcast_tree_type),
                                                  &release_gather_info_ptr->bcast_tree,
                                                  &topotree_fail[0],
-                                                 MPIR_CVAR_REDUCE_INTRANODE_TREE_KVAL,
+                                                 RELEASE_GATHER_FIELD(comm_ptr, reduce_tree_kval),
+                                                 RELEASE_GATHER_FIELD(comm_ptr, reduce_tree_type),
                                                  &release_gather_info_ptr->reduce_tree,
-                                                 &topotree_fail[1], &errflag);
-                MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+                                                 &topotree_fail[1]);
+                MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
             } else {
                 /* Finalize was already called and MPIR_Process.hwloc_topology has been destroyed */
                 topotree_fail[0] = -1;
                 topotree_fail[1] = -1;
             }
             mpi_errno = MPIR_Allreduce_impl(MPI_IN_PLACE, topotree_fail, 2, MPI_INT,
-                                            MPI_MAX, comm_ptr, &errflag);
-            if (mpi_errno) {
-                /* for communication errors, just record the error but continue */
-                errflag =
-                    MPIX_ERR_PROC_FAILED ==
-                    MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                MPIR_ERR_SET(mpi_errno, errflag, "**fail");
-                MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-            }
+                                            MPI_MAX, comm_ptr, errflag);
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
         } else {
             topotree_fail[0] = -1;
             topotree_fail[1] = -1;
@@ -322,40 +337,42 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
             if (topotree_fail[0] == 1)
                 MPIR_Treealgo_tree_free(&release_gather_info_ptr->bcast_tree);
             mpi_errno =
-                MPIR_Treealgo_tree_create(rank, num_ranks, MPIDI_POSIX_Bcast_tree_type,
-                                          MPIR_CVAR_BCAST_INTRANODE_TREE_KVAL, 0,
+                MPIR_Treealgo_tree_create(rank, num_ranks,
+                                          RELEASE_GATHER_FIELD(comm_ptr, bcast_tree_type),
+                                          RELEASE_GATHER_FIELD(comm_ptr, bcast_tree_kval), 0,
                                           &release_gather_info_ptr->bcast_tree);
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
         }
 
         if (topotree_fail[1] != 0) {
             if (topotree_fail[1] == 1)
                 MPIR_Treealgo_tree_free(&release_gather_info_ptr->reduce_tree);
             mpi_errno =
-                MPIR_Treealgo_tree_create(rank, num_ranks, MPIDI_POSIX_Reduce_tree_type,
-                                          MPIR_CVAR_REDUCE_INTRANODE_TREE_KVAL, 0,
+                MPIR_Treealgo_tree_create(rank, num_ranks,
+                                          RELEASE_GATHER_FIELD(comm_ptr, reduce_tree_type),
+                                          RELEASE_GATHER_FIELD(comm_ptr, reduce_tree_kval), 0,
                                           &release_gather_info_ptr->reduce_tree);
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
+
         }
 
-        release_gather_info_ptr->gather_state = release_gather_info_ptr->release_state
-            = MPIR_CVAR_BCAST_INTRANODE_NUM_CELLS + MPIR_CVAR_REDUCE_INTRANODE_NUM_CELLS;
+        release_gather_info_ptr->gather_state = release_gather_info_ptr->release_state =
+            RELEASE_GATHER_FIELD(comm_ptr, bcast_num_cells) +
+            RELEASE_GATHER_FIELD(comm_ptr, reduce_num_cells);
 
         release_gather_info_ptr->bcast_buf_addr = NULL;
         release_gather_info_ptr->reduce_buf_addr = NULL;
         release_gather_info_ptr->child_reduce_buf_addr = NULL;
 
-        RELEASE_GATHER_FIELD(comm_ptr, flags_shm_size) = flags_shm_size;
-        /* update the shared counter */
-        if (rank == 0)
-            MPL_atomic_fetch_add_uint64(MPIDI_POSIX_shm_limit_counter, flags_shm_size);
         mpi_errno =
             MPIDU_shm_alloc(comm_ptr, flags_shm_size,
-                            (void **) &(release_gather_info_ptr->flags_addr), &mapfail_flag);
-        if (mapfail_flag) {
+                            (void **) &(release_gather_info_ptr->flags_addr));
+        if (mpi_errno) {
+            /* for communication errors, just record the error but continue */
             MPIR_ERR_ADD(mpi_errno_ret, MPIR_ERR_OTHER);
         }
-        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno_ret, errflag);
+
+        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno_ret, errflag, mpi_errno_ret);
         MPIR_ERR_CHECK(mpi_errno_ret);
 
         /* Calculate gather and release flag address and initialize to the gather and release states */
@@ -369,26 +386,23 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
                                         release_gather_info_ptr->release_state);
 
         /* Make sure all the flags are set before ranks start reading each other's flags from shm */
-        mpi_errno = MPIR_Barrier_impl(comm_ptr, &errflag);
-        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+        mpi_errno = MPIR_Barrier_impl(comm_ptr, errflag);
+        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
     }
 
     if (initialize_bcast_buf) {
-        RELEASE_GATHER_FIELD(comm_ptr, bcast_shm_size) =
-            MPIR_CVAR_BCAST_INTRANODE_BUFFER_TOTAL_SIZE;
         if (rank == 0)
             MPL_atomic_fetch_add_uint64(MPIDI_POSIX_shm_limit_counter,
                                         RELEASE_GATHER_FIELD(comm_ptr, bcast_shm_size));
 
         /* Allocate the shared memory for bcast buffer */
         mpi_errno =
-            MPIDU_shm_alloc(comm_ptr, MPIR_CVAR_BCAST_INTRANODE_BUFFER_TOTAL_SIZE,
-                            (void **) &(RELEASE_GATHER_FIELD(comm_ptr, bcast_buf_addr)),
-                            &mapfail_flag);
-        if (mapfail_flag) {
+            MPIDU_shm_alloc(comm_ptr, RELEASE_GATHER_FIELD(comm_ptr, bcast_shm_size),
+                            (void **) &(RELEASE_GATHER_FIELD(comm_ptr, bcast_buf_addr)));
+        if (mpi_errno) {
             MPIR_ERR_ADD(mpi_errno_ret, MPIR_ERR_OTHER);
         }
-        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
     }
 
     if (initialize_reduce_buf) {
@@ -397,32 +411,28 @@ int MPIDI_POSIX_mpi_release_gather_comm_init(MPIR_Comm * comm_ptr,
         RELEASE_GATHER_FIELD(comm_ptr, child_reduce_buf_addr) =
             MPL_malloc(num_ranks * sizeof(void *), MPL_MEM_COLL);
 
-        RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size) =
-            MPIR_CVAR_BCAST_INTRANODE_BUFFER_TOTAL_SIZE;
         if (rank == 0)
             MPL_atomic_fetch_add_uint64(MPIDI_POSIX_shm_limit_counter,
                                         RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size));
 
         mpi_errno =
-            MPIDU_shm_alloc(comm_ptr, num_ranks * MPIR_CVAR_REDUCE_INTRANODE_BUFFER_TOTAL_SIZE,
-                            (void **) &(RELEASE_GATHER_FIELD(comm_ptr, reduce_buf_addr)),
-                            &mapfail_flag);
-        if (mapfail_flag) {
+            MPIDU_shm_alloc(comm_ptr, num_ranks * RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size),
+                            (void **) &(RELEASE_GATHER_FIELD(comm_ptr, reduce_buf_addr)));
+        if (mpi_errno) {
+            /* for communication errors, just record the error but continue */
             MPIR_ERR_ADD(mpi_errno_ret, MPIR_ERR_OTHER);
         }
-        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno_ret, errflag);
+        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno_ret, errflag, mpi_errno_ret);
         MPIR_ERR_CHECK(mpi_errno_ret);
 
         /* Store address of each of the children's reduce buffer */
+        char *addr;
+        addr = RELEASE_GATHER_FIELD(comm_ptr, reduce_buf_addr);
         for (i = 0; i < RELEASE_GATHER_FIELD(comm_ptr, reduce_tree.num_children); i++) {
-            MPIR_ERR_CHKANDJUMP(!utarray_eltptr
-                                (RELEASE_GATHER_FIELD(comm_ptr, reduce_tree.children), i),
-                                mpi_errno, MPI_ERR_OTHER, "**nomem");
+            int child_rank =
+                *(int *) utarray_eltptr(RELEASE_GATHER_FIELD(comm_ptr, reduce_tree).children, i);
             RELEASE_GATHER_FIELD(comm_ptr, child_reduce_buf_addr[i]) =
-                (char *) RELEASE_GATHER_FIELD(comm_ptr,
-                                              reduce_buf_addr) +
-                ((*utarray_eltptr(RELEASE_GATHER_FIELD(comm_ptr, reduce_tree.children), i))
-                 * MPIR_CVAR_REDUCE_INTRANODE_BUFFER_TOTAL_SIZE);
+                addr + (child_rank * RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size));
         }
     }
 
@@ -456,7 +466,7 @@ int MPIDI_POSIX_mpi_release_gather_comm_free(MPIR_Comm * comm_ptr)
 
     /* destroy and detach shared memory used for flags */
     mpi_errno = MPIDU_shm_free(RELEASE_GATHER_FIELD(comm_ptr, flags_addr));
-    MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+    MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
 
     if (RELEASE_GATHER_FIELD(comm_ptr, bcast_buf_addr) != NULL) {
         if (comm_ptr->rank == 0)
@@ -464,7 +474,7 @@ int MPIDI_POSIX_mpi_release_gather_comm_free(MPIR_Comm * comm_ptr)
                                         RELEASE_GATHER_FIELD(comm_ptr, bcast_shm_size));
         /* destroy and detach shared memory used for bcast buffer */
         mpi_errno = MPIDU_shm_free(RELEASE_GATHER_FIELD(comm_ptr, bcast_buf_addr));
-        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
     }
 
     if (RELEASE_GATHER_FIELD(comm_ptr, reduce_buf_addr) != NULL) {
@@ -473,7 +483,7 @@ int MPIDI_POSIX_mpi_release_gather_comm_free(MPIR_Comm * comm_ptr)
                                         RELEASE_GATHER_FIELD(comm_ptr, reduce_shm_size));
         /* destroy and detach shared memory used for reduce buffers */
         mpi_errno = MPIDU_shm_free(RELEASE_GATHER_FIELD(comm_ptr, reduce_buf_addr));
-        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+        MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag, mpi_errno_ret);
 
         MPL_free(RELEASE_GATHER_FIELD(comm_ptr, child_reduce_buf_addr));
     }
