@@ -1,9 +1,8 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2003 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #include "mpi.h"
 #include <stdio.h>
 #include "mpitest.h"
@@ -30,7 +29,7 @@ int main(int argc, char **argv)
         displs[i] = (7 - i) * sizeof(long);
         oldtypes[i] = MPI_LONG;
     }
-    MPI_Type_struct(8, blocklens, displs, oldtypes, &ntype1);
+    MPI_Type_create_struct(8, blocklens, displs, oldtypes, &ntype1);
     MPI_Type_contiguous(65536, ntype1, &ntype2);
     MPI_Type_commit(&ntype2);
 
