@@ -20,7 +20,7 @@ static char MTEST_Descrip[] = "Test contig asynchronous I/O";
    reads them back. The file name is taken as a command-line argument,
    and the process rank is appended to it.*/
 
-static void handle_error(int errcode, char *str)
+static void handle_error(int errcode, const char *str)
 {
     char msg[MPI_MAX_ERROR_STRING];
     int resultlen;
